#!/bin/sh
######################################################################
# extractgps.sh

# Extract embedded GPS data (latitude, longitude, altitude) metadata
# from specified photo or video file, if available.
######################################################################
# Copyright 2022 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

extractgps.sh                    Copyright 2022 Theodore B. Ruegsegger

Examines the embedded metadata (EXIF in photos, similar but unnamed
format in videos) for GPS data (latitude, longitude, altitude) and, if
available, displays it as space-separated signed decimal degrees and
meters.

Usage:

    extractgps.sh [ -h ] file

Options:

    -h
        Display this help message listing options.

Example:
    extractgps.sh img_0376.mov

EOF
return  
}

######################################################################
Usage="Usage: extractgps.sh [ -h ] file"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

if [ $# -lt 1 ]
then
    echo $Usage
    exit
else
    File=$1
fi

# Yields 66.66666666 N 66.66666666 W (lat N lon W) if it works
GPSString=$(exiftool -m -c "%.8f" -p '$gpslatitude $gpslongitude'\
		     $File)

if [ "z${GPSString}z" != "z z" ]
then
    Latitude=$(echo $GPSString | cut -d " " -f1)
    LatHem=$(echo $GPSString | cut -d " " -f2)
    if [ "z$LatHem" = "zS" ]
    then
	Latitude=-$Latitude
    fi
    Longitude=$(echo $GPSString | cut -d " " -f3)
    LonHem=$(echo $GPSString | cut -d " " -f4)
    if [ "z$LonHem" = "zW" ]
    then
	Longitude=-$Longitude
    fi
    GPSAlt=$(exiftool -m -c "%.8f" -p '$gpsaltitude' $File\
		       2>/dev/null)
    if [ "z${GPSAlt}z" != "zz" ]
    then
	Altitude=$(echo $GPSAlt | cut -d " " -f1)
	AltRef=$(echo $GPSAlt | cut -d " " -f3)
	if [ "z$AltRef" = "zBelow" ]
	then
	    Altitude=-$Altitude
	fi
	echo "$Latitude $Longitude $Altitude"
    else
	echo "$Latitude $Longitude"
    fi
else
    # Yields +66.6666 -666.6666 +666.666 (lat lon alt) if it works
    GPSString=$(exiftool -m -c "%.8f" \
			 -p '$ComAppleQuicktimeLocationIso6709' \
			 $File \
		    | sed -e 's#\/##' -e 's#[+-]# &#g' -e 's#^ ##')
    if [ "z${GPSString}z" != "zz" ]
    then
	Latitude=$(echo $GPSString | cut -d " " -f1)
	Longitude=$(echo $GPSString | cut -d " " -f2)
	Altitude=$(echo $GPSString | cut -d " " -f3)
	echo "$Latitude $Longitude $Altitude"
    else
	# Yields +66.6666 -666.6666 (lat lon) if it works
	GPSString=$(exiftool -m -c "%.8f" -p '$location'  $File \
			| sed -e 's#\/##' -e 's#[+-]# &#g' \
			      -e 's#^ ##')
	if [ "z${GPSString}z" != "zz" ]
	then
	    Latitude=$(echo $GPSString | cut -d " " -f1)
	    Longitude=$(echo $GPSString | cut -d " " -f2)
	    echo "$Latitude $Longitude"
	else
	    echo "No GPS Info"
	fi
    fi
fi
