#!/bin/sh
######################################################################
# extractgps2db.sh

# Record GPS data (latitude, longitude, altitude) in database from
# photos and videos that include it in their metadata.
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

extractgps2db.sh                 Copyright 2021 Theodore B. Ruegsegger

For each photo or video that lacks GPS data (latitude, longitude,
altitude) in the database, extracts the values from the embedded
metadata (EXIF in photos), if available, and adds them to the
database. This can take hours for a large photo collection.

Usage:

    extractgps2db.sh [ -h ] database serviceroot

Options:

    -h
        Display this help message listing options.

# Example:
#     sh extractgps2db.sh trove /home/web/webroot-ssl/trove

EOF
return  
}

PhotosSansGPS()
{
    # Returns serial numbers of photos missing geo-data
$Query <<EOF
SELECT photosn FROM photo
WHERE (latitude IS NULL OR longitude IS NULL OR altitude IS NULL)
ORDER BY photosn
EOF
return
}

QueryPhoto()
{
    # Return "foldername/filename, {vidfilename|"none" for photosn $1
$Query <<EOF
SELECT foldername || '/' || filename ||
', ' || COALESCE(vidfilename, 'none')
FROM photo p LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)
, photoset s
WHERE photosn = $1 AND s.photosetsn = p.photosetsn;
EOF
return  
}

UpdatePhoto()
{
    # Updates photosn $1 with latitude $2, longitude $3, altitude $4
$Query <<EOF
UPDATE PHOTO SET latitude = $2, longitude = $3, altitude = $4
WHERE photosn = $1
EOF
return  
}

######################################################################
Usage="Usage: extractgps2db.sh database serviceroot [ dbuser ]"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
      \?) echo $Usage
	  exit 1
	  ;;
          ;;
    esac
done

if [ $# -lt 2 ]
then
    echo $Usage
    exit
else
    Database=$1
    FullPath=$2/fullsize/
    if [ $# -gt 2 ]
    then
	DbUser=$3
    else
	DbUser=$(echo $Database | sed -e 's/_.*$//')
    fi
fi
Query="psql -qt -d $Database -U $DbUser"

# Identify photos/videos missing geo-data
PhotoSNs=$(PhotosSansGPS)

StartTime=$(date)
PhoUpdated=0
PhoSkipped=0
for PhotoSN in $PhotoSNs
do
    NoFile=
    NoGPS=
    PhotoInfo=$(QueryPhoto $PhotoSN)
    PhotoPath=$(echo $PhotoInfo | cut -d, -f1)
    VidFileName=$(echo $PhotoInfo | cut -d" " -f2)
    if [ "z$VidFileName" != "znone" ]
    then
	# there's a video
	FolderName=$(echo $PhotoPath | cut -d/ -f1)
	FilePath=$FullPath/$FolderName/$VidFileName
	What="Video $FolderName/$VidFileName"
    else
	# only the photo
	FilePath=$FullPath/$PhotoPath
	What="Photo $PhotoPath"
    fi
    if [ -e $FilePath ]
    then
	GPSLatLon=$(exiftool -m -c "%.8f" -p \
			     '$gpslatitude $gpslongitude' \
			     $FilePath 2>/dev/null)
	if [ "z${GPSLatLon}z" != "zz" ]
	then
	    Latitude=$(echo $GPSLatLon | cut -d " " -f1)
	    LatHem=$(echo $GPSLatLon | cut -d " " -f2)
	    if [ $LatHem = "S" ]
	    then
		Latitude=-$Latitude
	    fi
	    Longitude=$(echo $GPSLatLon | cut -d " " -f3)
	    LonHem=$(echo $GPSLatLon | cut -d " " -f4)
	    if [ $LonHem = "W" ]
	    then
		Longitude=-$Longitude
	    fi
	    GPSAlt=$(exiftool -m -c "%.8f" -p '$gpsaltitude' \
			      $FilePath 2>/dev/null)
	    if [ "z${GPSAlt}z" != "zz" ]
	    then
		Altitude=$(echo $GPSAlt | cut -d " " -f1)
		AltRef=$(echo $GPSAlt | cut -d " " -f3)
		if [ "z$AltRef" = "zBelow" ]
		then
		    Altitude=-$Altitude
		fi
	    else
		Altitude="NULL"
	    fi
	else
	    NoGPS=True
	    PhoSkipped=$(( $PhoSkipped + 1))
	fi
    else
	NoFile=True
	NoGPS=True
	PhoSkipped=$(( $PhoSkipped + 1))
    fi
    if [ $NoFile ]
    then
	echo "$What: File not found"
    elif [ $NoGPS ]
    then
	echo "$What: No GPS info"
    else
	UpdatePhoto $PhotoSN $Latitude $Longitude $Altitude
	echo "$What: $Latitude, $Longitude, $Altitude"
	PhoUpdated=$(( $PhoUpdated + 1))
    fi    
done   
FinishTime=$(date)
echo Started \ $StartTime
echo Finished $FinishTime
echo Updated $PhoUpdated photos/videos.
echo Skipped $PhoSkipped photos/videos.
