#!/bin/sh
######################################################################
# filenames-case.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

filenames-case.sh                Copyright 2021 Theodore B. Ruegsegger

Converts filenames of all photos in a specified photoset to either
upper- or lowercase. This is useful when moving photos from one set to
another, to distinguish photos with the same filename by changing the
case of one set.

Usage:

    filenames-case.sh [ -h ] database foldername { UPPER | LOWER }

Where:

    case is UPPER or LOWER
	specifies the desired case after conversion

Options:

    -h
        Display a help message listing options.

Examples:

    filenames-case.sh trove "20190601-215330" UPPER

Must run as a user with sufficient sudo privilege to become the
webserver user "www-data", in order to write to the fullsize and
webpics subfolders.

EOF
    return  
}

FixFileNames()
{
    # Change case of all files in $1 per $2
    for File in $1
    do
	New=$(echo $File | tr $2)
	if [ $File != $New ]
	then
	    sudo -u www-data mv $File $New
	fi
    done
    return
}

ListFileNames()
{
    # Return all filenames in photoset $1
    $Query -c "SELECT filename FROM photo WHERE photosetsn = $1"
    return
}

######################################################################
Usage="Usage: $0 [ -h ] database foldername { UPPER | LOWER }"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

if [ $# -lt 3 ]
then
    echo $Usage
    exit
fi

if echo $0 | grep '/' >/dev/null
then
    ScriptDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    ScriptDir=$(pwd)
fi
Database=$1
FolderName=$2
Case=$(echo $3 | tr "a-z" "A-Z")

DbUser=$(echo $Database | sed -e 's/_.*$//')
Env=$(echo $Database | sed -ne 's/^.*_\([^_][^_]*$\)/\1/p')
Query="psql -qt -d $Database -U $DbUser"
PhotoSetSn=$($Query -c "SELECT photosetsn FROM photoset
		    WHERE foldername = '$FolderName'")

if ! [ $Case = "UPPER" -o $Case = "LOWER" ]
then
    echo $Usage
    exit
fi

# Check for duplicate (except for case) filenames in set:
CountAllNames=$(ListFileNames $PhotoSetSn | sort -f | wc -l)
CountUniqueNames=$(ListFileNames $PhotoSetSn | sort -fu | wc -l)
if [ $CountUniqueNames -ne $CountAllNames ]
then
    echo "*********************************************************"
    echo "Changing case would create duplicate filenames; aborting."
    echo "*********************************************************"
    exit
fi

StartDir=$(pwd)
FileNames=$(ListFileNames $PhotoSetSn | sort -f)

# Load definitions
. $ScriptDir/local/def_common.sh

ServiceRoot="/$ServiceName/"
if [ "z$Env" != "z" ]
then
    ServiceRoot="$ServiceRoot$Env/"
fi
ServiceRootPath=${WebServerSslDocumentRoot}$ServiceRoot

# Change case in database:
$Query <<EOF
UPDATE photo SET filename = $Case(filename)
WHERE photosetsn = $PhotoSetSn;
EOF

# Now change filenames in fullsize, webpics, thumbnails
if [ $Case = "UPPER" ]
then
    Swap="a-z A-Z"
else
    Swap="A-Z a-z"
fi
cd ${ServiceRootPath}fullsize/$FolderName
FixFileNames "$FileNames" "$Swap"
cd ${ServiceRootPath}webpics/$FolderName
FixFileNames "$FileNames" "$Swap"
cd ${ServiceRootPath}webpics/$FolderName/thumbnails
FixFileNames "$FileNames" "$Swap"
cd $StartDir
