#!/bin/sh
######################################################################
# fixexifrotation.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

fixexifrotation.sh           Copyright (C) 2021 Theodore B. Ruegsegger

The EXIF metadata in many images and most photographs includes an
Orientation tag that specifies whether and how much the image is
rotated, e.g.,

Orientation                     : Rotate 90 CW
Orientation                     : Horizontal (normal)

In theory, image-rendering software in general, and web browsers in
particular, would recognize this tag and display the image properly
oriented, eliminating the need to rotate the actual image by
transposing the pixels in the file.

Unfortunately, support for this feature isn't universal, and in fact
the tag is often erroneous (smart phones and cameras can't always
tell how they're positioned), so PhotoTrove rotates images the hard
way. The tools to do this don't automatically correct the Orientation
tag, i.e., set it to "Horizontal (normal)".

The right time to correct the Orientation tag is when the images are
first imported. Then we can see the images in the gallery as they are,
without well-intentioned rotations by the browser, select the ones
that need rotation, and accomplish it in a few seconds.

All that remains is the legacy photo collection. This script goes
through all image files and corrects any Orientation tag that needs
it. It should need to be run only once through the collection, though
it should do no harm to run it again.

Usage:

[Must run as a user with sufficient sudo privilege to become the
webserver user "www-data", in order to write to the image subfolders.]

    fixexifrotation.sh [ -h ] troveroot firstfolder lastfolder logfile

Where:

    troveroot

	Path to service root for the PhotoTrove collection.

    firstfolder, lastfolder
	These set the bounds for the operation, since it can take a
	long time for a large photo collection. Folders are processed
	in alphabetical order.

    logfile
	Path (not just a name) to a file, writeable by the user, for
	recording the names of completed folders. If the file exists,
	new names will be appended.

Options:

    -h
        Display this help message.

Examples:

    fixexifrotation.sh /home/web/webroot-ssl/trove 19860906-000000 \\
	20030321-131512 $HOME/foldersdone

	In the image folders (fullsize, webpics) under the service
	root /home/web/webroot-ssl/trove, scan images in all folders
	from "19860906-000000" through "20030321-131512" (inclusive,
	in alphabetical order) and change EXIF Orientation tags to
	"Horizontal (normal)" if they're set to anything else. After
	correcting each folder, append its name to the log file
	foldersdone in the user's home directory.

EOF
return
}

FixAllThree()
{
    # $1 is Folder $2 is File
    # Set Orientation tag to Horizontal for all three images of $1/$2
    sudo -u www-data \
	 exiftool -Orientation='Horizontal (normal)' \
	 -overwrite_original $TroveRoot/fullsize/$1/$2 \
	 $TroveRoot/webpics/$1/$2 $TroveRoot/webpics/$1/thumbnails/$2
    return
}

######################################################################
Usage="Usage: $0 [ -h ] troveroot firstfolder lastfolder logfile"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

if [ $# -ne 4 ]
then
    echo $Usage
    exit
fi
TroveRoot=$1
FirstFolder=$2
LastFolder=$3
LogFile=$4

if [ ! -d $TroveRoot/fullsize -o ! -d $TroveRoot/webpics ]
then
    echo Service root \"$TroveRoot\" lacks image folders.
fi

if [ ! -d $TroveRoot/fullsize/$FirstFolder ]
then
    echo Folder $FirstFolder not found.
    exit
fi

if [ ! -d $TroveRoot/fullsize/$LastFolder ]
then
    echo Folder $LastFolder not found.
    exit
fi

# Get sudo permission up-front
sudo -u www-data ls >/dev/null
touch $LogFile
date >> $LogFile
FileCount=0
FixCount=0

Folders=$(ls -1 $TroveRoot/webpics \
	      | sed -ne "/$FirstFolder/,/$LastFolder/p")

for Folder in $Folders
do
    echo Scanning folder $Folder
    Files=$(cd $TroveRoot/webpics/$Folder; ls *.*)
    for File in $Files
    do
	if (exiftool $TroveRoot/webpics/$Folder/$File \
		| grep Orientation \
		| grep -v 'Horizontal (normal)' >/dev/null)
	then
	    echo $File has rotated Orientation
	    FixAllThree $Folder $File
	    FixCount=$(( $FixCount + 1 ))
	else
	    echo $File ok
	fi
	FileCount=$(( $FileCount + 1 ))
    done
    echo $Folder >> $LogFile	
done
echo "$FileCount files scanned, $FixCount fixed." >> $LogFile
date >> $LogFile

