#!/bin/sh
######################################################################
# fotoupload.sh
######################################################################
# Copyright 2023 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

fotoupload.sh               Copyright      2023 Theodore B. Ruegsegger

Mounts any of various memory card types and copies all files to a new
subfolder (with a unique name based on the date and time) of
$UploadPath.

Usage:
    fotoupload.sh [-hs] [-l|-u] [-ex]

Example:
    fotoupload.sh -l

Options:
    -e
        Erase and unmount memory card after local upload, without
        prompting. Default action is to ask first.

    -h
        Display a help message listing options.

    -l
        Force filenames to lowercase.

    -s
	Display system and configuration requirements.

    -u
        Force filenames to uppercase.

    -x
        Purge superfluous files (e.g., thumbnails) before transfer.

EOF
return  
}

PrintReqts()
{
cat <<EOF

Requirements:

fotoupload.sh is adapted for memory cards that contain images in one
or more subfolders under a "dcim" or "DCIM" folder, which most digital
cameras create per the Digital Camera IMages specification:
http://en.wikipedia.org/wiki/Design_rule_for_Camera_File_system

Can handle CompactFlash/MicroDrive in PCMCIA adapter (appears as IDE
drive) as well as various cards in USB devices (appears as SCSI
drive).

The \$Devices variable lists devices to consider, currently:
$Devices
but the script ignores any devices with no corresponding mount point
under the /media/ folder; this avoids conflicts with other devices
like permanently-mounted SCSI disk drives.

Note that all mount points require an entry in /etc/fstab, for
example:

Device:		/dev/sdb1
Mount point:	/media/sdb1
/etc/fstab line:
/dev/sdb1	/media/sdb1	auto	user,noauto	0	0

EOF
return  
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

PurgeSuperfluousFiles()
{
    TargetFolder=$1
    echo "Purging superfluous files (e.g., thumbnails)."
    for Ext in $SuperfluousFileExtensions
    do
	rm $TargetFolder/*$Ext 2>/dev/null
    done
    Count=$(ls -1 $TargetFolder | wc -l)
    echo "$Count files remain."
    return
}

SetCaseLower()
{
    # Converts filenames in folder $1 to lowercase
    cd $1
    for File in *
    do
	New=$(echo $File | tr [:upper:] [:lower:])
	if [ $File != $New ]
	then
	    mv "$File" "$New"
	fi
    done
    echo Converted filenames to lowercase.
    cd ..
    return
}

SetCaseUpper()
{
    # Converts filenames in folder $1 to uppercase
    cd $1
    for File in *
    do
	New=$(echo $File | tr [:lower:] [:upper:])
	if [ $File != $New ]
	then
	    mv "$File" "$New"
	fi
    done
    echo Converted filenames to uppercase.
    cd ..
    return
}

WarnFailedLocalCopies()
{
    # Report failure to copy files from removable device $1 to
    # $UploadPath
    S="There was a problem copying files from one or more image"
    S="$S folders on removable device $1 to corresponding folders"
    S="$S under $UploadPath."
    echo $S
    echo Leaving all files on device $1, still mounted as /media/$1.
    echo "You'll need to fix this manually."
    return
}

WarnFailedLocalCopy()
{
    # Report failure to copy files from image folder $1 to target
    # folder $2
    echo '****************************************'
    echo '*           File Copy Failed!          *'
    echo '****************************************'
    echo
    echo There was a problem copying files from $1 to $2.
    S="Be sure to correct the problem before erasing removable"
    S="$S storage device $Device."
    echo $S
    return
}

######################################################################
UploadPath='/home/public/photos'
Devices="
hde1
mmcblk0p1
sda1
sdb1
sdc1
sdd1
sde1
"
SuperfluousFileExtensions="
thm
THM
"
Usage="Usage: fotoupload.sh [-hs] [-l|-u] [-e]"
######################################################################
Host=$(hostname)

# Parse command line options
while getopts "ehlsux" Option
do
    case $Option in
      e ) EraseAfterCopy=True
          ;;
      h ) PrintHelp
          exit
          ;;
      l ) MakeLowercase=True
          ;;
      s ) PrintReqts
          exit
          ;;
      u ) MakeUppercase=True
          ;;
      x ) PurgeSuperfluous=True
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

CaseChange=None
if [ "$MakeLowercase" = True -a "$MakeUppercase" = True ]
then
    echo "Lowercase (-l) or uppercase (-u) or neither; pick one!"
    echo "Can't continue."
    exit
elif [ "$MakeLowercase" = True ]
then
    CaseChange=Lower
elif [ "$MakeUppercase" = True ]
then
    CaseChange=Upper
fi

MountPoints=""
for Device in $Devices
do
    if [ -d /media/$Device ]
    then
	if grep /media/$Device /etc/fstab >/dev/null
	then
	    MountPoints="$MountPoints $Device"
	fi
    fi
done
Count=$(echo $MountPoints | wc -w)
if [ $Count -eq 0 ]
then
    echo None of these devices: $Devices
    echo "has both a mount point in the /media/ folder"
    echo "and an entry in the /etc/fstab file."
    echo "Can't continue."
    exit
else
    Devices=$MountPoints
fi

DevicesPresent=""
for Device in $Devices
do
    if [ -b /dev/$Device ]
    then
	DevicesPresent="$DevicesPresent $Device"
    fi
done
Count=$(echo $DevicesPresent | wc -w)
if [ $Count -eq 0 ]
then
    echo Found none of these devices: $Devices
    echo "Can't continue."
    exit
elif [ $Count -gt 1 ]
then
    echo Found more than one device: $DevicesPresent
    echo "Can't continue (yet; in future, I'll ask which you want)."
    exit
else
    Device=$(echo $DevicesPresent)
    if [ ! -d /media/$Device/dcim -a ! -d /media/$Device/DCIM ]
    then
	mount /media/$Device
    fi
    if [ ! -d /media/$Device/dcim -a ! -d /media/$Device/DCIM ]
    then
	echo "Couldn't find a dcim or DCIM folder."
	echo "Might have failed to mount the memory device"
	echo "Can't continue."
	exit
    else
	DcimFolder=$(ls /media/$Device | grep -i dcim)
    fi
fi

# If we got this far, then /dev/$Device is mounted as /media/$Device
# and contains a dcim/DCIM folder $DcimFolder
StartDir=$(pwd)
cd $UploadPath
# Exclude the canonmsc folder; don't seem to need those files:
ImageFolders=$(ls -d /media/$Device/$DcimFolder/* \
    | grep -vi 'canonmsc')

for ImageFolder in $ImageFolders
do
    # Load only non-empty folders
    if [ -d $ImageFolder -a \
	$(ls -1 $ImageFolder/ | sed -ne '1p')"ZZ" != "ZZ" ]
    then
	echo Found image folder $ImageFolder
	TargetFolder=$(date +%Y%m%d-%H%M%S)
	# Verify that folder doesn't already exist
	while [ -d "$TargetFolder" ]
	do
	    sleep 1
	    TargetFolder=$(date +%Y%m%d-%H%M%S)
	done
	echo Creating $TargetFolder
	mkdir $TargetFolder
	cp $ImageFolder/* $TargetFolder
	sync ; sync
	chmod 664 $TargetFolder/*
	ls -FAl $TargetFolder
	# check if folder contents match
	SrcContents=$(ls -l $ImageFolder | grep -v '^total ' \
	    | awk '{print $5,$NF}')
	TgtContents=$(ls -l $TargetFolder | grep -v '^total ' \
	    | awk '{print $5,$NF}')
	if [ "$SrcContents" = "$TgtContents" ]
	then
	    Count=$(ls -1 $TargetFolder | wc -l)
	    echo "$Count files uploaded to $Host."
	    if [ $PurgeSuperfluous ]
	    then
		PurgeSuperfluousFiles $TargetFolder
	    fi
	    case $CaseChange in
		"Lower") SetCaseLower $TargetFolder
			 ;;
		"Upper") SetCaseUpper $TargetFolder
			 ;;
	    esac
	else
	    FailedCopy="$FailedCopy $TargetFolder"
	    WarnFailedLocalCopy $ImageFolder $TargetFolder
	fi
	echo
    fi
done

# If any local copy failed, warn and abort
if [ $FailedCopy ]
then
    WarnFailedLocalCopies $Device
    cd $StartDir
    exit
fi

# Local copies all succeeded. Erase and unmount the storage device
# unless user wishes otherwise
if [ $EraseAfterCopy ]
then
    rm -r $ImageFolders
    umount /media/$Device
    echo You may safely remove removable device $Device.
else
    PromptYesNo "Erase and unmount removable device $Device"
    if [ $Response = "Y" ]
    then
	rm -r $ImageFolders
	umount /media/$Device
	echo You may safely remove removable device $Device.
    else
	echo Leaving all files on removable device $Device.
	echo "You'll need to clear it manually."
	PromptYesNo "Unmount $Device"
	if [ $Response = "Y" ]
	then
	    umount /media/$Device
	    echo You may safely remove removable device $Device.
	else
	    echo Leaving device mounted as /media/$Device
	fi
    fi
fi

exit
