/*********************************************************************
createbasictables.sql
____________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
____________________________________________________________

Creates basic database tables likely to be needed for a typical
web-based service:

    person		Known persons
    reguser		Users who can log in
    altauth		Challenge/response for alternate
			authentication
    privilege		Privileges that link tools with users
    nogrant		Privileges that apply to tools but not users
    user_priv		Privileges granted to users
    user_metapriv	Privilege-granting privileges granted to users
    tool		Tools (PHP scripts) available to run
    hiddentool		Tools that don't appear in the user's menu
    tool_priv		Privileges needed to run tools
    report		Reports (PHP scripts) available to run
    report_priv		Privileges needed to generate reports
    dbversion		Current release of database

Usage:
    psql -d database -U dbuser -f createbasictables.sql
or, more quietly:
    psql -qd database -U dbuser -f createbasictables.sql 2>&1 \
	| grep -v 'NOTICE:'

*********************************************************************/

/*********************************************************************
Following table definitions are output from pgModeler, with redundant
ALTER TABLE .. OWNER TO .. lines commented out.
*********************************************************************/
\i gapp.sql
/*********************************************************************
End of table definitions imported from DBDesigner
*********************************************************************/

GRANT SELECT, INSERT, UPDATE, DELETE ON
    altauth,
    dbversion,
    hiddentool,
    nogrant,
    person,
    privilege,
    reguser,
    report,
    report_priv,
    tool,
    tool_priv,
    user_metapriv,
    user_priv
TO "www-data";

GRANT SELECT, UPDATE ON
    person_personsn_seq,
    privilege_privilegesn_seq,
    reguser_regusersn_seq,
    report_reportsn_seq,
    tool_toolsn_seq
TO "www-data";

-- Now populate the initial users and privileges

-- "System Administrator (admin), initial password "I'm the BOFH"
INSERT INTO person (lastname, firstname)
VALUES ('Administrator', 'System');

INSERT INTO reguser (userid, personsn, passwd_hash, passwd_changed)
 SELECT 'admin', personsn, '$1$XfWODH3.$tHzA9ZL2r4Ye7qWgXqQhY/', NULL
  FROM person
  WHERE lastname = 'Administrator' AND firstname = 'System';

-- Privileges assignable to users and tools
INSERT INTO privilege (privname, description)
VALUES ('admin', 'Administer system and users');
INSERT INTO privilege (privname, description)
VALUES ('adminsvc', 'Administer service');
INSERT INTO privilege (privname, description)
VALUES ('webdev', 'Develop web services');
INSERT INTO privilege (privname, description)
VALUES ('mnguser', 'Manage Users');
INSERT INTO privilege (privname, description)
VALUES ('mngpriv', 'Manage privileges');
INSERT INTO privilege (privname, description)
VALUES ('mngtool', 'Manage tools');
INSERT INTO privilege (privname, description)
VALUES ('metameta', 'Grant/revoke metaprivileges');
INSERT INTO privilege (privname, description)
VALUES ('mngrpt', 'Manage reports');

INSERT INTO privilege (privname, description)
VALUES ('grant', 'Available to users with metaprivileges');
INSERT INTO privilege (privname, description)
VALUES ('anylogin', 'Available to any logged-in user');
INSERT INTO privilege (privname, description)
VALUES ('public', 'No privilege needed');
-- Privileges assignable to tools only, not users
INSERT INTO nogrant(privilegesn)
SELECT privilegesn FROM privilege
WHERE privname IN ('grant', 'anylogin', 'public');

INSERT INTO tool (name, dispord, description)
VALUES ('manageuser.php', 100, 'Manage User Accounts');
INSERT INTO tool (name, dispord, description)
VALUES ('report.php', 101, 'Generate Reports');
INSERT INTO tool (name, dispord, description)
VALUES ('managetool.php', 200, 'Manage Tools');
INSERT INTO tool (name, dispord, description)
VALUES ('managepriv.php', 201, 'Manage Privileges');
INSERT INTO tool (name, dispord, description)
VALUES ('grant.php', 202, 'Grant Privileges to Users');
INSERT INTO tool (name, dispord, description)
VALUES ('managereport.php', 203, 'Manage Reports');
INSERT INTO tool (name, dispord, description)
VALUES ('showfunctions.php', 300,
 'View all PHP function definitions');

INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'manageuser.php' AND privname = 'mnguser';
INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'report.php' AND privname = 'anylogin';
INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'managetool.php' AND privname = 'mngtool';
INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'managepriv.php' AND privname = 'mngpriv';
INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'grant.php' AND privname = 'grant';
INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'managereport.php' AND privname = 'mngrpt';
INSERT INTO tool_priv (toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'showfunctions.php' AND privname = 'webdev';

INSERT INTO user_priv (regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser, privilege
WHERE userid = 'admin' AND privname = 'mngpriv';
INSERT INTO user_priv (regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser, privilege
WHERE userid = 'admin' AND privname = 'mngtool';
INSERT INTO user_priv (regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser, privilege
WHERE userid = 'admin' AND privname = 'mnguser';
INSERT INTO user_priv (regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser, privilege
WHERE userid = 'admin' AND privname = 'mngrpt';

-- Ensure admin can grant all user-assignable privileges
INSERT INTO user_metapriv(regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser r, privilege p
WHERE userid = 'admin' AND NOT EXISTS
(SELECT * FROM nogrant WHERE privilegesn = p.privilegesn)
AND NOT EXISTS
(SELECT * FROM user_metapriv WHERE regusersn = r.regusersn
AND privilegesn = p.privilegesn);
-- Ensure admin can grant metaprivileges
INSERT INTO user_priv (regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser, privilege
WHERE userid = 'admin' AND privname = 'metameta';
