#!/bin/sh
######################################################################
# createrelease.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

**********************************************************************
createrelease.sh

Creates a date-stamped release branch in CVS.

Usage:

    createrelease.sh [ options ] module

Options:
    -h
	Display a help message listing options.
    -p projectroot
	Full path of the top of the project's working folder; default
	$HOME/public_html/projects/$Module

**********************************************************************
EOF
return
}

######################################################################
ProjectRoot=
# Parse command line options
while getopts "hp:" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      p ) ProjectRoot=$OPTARG
          ;;
    esac
done
shift $(( $OPTIND - 1 ))

if [ $# -ne 1 ]
then
    PrintHelp
    exit
else
    Module=$1
fi

# Check that project root exists
if [ ! $ProjectRoot ]
then
    ProjectRoot=$HOME/public_html/projects/$Module
    if [ ! -d $ProjectRoot ]
    then
	echo "Cannot find a project root folder at $ProjectRoot"
	echo "Use -p option to specify correct path."
	exit
    fi
else
    if [ ! -d $ProjectRoot ]
    then
	echo "Cannot find a project root folder at $ProjectRoot"
	exit
    fi
fi

# Check that project root contains a CVS folder
if [ ! -d $ProjectRoot/CVS ]
then
    echo "Cannot find a CVS folder at $ProjectRoot/CVS"
    exit
else
    CVSRoot=$(cat $ProjectRoot/CVS/Root)
fi

cd $ProjectRoot

# Create a datestamped tag, e.g, rel_20080430
ReleaseDate=$(date +%Y%m%d)
ReleaseTag="rel_$ReleaseDate"
echo $ReleaseDate $ReleaseTag

# Check whether the release tag already exists; if so, add suffix
TempFile=$(mktemp)
cvs -q log Version > $TempFile
Suffix=0

while (grep $ReleaseTag: $TempFile >/dev/null )
do
    echo "Branch $ReleaseTag already exists."
    Suffix=$(( $Suffix + 1 ))
    ReleaseTag="rel_${ReleaseDate}_$Suffix"
    echo "Trying $ReleaseTag as branch tag."
done

# Update Version file, replacing first line with new release tag
touch Version
sed -e '1d' Version > $TempFile
echo "$Module version $ReleaseTag" > Version
cat $TempFile >> Version
rm $TempFile

# Commit and create release branch
cvs -q commit -m "Ready for release $ReleaseTag"
cvs -q rtag -b $ReleaseTag $Module

echo Created $Module release branch $ReleaseTag
