#!/bin/sh
######################################################################
# dumpdb.sh
######################################################################
# Copyright 2019 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

dumpdb.sh

Checks that specified PostgreSQL database, owned by specified user,
exists. If so, makes a full dump of the specified database and saves
it to a designated dump directory. Optionally, copies (scp) the
dumpfile to a remote host's dump directory as a backup.

The dumpfile includes restore instructions as a comment at the head of
the file.

Usage:
    dumpdb.sh [ options ] database dbuser

Example:
    dumpdb.sh -r ted@pikachu:/home/dumps foto_tst foto

Options:

    -d dumpdir
        Specify the directory to dump the database (default
        /home/dumps).

    -h
        Display a help message listing options.

    -o
	Dump object identifiers (OIDs) as part of the data for every
	table. Use of OIDs for user-defined tables is now deprecated,
	so try to avoid using this option. In any case, it won't work
	unless postgresql.conf explicitly enables OIDs.

    -r remotepath
        After dumping database, copy the dumpfile via scp to
        remotepath.

    -z
	Compress the dump with gzip.

EOF
return  
}

Explain()
{
    cat <<EOF
-- File created with:
-- $DumpCmd
-- To restore:
-- $RestoreCmd
/*********************************************************************
                Note: Older (pre-8.x) PostgreSQL versions

Older versions of pg_dump may put a privileged command out of order in
the generated file. Specifically, a DROP PROCEDURAL LANGUAGE command
appears ahead of the "\connect - postgres" line instead of after it,
which causes an error upon restore.  The workaround is to insert a
"\connect - postgres" line ahead of the DROP PROCEDURAL LANGUAGE
command, using sed. The restore command becomes (exactly, including
the \ and the line break and the double \):

EOF
    if [ $Compress ]
    then
	cat <<EOF
zcat ${DumpFile}.gz | sed -e '/DROP PROCEDURAL LANGUAGE/i\\
EOF
    else
	cat <<EOF
cat $DumpFile | sed -e '/DROP PROCEDURAL LANGUAGE/i\\
EOF
    fi
    cat <<EOF
\\\\connect - postgres' | psql -d $Database -U $DbUser

This workaround is completely unnecessary for newer versions of
PostgreSQL and pg_dump, but does no harm.

*********************************************************************/
EOF
    return
}

######################################################################
Compress=
WithOIDs=""
DumpDir=/home/dumps

# Parse command line options
while getopts "d:hor:z" Option
do
    case $Option in
      d ) DumpDir=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      o ) WithOIDs="o"
	  ;;
      r ) RemotePath=$OPTARG
          ;;
      z ) Compress=True
          ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -lt 2 ]
then
    U="Usage: $0 [ -hoz ][ -d dumpdir ][ -r remotepath ]"
    U="$U database dbuser"
    echo $U
    exit
else
    Database=$1
    DbUser=$2
fi

DbTuple=$(psql -l -U postgres -t |grep "$Database *\| *$DbUser ")
if [ "z$DbTuple" = "z" ]
then
    echo "Can't find database '$Database' owned by user '$DbUser'"
    echo "Can't dump; aborting"
    exit
fi

DumpFile=$DumpDir/$Database`date +%Y%m%d%H%M`.dump
if [ $Compress ]
then
    DumpCmd="pg_dump -c${WithOIDs}U $DbUser $Database | gzip"
    DumpCmd="$DumpCmd > ${DumpFile}.gz"
    RestoreCmd="zcat ${DumpFile}.gz | psql -d $Database -U $DbUser"
    echo Dumping ${DumpFile}.gz
    (Explain ; pg_dump -c${WithOIDs}U $DbUser $Database) | gzip \
	> ${DumpFile}.gz
else
    DumpCmd="pg_dump -c${WithOIDs}U $DbUser $Database > $DumpFile"
    RestoreCmd="cat $DumpFile | psql -d $Database -U $DbUser"
    echo Dumping $DumpFile
    (Explain ; pg_dump -c${WithOIDs}U $DbUser $Database) > $DumpFile
fi

if [ $RemotePath ]
then
    if [ $Compress ]
    then
	File=${DumpFile}.gz
    else
	File=$DumpFile
    fi
    echo Copying $File to $RemotePath/
    scp $File $RemotePath/
fi
