#!/bin/sh
######################################################################
# dumpdbandpurgeold
######################################################################
# Copyright 2019 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

dumpdbandpurgeold.sh

Checks that specified PostgreSQL database, owned by specified user,
exists. If so, dumps the specified PostgreSQL database, optionally
compressing it with gzip, and saves it as file
<database><date-timestamp>.dump[.gz] in the specified dump
directory. Checks for older dumps and deletes all but the few (default
4) most recent.

The dumpfile includes restore instructions as a comment at the head of
the file.

Usage:
    dumpdbandpurgeold.sh [ options ] database dbuser

Example:
    dumpdbandpurgeold.sh -z foto_tst foto

Options:

    -d dumpdir
        Specify the directory to dump the database (default
        /home/dumps).

    -h
        Display a help message listing options.

    -n count
	Retain the count most recent dumps (default 4).

    -o
	Dump object identifiers (OIDs) as part of the data for every
	table. Use of OIDs for user-defined tables is now deprecated,
	so try to avoid using this option. In any case, it won't work
	unless postgresql.conf explicitly enables OIDs.

    -z
	Compress the dump with gzip.

EOF
return  
}

Explain()
{
    cat <<EOF
-- File created with:
-- $DumpCmd
-- To restore:
-- $RestoreCmd
/*********************************************************************
                Note: Older (pre-8.x) PostgreSQL versions

Older versions of pg_dump may put a privileged command out of order in
the generated file. Specifically, a DROP PROCEDURAL LANGUAGE command
appears ahead of the "\connect - postgres" line instead of after it,
which causes an error upon restore.  The workaround is to insert a
"\connect - postgres" line ahead of the DROP PROCEDURAL LANGUAGE
command, using sed. The restore command becomes (exactly, including
the \ and the line break and the double \):

EOF
    if [ $Compress ]
    then
	cat <<EOF
zcat ${DumpFile}.gz | sed -e '/DROP PROCEDURAL LANGUAGE/i\\
EOF
    else
	cat <<EOF
cat $DumpFile | sed -e '/DROP PROCEDURAL LANGUAGE/i\\
EOF
    fi
    cat <<EOF
\\\\connect - postgres' | psql -d $Database -U $DbUser

This workaround is completely unnecessary for newer versions of
PostgreSQL and pg_dump, but does no harm.

*********************************************************************/
EOF
    return
}

######################################################################
Compress=
WithOIDs=""
DumpDir=/home/dumps
Count=4

# Parse command line options
while getopts "d:hn:oz" Option
do
    case $Option in
      d ) DumpDir=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      n ) Count=$OPTARG
	  ;;
      o ) WithOIDs="o"
	  ;;
      z ) Compress=True
          ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -lt 2 ]
then
    U="Usage: $0 [ -hoz ][ -d dumpdir ][ -n count ] database dbuser"
    echo $U
    exit
elif [ $Count -lt 1 ]
then
    echo "Count must be at least 1 (keeping only the latest dump)."
    U="Usage: $0 [ -hoz ][ -d dumpdir ][ -n count ] database dbuser"
    echo $U
    exit
else
    Database=$1
    DbUser=$2
fi

DbTuple=$(psql -l -U postgres -t |grep "$Database *\| *$DbUser ")
if [ "z$DbTuple" = "z" ]
then
    echo "Can't find database '$Database' owned by user '$DbUser'"
    echo "Can't dump; aborting"
    exit
fi

# ensure members of same group can purge file later
umask 0002

DumpFile=$DumpDir/$Database`date +%Y%m%d%H%M`.dump
if [ $Compress ]
then
    DumpFile="${DumpFile}.gz"
    DumpCmd="pg_dump -c${WithOIDs}U $DbUser $Database | gzip"
    DumpCmd="$DumpCmd > $DumpFile"
    RestoreCmd="zcat $DumpFile | psql -d $Database -U $DbUser"
    (Explain ; pg_dump -c${WithOIDs}U $DbUser $Database) | gzip \
	> $DumpFile
else
    DumpCmd="pg_dump -c${WithOIDs}U $DbUser $Database > $DumpFile"
    RestoreCmd="cat $DumpFile | psql -d $Database -U $DbUser"
    (Explain ; pg_dump -c${WithOIDs}U $DbUser $Database) > $DumpFile
fi
if [ -e $DumpFile ]
then
    echo Dumped $DumpFile
else
    echo Error: Tried to dump $DumpFile but failed!
fi

FileCount=$(ls $DumpDir | grep ^${Database}'[0-9][0-9]*\.dump.*' \
    | wc -w)
if [ $FileCount -gt $Count ]
then
    DropCount=$(( $FileCount - $Count ))
    FilesToDrop=$(ls $DumpDir \
	| grep ^${Database}'[0-9][0-9]*\.dump.*' \
	| sort | sed -ne "1,${DropCount}p")
    cd $DumpDir
    rm $FilesToDrop
    echo Purged older dumps of database $Database
fi
