#~/bin/sh
######################################################################
# gksetup.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

gksetup.sh

Creates a web application development environment based on the GAPP
Kit, including a project development folder, a development database
and web tree, a test database and web tree, and a service-specific
PostgreSQL user.

Be prepared to provide the following information:

* Web server document root (be sure your user account can write to it)
* Module name, as the project will be known to CVS
* Service abbreviation, for databases and URL paths
* Service Title, displayed on web pages
* CVS root, typically user@server:path

Usage:
    gksetup.sh [ options ]

Options:
    -h
	Display a help message listing options.

EOF
return  
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptNonBlank ()
{
    # Continues prompting until user enters a nonblank response
    RawResponse=""
    while [ "z$RawResponse" = "z" ]
    do
	Prompt "$1"
    done
    return
}

######################################################################
# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
    esac
done

User=$(whoami)
Host=$(hostname -f)

PromptNonBlank "Web server document root?"
DocumentRoot=$RawResponse
if [ ! -w $DocumentRoot ]
then
    echo "Oops! User $User cannot write to $DocumentRoot."
    echo "Check that $DocumentRoot has appropriate owner, group and"
    echo "permissions, and that user $User is in appropriate group"
    echo "(e.g., webdev)."
    exit
fi

PromptNonBlank "Module name, as the project will be known to CVS?"
Module=$RawResponse
PromptNonBlank "Service abbreviation, for databases and URL paths?"
Abbr=$RawResponse
PromptNonBlank "Service Title, displayed on web pages?"
ServiceTitle=$RawResponse
PromptNonBlank "CVS root (typically user@server:path)?"
CvsRoot=$RawResponse

# Determine development environment label
Initial=$(grep $User /etc/passwd | cut -d: -f 5 | cut -c 1 \
    | tr [:upper:] [:lower:])
Prompt "Specify an environment [default dev${Initial}]:"
if [ "z$RawResponse" = "z" ]
then
	Env=dev${Initial}
else
	Env="$RawResponse"
fi

mkdir -p $DocumentRoot/$Abbr

while [ -d $DocumentRoot/$Abbr/$Env ]
do
    if [ "z$Env" = "z" ]
    then
	echo "You must enter something nonblank"
    else
	echo "Sorry, but $Env is taken."
    fi
    echo "Try something else, perhaps dev followed by your initials."
    Prompt "Specify an environment:"
    Env="$RawResponse"
done

# Fetch and unpack GAPP Kit
TempDir=$(mktemp -d)
cd $TempDir
echo Fetching latest GAPP Kit
wget http://www.tux.org/~tbr/gappkit/gappkit-latest.tgz
echo Unpacking GAPP Kit
tar xzf gappkit-latest.tgz
cd gappkit

# Trim stuff we won't be needing soon, create empty release notes and
# to-do list
mkdir gknotes
mv examples.html manual.html README ReleaseNotes gknotes/
touch web/releasenotes.html
echo "* Document requirements.
* Build data model.
* Define schema in scripts/local/createapptables.sql.
* Build and demonstrate prototypes.
* Document design.
* Write test scripts.
* Code and test.
* Demo or die"\! > todo-$User.txt

# Tailor environment definitions in web/local/def_env.php
TempFile=$(tempfile)
sed -e "
s/\(define (\"DEBUG\",\).*$/\1 True);/
s/WhizBang/$ServiceTitle/
s/myservice/$Abbr/
s/My Web-Based Service//
s/dev/$Env/
s/\(SessionTimeout\",\).*$/\1 20);	\/\/ minutes/
s/Development/${User}'s Development/
" web/local/def_env.php > $TempFile
mv $TempFile web/local/def_env.php

# Import to CVS
echo "Importing to CVS."
echo "You may need to enter a password for $CvsRoot"
cvs -d $CvsRoot -Q import \
    -m "Creation, using GAPP Kit as template" $Module $User initial

echo "Building working copy."
echo "You may need to enter a password for $CvsRoot"
# Check out working copy from CVS into projects folder
cd $HOME/public_html/projects
cvs -d $CvsRoot -Q co $Module
rm -r $TempDir

cd $HOME/public_html/projects/$Module/

# Create document root subtree 
mkdir -p $DocumentRoot/$Abbr/$Env
# Move web folder contents to document root subtree
echo "Moving web folder contents to $DocumentRoot/$Abbr/$Env/"
cd $HOME/public_html/projects/$Module/
mv web/* $DocumentRoot/$Abbr/$Env/
# Replace web folder with a link to document root subtree
rmdir web
ln -s $DocumentRoot/$Abbr/$Env web
if [ -x "$(which symlinks)" ]
then
    symlinks -c ./ > /dev/null
    symlinks -cs ./ > /dev/null
else
    echo "Symbolic link \"web\" is absolute; symlinks tool not found."
    echo "Consider installing symlinks, then executing:"
    echo "    cd $HOME/public_html/projects/$Module/"
    echo "    symlinks -c ./"
    echo "    symlinks -cs ./"
    echo "Otherwise, convert  \"web\" manually to a relative link."
    echo "This is optional, but good practice."
fi

echo "Creating separate web tree $DocumentRoot/$Abbr/tst/"
mkdir -p $DocumentRoot/$Abbr/tst
cp -R $DocumentRoot/$Abbr/$Env/* $DocumentRoot/$Abbr/tst/

DevDatabase=${Abbr}_$Env
TstDatabase=${Abbr}_tst
DbUser=$Abbr

# Create personal development and test databases [ may get errors
# saying users "$DbUser" or "www-data" already exist ]
echo "Creating databases $DevDatabase and $TstDatabase."
sh scripts/gk/createdatabases.sh -ts $Env -w "www-data" $Abbr
sh scripts/gk/createbasictables.sh $DevDatabase
sh scripts/gk/createbasictables.sh $TstDatabase

echo " "
echo "Your development environment is set up."
echo "To view default operation, browse to:"
echo "https://$Host/$Abbr/$Env/"
echo "Now comes the work:"
echo "  * Document requirements."
echo "  * Build data model."
echo "  * Define schema in scripts/local/createapptables.sql."
echo "  * Build and demonstrate prototypes."
echo "  * Document design."
echo "  * Write test scripts."
echo "  * Code and test."
echo "  * Demo or die"\!
