#!/bin/sh
######################################################################
# makeslides.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
more <<EOF

makeslides.sh

Parses a formatted markup file into a set of slides contained in a
folder within the current directory. Each slide contains all the HTML
from a level 1 header <h1> up to but not including the next level 1
header. Also creates an outline slide.

Usage:
    makeslides.sh [ options ] inputfile
    makeslides.sh -h

Example:

    The input file is just an HTML file. Each slide begins with an
    <h1> tag and may contain any other HTML tags, for example:

    <html>
    <body>
    <h1>First Slide Title</h1>
    <p>Some text on first slide</p>

    <h1>Second Slide Title</h1>
    <ul>
      <li>A bullet</li>
      <li>Another bullet</li>
    </ul>

    ...

    <h1>Last Slide Title</h1>
    <p>Some text</p>

    </body>
    </html>

    To make slides from this file:
    makeslides.sh inputfile.html
Options:

    -f foldername
	Specify a name for the folder containing the slides. By
	default, the folder is the basename of inputfile with no
	extension

    -h
	Display a help message listing options.

    -o outline-css-file
	Specify a css file for outline instead of the generated
	default

    -p
	Give generated files the extension .phtml, to enable the
	webserver to process embedded PHP code. Default is .html

    -s slides-css-file
	Specify a css file for slides instead of the generated default

EOF
return  
}

ExtractHeaderTag()
{
    echo $* | sed -e 's/^\(<[Hh]1[^>]*>\).*$/\1/'
    return
}

ExtractTitle()
{
    T=$(echo $* | sed -e 's/^<[Hh]1[^>]*>\(.*\)<\/[Hh]1>/\1/')
    if (echo $T | grep -i '<h1' >/dev/null)
    then
	# Title spans multiple lines; get next line and reparse
	read Z2
	echo $(ExtractTitle "$T $Z2")
    else
	echo $T
    fi
    return
}

WriteOutlineEnd()
{
    cat <<EOF
</ol>
<hr />
<table cellpadding=3 cellspacing=3 border=0 width="100%">
<tr>
<td align="left">&nbsp;</td>
<td align=right><a href="./">Start</a></td>
</tr>
</table>
</body>
</html>
EOF
    return
}

WriteOutlineItem()
{
    Link=$1
    Title=$2
    echo '<li><a href="'${Link}'">'${Title}'</a></li>'
    return
}

WriteOutlineStart()
{
    Title=$*
    cat <<EOF
<html>
<head>
  <link type="text/css" rel="stylesheet" href="outline.css" />
  <title>$Title</title>
</head>
<body>
<h1>$Title</h1>
<h2>Outline</h2>
<hr />
<ol>
EOF
}

WriteSlideEnd()
{
    PrevSlide=$1
    NextSlide=$2
    cat <<EOF
<hr />
<table cellpadding=3 cellspacing=3 border=0 width="100%">
<tr>
<td align="left"><a href="$PrevSlide">Back</a> \
<a href="outline.$Extension">Up</a> <a href="$NextSlide">Next</a></td>
<td align=right><a href="./">Start</a></td>
</tr>
</table>
</body>
</html>
EOF
    return
}

WriteSlideStart()
{
    Title=$*
    cat <<EOF
<html>
<head>
  <link type="text/css" rel="stylesheet" href="slides.css" />
  <title>$Title</title>
</head>
<body>
EOF
    return
}

######################################################################
Extension=html
# Parse command line options
while getopts "f:ho:ps:" Option
do
    case $Option in
      f ) FolderName=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      o ) OutlineCss=$OPTARG
          ;;
      p ) Extension=phtml
          ;;
      s ) SlideCss=$OPTARG
          ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -ne 1 ]
then
    echo "Usage: makeslides.sh [ options ] inputfile"
    exit
else
    InFile=$1
fi
if [ $FolderName ]
then
    Folder=$FolderName
else
    Folder=$(basename $InFile | sed  -e 's/\..*$//')
fi

mkdir -p $Folder

if [ $OutlineCss ]
then
    cp $OutlineCss $Folder/outline.css
else
    cat >$Folder/outline.css <<EOF
H1 {color: #663399; font-size: 14pt; text-align: left;
    font-family: verdana,helvetica,sans-serif;}
H2 {color: #666699; font-size: 12pt;}
H3 {color: #669999; font-size: 10pt;}
P {color: #FFFFFF}
P.hl {color: #669999; font-size: 10pt; font-weight: bold;}
BODY {background: #FFFFFF; font-family: verdana,helvetica,sans-serif;
      font-size: 10pt;}
EOF
fi

if [ $SlideCss ]
then
    cp $SlideCss $Folder/slides.css
else
    cat >$Folder/slides.css <<EOF
H1 {color: #663399; font-size: 30pt; text-align: left;
    font-family: verdana,helvetica,sans-serif;}
H2 {color: #666699; font-size: 22pt;}
H3 {color: #669999}
zP {color: #FFFFFF}
BODY {background: #FFFFFF; font-family: verdana,helvetica,sans-serif;
      font-size: 22pt;}
.parabox {border: solid #663399; width: 66%; margin: auto;
      padding: 1ex}
EOF
fi

PrevSlide=outline.$Extension
SlideNum=1
SlideFile=index.$Extension
NextSlideNum=$(( $SlideNum + 1 ))
NextSlide=${NextSlideNum}.$Extension
LastSlideNum=$(
    sed -ne '/<body/,/<\/body/p' $InFile \
	| sed -e 1d | sed -e \$d \
	| while read Z
	do
	    if (echo $Z | grep -i '<h1' >/dev/null)
	    then
		if [ $SlidesStarted ]
		then
		    WriteSlideEnd $PrevSlide $NextSlide \
			>> ${Folder}/$SlideFile
		    PrevSlide=$SlideFile
		    SlideNum=$NextSlideNum
		    SlideFile=${SlideNum}.$Extension
		    NextSlideNum=$(( $SlideNum + 1 ))
		    NextSlide=${NextSlideNum}.$Extension
		else
		    WriteOutlineStart "$Title" \
			> ${Folder}/outline.$Extension
		    SlidesStarted=True
		fi
		H1=$(ExtractHeaderTag "$Z")
		Title=$(ExtractTitle "$Z")
		WriteOutlineItem $SlideFile "$Title" \
		    >> ${Folder}/outline.$Extension
		WriteSlideStart "$Title" > ${Folder}/$SlideFile
		echo "$H1$Title</h1>" >> ${Folder}/$SlideFile
		echo '<hr />' >> ${Folder}/$SlideFile
	    else
		echo "$Z" >> ${Folder}/$SlideFile
	    fi
	    echo $SlideNum
	done | sed -ne \$p)
LastSlide=${LastSlideNum}.$Extension
if [ -e ${Folder}/$LastSlide ]
then
    PrevSlide=$(( $LastSlideNum - 1 )).$Extension
    NextSlide=outline.$Extension
    WriteSlideEnd $PrevSlide $NextSlide >> ${Folder}/$LastSlide
fi
WriteOutlineEnd >> ${Folder}/outline.$Extension
if [ "$Extension" != "html" ]
then
    rm -f ${Folder}/index.html
    ln -s index.$Extension ${Folder}/index.html
fi