#!/bin/sh
######################################################################
# ifotoload.sh
######################################################################
# Copyright 2022 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

ifotoload.sh    Copyright                  2022 Theodore B. Ruegsegger

Given a ZIP file of photos, generated by the iPhone "File Transfer"
app by Evgenii Shcherbinin and downloaded to the current folder with a
file name matching "FilesYYYY-MM-DD_hh-mm-ss.zip":

1. Unzips it to a new image folder (with a unique name based on the
   timestamp in the ZIP file name) under $UploadPath.

2. Upon successful unzip, deletes the ZIP file.

Usage:
    ifotoload.sh [-h] [-l|-u]

Example:
    ifotoload.sh -l

Options:
    -h
        Display a help message listing options.

    -l
        Force filenames to lowercase.

    -u
        Force filenames to uppercase.

EOF
return  
}

FoundInUploadPath()
{
    # Returns True if folder $1 exists in $UploadPath,
    # else False
    if [ -d $UploadPath/$1 ]
    then
	echo True
    else
	echo False
    fi
    return
}

NewUniqueTimestamp()
{
    # Returns a timestamp string that isn't already in $UploadPath
    Timestamp=$(date +%Y%m%d-%H%M%S)
    # Check whether folder $Timestamp already exists in $UploadPath
    ExistsInUploadPath=$(FoundInUploadPath $Timestamp)
    # Increment Timestamp until new and unique
    while [ $ExistsInUploadPath = "True" ]
    do
	sleep 1
	Timestamp=$(date +%Y%m%d-%H%M%S)
	ExistsInUploadPath=$(FoundInUploadPath $Timestamp)
    done
    echo $Timestamp
    return
}

SetCaseLower()
{
    # Converts filenames in folder $1 to lowercase
    WD=$(pwd)
    cd $1
    for File in *
    do
	New=$(echo $File | tr [:upper:] [:lower:])
	if [ $File != $New ]
	then
	    mv "$File" "$New"
	fi
    done
    echo Converted filenames to lowercase.
    cd $WD
    return
}

SetCaseUpper()
{
    # Converts filenames in folder $1 to uppercase
    WD=$(pwd)
    for File in *
    do
	New=$(echo $File | tr [:lower:] [:upper:])
	if [ $File != $New ]
	then
	    mv "$File" "$New"
	fi
    done
    echo Converted filenames to uppercase.
    cd $WD
    return
}

######################################################################
UploadPath='/home/public/photos'
Usage="ifotoload.sh [-h] [-l|-u]"
######################################################################
LocalHost=$(hostname)

# Parse command line options
while getopts "hlu" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      l ) MakeLowercase=True
          ;;
      u ) MakeUppercase=True
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

CaseChange=None
if [ "$MakeLowercase" = True -a "$MakeUppercase" = True ]
then
    echo "Lowercase (-l) or uppercase (-u) or neither; pick one!"
    echo "Can't continue."
    exit
elif [ "$MakeLowercase" = True ]
then
    CaseChange=Lower
elif [ "$MakeUppercase" = True ]
then
    CaseChange=Upper
fi

######################################################################

# Locate ZIP file with pattern like Files2019-09-02_13-07-24.zip
D="[0-9]"
ZipFile=$(ls Files$D$D$D$D-$D$D-$D$D\_$D$D-$D$D-$D$D.zip 2>/dev/null)
FileCount=$(echo $ZipFile | wc -w)
if [ $FileCount -gt 1 ]
then
    echo "Too many ZIP files. Aborting."
    exit
elif [ $FileCount -ne 1 ]
then
    echo "No file found matching \"FilesYYYY-MM-DD_hh-mm-ss.zip\""
    echo "Aborting."
    exit
else
    echo "Found $ZipFile, continuing."	
fi

# Use timestamp from ZIP file name if possible
TargetFolder=$(echo $ZipFile | sed -e 's/Files//' | sed -e 's/-//g'\
		   | sed -e 's/_/-/' | sed -e 's/.zip//')

# If $TargetFolder exists under $UploadPath (perhaps from a previous
# trial), rename it as a unique timestamp

ExistsInUploadPath=$(FoundInUploadPath $TargetFolder)

if [ $ExistsInUploadPath = "True" ]
then
    TargetFolder=$(NewUniqueTimestamp)
fi

# Unzip archive to target folder, exit on failure
if ! (unzip -q $ZipFile -d $UploadPath/$TargetFolder)
then
    echo "Aborting."
    exit
fi    

Count=$(ls -1 $UploadPath/$TargetFolder | wc -l)
echo "$Count files extracted to $UploadPath/$TargetFolder."

case $CaseChange in
    "Lower") SetCaseLower $UploadPath/$TargetFolder
	     ;;
    "Upper") SetCaseUpper $UploadPath/$TargetFolder
	     ;;
esac

echo "File extraction succeeded. Erasing ZIP file."
rm -r $ZipFile 

