#!/bin/sh
######################################################################
# ifotoxfer.sh
######################################################################
# Copyright 2022 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

ifotoxfer.sh    Copyright                  2022 Theodore B. Ruegsegger

Given a ZIP file of photos, generated by the iPhone "File Transfer"
app by Evgenii Shcherbinin and downloaded to the current folder with a
file name matching "FilesYYYY-MM-DD_hh-mm-ss.zip":

1. Unzips it to a new image folder (with a unique name based on the
   timestamp in the ZIP file name);

2. Copies the image folder to the specified upload folder on a remote
   host;

3. Upon successful file transfer, deletes the local image folder and
   the ZIP file.

Usage:
    ifotoxfer.sh [-h] [-l|-u] [-d uploadpath] [user@] remotehost"

Where:

    remotehost is the IP address or resolvable name of the PhotoTrove
    server

    user is an account on remotehost with write access to the upload
    folder (defaults to same name as current user on local host)

    uploadpath is the full path to the upload folder on the remote
    host where the new image folder will be saved (defaults to
    /home/public/photos/)

Example:
    ifotoxfer.sh -l ted@thyme

Options:
    -d uploadpath
	Explicitly specify upload folder on remote host, overriding
	default /home/public/photos/

    -h
        Display a help message listing options.

    -l
        Force filenames to lowercase.

    -u
        Force filenames to uppercase.

Note:
    Saves a temporary copy of images in current working directory, so
    local filesystem must have adequate space for the uploaded images.

EOF
return  
}

FoundOnRemote()
{
    # Returns True if folder $1 exists in $UploadPath on $RemoteHost,
    # else False
    ssh $RemoteHost \
	"if [ -d $UploadPath/$1 ]
	 then echo True; else echo False; fi"
    return
}

NewUniqueTimestamp()
{
    # Returns a timestamp string that isn't already on remote host or
    # a folder in current directory
    Timestamp=$(date +%Y%m%d-%H%M%S)
    # Check whether folder $Timestamp already exists on remote host
    ExistsOnRemote=$(FoundOnRemote $Timestamp)
    # Increment Timestamp until new and unique
    while [ -d $Timestamp -o $ExistsOnRemote = "True" ]
    do
	sleep 1
	Timestamp=$(date +%Y%m%d-%H%M%S)
	ExistsOnRemote=$(FoundOnRemote $Timestamp)
    done
    echo $Timestamp
    return
}

SetCaseLower()
{
    # Converts filenames in folder $1 to lowercase
    cd $1
    for File in *
    do
	New=$(echo $File | tr [:upper:] [:lower:])
	if [ $File != $New ]
	then
	    mv "$File" "$New"
	fi
    done
    echo Converted filenames to lowercase.
    cd ..
    return
}

SetCaseUpper()
{
    # Converts filenames in folder $1 to uppercase
    cd $1
    for File in *
    do
	New=$(echo $File | tr [:lower:] [:upper:])
	if [ $File != $New ]
	then
	    mv "$File" "$New"
	fi
    done
    echo Converted filenames to uppercase.
    cd ..
    return
}

######################################################################
UploadPath='/home/public/photos'
Usage="Usage: ifotoxfer.sh [-h] [-l|-u] [-d uploadpath] [user@]"
Usage="$Usage remotehost"
######################################################################
LocalHost=$(hostname)

# Parse command line options
while getopts "d:hlu" Option
do
    case $Option in
      d ) UploadPath=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      l ) MakeLowercase=True
          ;;
      u ) MakeUppercase=True
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

CaseChange=None
if [ "$MakeLowercase" = True -a "$MakeUppercase" = True ]
then
    echo "Lowercase (-l) or uppercase (-u) or neither; pick one!"
    echo "Can't continue."
    exit
elif [ "$MakeLowercase" = True ]
then
    CaseChange=Lower
elif [ "$MakeUppercase" = True ]
then
    CaseChange=Upper
fi

shift $(( $OPTIND - 1 ))

if [ $# -lt 1 ]
then
    echo $Usage
    exit
fi
RemoteHost=$1

if ! (ping -c 1 -w 1 $RemoteHost >/dev/null)
then
    echo Remote host $RemoteHost is unreachable.
    echo "Can't continue."
    exit
fi

RemotePath=$RemoteHost:$UploadPath

######################################################################

# Locate ZIP file with pattern like Files2019-09-02_13-07-24.zip
D="[0-9]"
ZipFile=$(ls Files$D$D$D$D-$D$D-$D$D\_$D$D-$D$D-$D$D.zip 2>/dev/null)
FileCount=$(echo $ZipFile | wc -w)
if [ $FileCount -gt 1 ]
then
    echo "Too many ZIP files. Aborting."
    exit
elif [ $FileCount -ne 1 ]
then
    echo "No file found matching \"FilesYYYY-MM-DD_hh-mm-ss.zip\""
    echo "Aborting."
    exit
else
    echo "Found $ZipFile, continuing."	
fi

# Use timestamp from ZIP file name if possible
TargetFolder=$(echo $ZipFile | sed -e 's/Files//' | sed -e 's/-//g'\
		   | sed -e 's/_/-/' | sed -e 's/.zip//')

# If $TargetFolder exists locally or on remote host (perhaps from a
# previous trial), rename it as a unique timestamp

ExistsOnRemote=$(FoundOnRemote $TargetFolder)

if [ -d $TargetFolder -o $ExistsOnRemote = "True" ]
then
    TargetFolder=$(NewUniqueTimestamp)
fi

# Unzip archive to target folder, exit on failure
if ! (unzip -q $ZipFile -d $TargetFolder)
then
    echo "Aborting."
    exit
fi    

Count=$(ls -1 $TargetFolder | wc -l)
echo "$Count files extracted to local host $LocalHost."

case $CaseChange in
    "Lower") SetCaseLower $TargetFolder
	     ;;
    "Upper") SetCaseUpper $TargetFolder
	     ;;
esac

if ! (rsync -rv $TargetFolder $RemotePath)
then
    echo "File transfer to $RemotePath failed. Aborting."
    exit
fi

echo "Transfer succeeded. Erasing local files."
rm -r $ZipFile $TargetFolder

