#!/bin/sh
######################################################################
# integrate.sh

# Imports one or more photo sets from one PhotoTrove database to
# another, for example, from a traveling laptop to production.
######################################################################
# Copyright 2022 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

integrate.sh                     Copyright 2022 Theodore B. Ruegsegger

Imports one or more photo sets from one PhotoTrove database to
another, for example, from a traveling laptop to production:

  * Updates the target database to match the entries for the imported
    photos
  * Adds all new photosets, photos, videos, albums, locations and tags
  * Doesn't add new users or change passwords or any other information
    not directly related to photo management

Notes:

 1. This script handles only the database reconciliation. The physical
    transfer of the image files must be accomplished manually, as
    described in the instructions.

 2. Can't merge conflicts. Assumes target database hasn't been
    modified since traveling database was derived from it.

Instructions:

 1. On the source host (the one with the new photos), collect all new
    images as zipfiles:

    cd <serviceroot>/fullsize
    zip -r /home/public/photos/fullsize.zip *
    cd <serviceroot>/webpics
    zip -r /home/public/photos/webpics.zip *

 2. Use dumpdb.sh or dumpdbandpurgeold.sh to make a dumpfile of the
    source database.

 3. Copy the zipfiles and the dumpfile to the target host:

    scp /home/public/photos/fullsize.zip \
	/home/public/photos/webpics.zip \
	<targethost>:/home/public/photos/
    scp /home/dumps/<dumpfile> <targethost>:/home/dumps/

 4. On the target host, unzip the new images into the file tree. Do
    this as root (or use sudo) because the owner is the webserver user
    www-data:

    su
    cd <serviceroot>/fullsize
    unzip /home/public/photos/fullsize.zip
    cd <serviceroot>/webpics
    unzip /home/public/photos/webpics.zip
    exit

 5. Run this script on the target host 

    integrate.sh [ -h ] targetdb dumpfile

Where:

    targetdb
	is the target database to be updated.

    dumpfile
	is the path of a dumpfile of the source database containing
	the new information to be integrated). Gzipped dumps work.

Options:

    -h
	Display a help message listing options.

 6. Optionally, copy any uploaded camera folders in
    home/public/photos/ to the target host for archival purposes.

 7. The zipfiles fullsize.zip and webpics.zip, once transferred and
    unzipped, are no longer needed. Delete them on both hosts to
    reclaim the space.

EOF
return  
}


AddAlbumPages()
{
    Q="INSERT INTO albumpage (albumsn, page, heading, titlepicord,"
    Q="$Q titlepiccaption, pagetext)"
    Q="$Q SELECT a.albumsn, page, heading, titlepicord,"
    Q="$Q titlepiccaption, pagetext"
    Q="$Q FROM zzalbumpage z, zzalbum za, album a"
    Q="$Q WHERE za.albumsn = z.albumsn AND a.name = za.name"
    $QueryTgt -c "$Q"
    return
}

AddAlbumPhotos()
{
    Q="INSERT INTO albumphoto"
    Q="$Q (albumpagesn, photosn, ordinal, caption)"
    Q="$Q SELECT pg.albumpagesn, p.photosn, ordinal, caption"
    Q="$Q FROM zzalbumphoto z, zzalbumpage zpg, zzalbum za,"
    Q="$Q album a, albumpage pg,"
    Q="$Q zzphoto zp, zzphotoset zs, photoset s, photo p"
    Q="$Q WHERE zpg.albumpagesn = z.albumpagesn"
    Q="$Q AND za.albumsn = zpg.albumsn AND a.name = za.name"
    Q="$Q AND pg.albumsn = a.albumsn AND pg.page = zpg.page"
    Q="$Q AND zp.photosn = z.photosn"
    Q="$Q AND zs.photosetsn = zp.photosetsn"
    Q="$Q AND s.foldername = zs.foldername"
    Q="$Q AND p.photosetsn = s.photosetsn"
    Q="$Q AND p.filename = zp.filename"
    $QueryTgt -c "$Q"
    return
}

AddAlbums()
{
    Q="INSERT INTO album (name, title, pubdate, bgcolor, textcolor,"
    Q="$Q vidbgcolor, author, url, notes)"
    Q="$Q SELECT name, title, pubdate, bgcolor, textcolor,"
    Q="$Q vidbgcolor, author, url, notes"
    Q="$Q FROM zzalbum z WHERE NOT EXISTS"
    Q="$Q (SELECT 1 FROM album WHERE name = z.name)"
    $QueryTgt -c "$Q"
    return
}

AddArchivedAlbums()
{
    Q="INSERT INTO archivedalbum (albumsn) SELECT a.albumsn"
    Q="$Q FROM zzarchivedalbum z, zzalbum za, album a"
    Q="$Q WHERE za.albumsn = z.albumsn AND a.name = za.name"
    $QueryTgt -c "$Q"
    return
}

AddArchivedTags()
{
    Q="INSERT INTO archivedtag (tagsn) SELECT t.tagsn"
    Q="$Q FROM zzarchivedtag z, zztag zt, tag t"
    Q="$Q WHERE zt.tagsn = z.tagsn AND t.name = zt.name"
    Q="$Q AND NOT EXISTS"
    Q="$Q (SELECT 1 FROM archivedtag WHERE tagsn = z.tagsn)"
    $QueryTgt -c "$Q"
    return
}

AddDeletedSets()
{
    Q="INSERT INTO deletedset (foldername)"
    Q="$Q SELECT foldername"
    Q="$Q FROM zzdeletedset z WHERE NOT EXISTS"
    Q="$Q (SELECT 1 FROM deletedset WHERE foldername = z.foldername)"
    $QueryTgt -c "$Q"
    return
}

AddDerivedPhotos()
{
    # Not assuming child and parent in same photoset
    Q="INSERT INTO derived_photo (childphotosn, parentphotosn)"
    Q="$Q SELECT c.photosn, p.photosn FROM zzderived_photo z,"
    Q="$Q zzphoto zc, zzphotoset zcs, photoset cs, photo c,"
    Q="$Q zzphoto zp, zzphotoset zps, photoset ps, photo p"
    Q="$Q WHERE zc.photosn = z.childphotosn"
    Q="$Q AND zcs.photosetsn = zc.photosetsn"
    Q="$Q AND cs.foldername = zcs.foldername"
    Q="$Q AND c.photosetsn = cs.photosetsn"
    Q="$Q AND c.filename = zc.filename"
    Q="$Q AND zp.photosn = z.parentphotosn"
    Q="$Q AND zps.photosetsn = zp.photosetsn"
    Q="$Q AND ps.foldername = zps.foldername"
    Q="$Q AND p.photosetsn = ps.photosetsn"
    Q="$Q AND p.filename = zp.filename"
    $QueryTgt -c "$Q"
    return
}

AddLocations()
{
    Q="INSERT INTO location (name, last_used, notes)"
    Q="$Q SELECT name, last_used, notes FROM zzlocation z"
    Q="$Q WHERE NOT EXISTS"
    Q="$Q (SELECT 1 FROM location WHERE name = z.name)"
    $QueryTgt -c "$Q"
    return
}

AddPhotos()
{
    # Add photos with locationsn set:
    Q="INSERT INTO photo(photosetsn, filename, locationsn,"
    Q="$Q description, width, height, thumbw, thumbh, time, latitude,"
    Q="$Q longitude, altitude)"
    Q="$Q SELECT s.photosetsn, filename, l.locationsn, description,"
    Q="$Q width, height, thumbw, thumbh, time, latitude,"
    Q="$Q longitude, altitude"
    Q="$Q FROM zzphoto z"
    Q="$Q LEFT OUTER JOIN zzlocation zl USING (locationsn),"
    Q="$Q location l, zzphotoset zs, photoset s"
    Q="$Q WHERE l.name = zl.name AND zs.photosetsn = z.photosetsn"
    Q="$Q AND s.foldername = zs.foldername"
    $QueryTgt -c "$Q"
    # Add photos with locationsn unset
    Q="INSERT INTO photo(photosetsn, filename, locationsn,"
    Q="$Q description, width, height, thumbw, thumbh, time, latitude,"
    Q="$Q longitude, altitude)"
    Q="$Q SELECT s.photosetsn, filename, NULL, description,"
    Q="$Q width, height, thumbw, thumbh, time, latitude,"
    Q="$Q longitude, altitude"
    Q="$Q FROM zzphoto z, zzphotoset zs, photoset s"
    Q="$Q WHERE z.locationsn IS NULL AND zs.photosetsn = z.photosetsn"
    Q="$Q AND s.foldername = zs.foldername"
    $QueryTgt -c "$Q"
    return
}

AddPhotoSets()
{
    Q="INSERT INTO photoset (foldername, notes, nickname)"
    Q="$Q SELECT foldername, notes, nickname FROM zzphotoset z"
    Q="$Q WHERE NOT EXISTS"
    Q="$Q (SELECT 1 FROM photoset WHERE foldername = z.foldername)"
    $QueryTgt -c "$Q"
    return
}

AddTags()
{
    Q="INSERT INTO tag (name, notes)"
    Q="$Q SELECT name, notes FROM zztag z WHERE NOT EXISTS"
    Q="$Q (SELECT 1 FROM tag WHERE name = z.name)"
    $QueryTgt -c "$Q"
    return
}

AddVideos()
{
    Q="INSERT INTO video (vidfilename, stillsn, width, height)"
    Q="$Q SELECT vidfilename, p.photosn, z.width, z.height"
    Q="$Q FROM zzvideo z, zzphoto zp, zzphotoset zs, photoset s,"
    Q="$Q photo p WHERE zp.photosn = z.stillsn"
    Q="$Q AND zs.photosetsn = zp.photosetsn"
    Q="$Q AND s.foldername = zs.foldername"
    Q="$Q AND p.photosetsn = s.photosetsn"
    Q="$Q AND p.filename = zp.filename"
    $QueryTgt -c "$Q"
    return
}

CopyAsZzTable()
{
    # Creates a new table from $1 with same content but prefixed with
    # "zz" and without keys or indices or other constraints
    Q="CREATE TABLE zz$1 AS SELECT * FROM $1"
    $QueryTemp -c "$Q" 2>&1 | grep -v NOTICE
    return
}

DropZzTables()
{
    # Prepend each word in $1 with "zz" then drop all
    Q="DROP TABLE"
    for Table in $1
    do
	Q="$Q, zz$Table"
    done
    Q=$(echo "$Q" | sed -e 's/^DROP TABLE,/DROP TABLE/')
    $QueryTgt -c "$Q"
    return
}

IsValueInTemplate1()
{
    # Return 1 if table $1, column $2 contains value $3, 0 if not
    # Usage: if [ $(IsValueInTemplate1 table column value) -eq 1 ]
    $QueryTemplate1 -t -c "$(QIsValueInTableCol $1 $2 $3)"
    return
}

QIsValueInTableCol()
{
    # Construct query for IsValueIn{Template1 | Trove}
    # Usage: $Query___ -c "$(IsValueInTableCol table column value)"
    echo "SELECT COUNT(*) FROM $1 WHERE $2 = '$3'"
    return
}

TagPhotos()
{
    Q="INSERT INTO tag_photo(tagsn, photosn)"
    Q="$Q SELECT t.tagsn, p.photosn"
    Q="$Q FROM zztag_photo z, zztag zt, tag t, zzphoto zp,"
    Q="$Q zzphotoset zs, photoset s, photo p"
    Q="$Q WHERE zt.tagsn = z.tagsn AND t.name = zt.name"
    Q="$Q AND zp.photosn = z.photosn"
    Q="$Q AND zs.photosetsn = zp.photosetsn"
    Q="$Q AND s.foldername = zs.foldername"
    Q="$Q AND p.photosetsn = s.photosetsn"
    Q="$Q AND p.filename = zp.filename"
    $QueryTgt -c "$Q"
    return
}

######################################################################
TempDb=trovexfer
QueryTemp="psql -qd $TempDb -U trove"
QueryTemplate1="psql -qd template1 -U postgres"
Tables="
album
albumpage
albumphoto
archivedalbum
archivedtag
deletedset
derived_photo
location
photo
photoset
tag
tag_photo
video
"
Usage="integrate.sh [ -h ] targetdb dumpfile"
######################################################################
# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done
shift $(( $OPTIND - 1 ))

if [ $# -lt 2 ]
then
    echo $Usage
    exit
fi
TargetDb=$1
DumpFile=$2
QueryTgt="psql -qd $TargetDb -U trove"

if [ $(IsValueInTemplate1 pg_database datname $TempDb) -eq 1 ]
then
    dropdb -U trove $TempDb
fi
echo Creating temporary database $TempDb
createdb -U trove $TempDb
echo Loading temporary database with trove dump $DumpFile
restoredb.sh $DumpFile $TempDb trove > /dev/null

# Prefix $TempDb tables with "zz" and add them to $TargetDb
for Table in $Tables
do
    CopyAsZzTable $Table
done
T="Transferring legacy data to temporary tables in database $TargetDb"
echo $T
pg_dump -U trove -O -t 'zz*' $TempDb | $QueryTgt
dropdb -U trove $TempDb

echo Adding new data to the tables, one-by-one:
echo "   " deletedset
AddDeletedSets
echo "   " album
AddAlbums
echo "   " archivedalbum
AddArchivedAlbums
echo "   " albumpage
AddAlbumPages
echo "   " location
AddLocations
echo "   " photoset
AddPhotoSets
echo "   " photo
AddPhotos
echo "   " video
AddVideos
echo "   " albumphoto
AddAlbumPhotos
echo "   " tag
AddTags
echo "   " archivedtag
AddArchivedTags
echo "   " tag_photo
TagPhotos
echo "   " derived_photo
AddDerivedPhotos

DropZzTables "$Tables"
