#!/bin/sh
######################################################################
# replacedummystillfiles.sh

# Generate Ogg-file frame images to replace any dummy placeholder
# still files ('_@_%.png') for videos
######################################################################
# Copyright 2014 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
# For each existing video that has a dummy stillfile ('_@_%.png'):

# 1. Replace dummy stillfile with a frame extracted from the Ogg file
#    in the webpics subfolder
# 2. Ensure each new stillfile has a timestamp (extract from video)

# Usage: Run as web-server user, owner of the files:

#     replacedummystillfiles.sh database serviceroot \
#         [ webpiclength [ thumbnaillength ]]
# Example:
#     sudo -u www-data replacedummystillfiles.sh trove \
#         /home/web/webroot-ssl/trove 768 192 \
#         | tee /home/public/replacedummystillfiles.log
######################################################################
ProcessImage()
{
    # Usage: ProcessImage $File
    # Generates reduced-size webpic and thumbnail
    # Generate webpic (limit webpic size to full size)
    LongShort=$(gm identify -format '%w\n%h\n' ${FullFolder}/$1 \
	| sort -rn | tr "\n" " ")
    FullLength=$(echo $LongShort | awk '{print $1}')
    FullShort=$(echo $LongShort | awk '{print $2}')
    # Adjust WebPicLength for extra-wide or extra-tall images
    if [ $(( 4 * $FullShort )) -ge $(( 3 * $FullLength )) ]
    then
	WebLength=$WebPicLength
    else
	WebLength=$(( 3 * $FullLength * $WebPicLength / $FullShort \
	    / 4 ))
    fi
    if [ $WebLength -gt $FullLength ]
    then
	TargetLength=$FullLength
    else
	TargetLength=$WebLength
    fi
    Resize $TargetLength $1 $WebFolder
    # Generate thumbnail image (limit thumbnail size to full size)
    if [ $ThumbnailLength -gt $FullLength ]
    then
	TargetLength=$FullLength
    else
	TargetLength=$ThumbnailLength
    fi
    Resize $TargetLength $1 $ThumbFolder
    return
}

RemoveDummyImages()
{
    # Usage: RemoveDummyImages $DummyFileName
    for ImageFolder in $FullFolder $WebFolder $ThumbFolder
    do
	rm $ImageFolder/$1
    done
    return
}

Resize()
{
    # Usage: Resize $PicLength $Pic $Folder
    # Puts a $PicLength-sized copy of the full-size $Pic into $Folder
    gm convert -size ${1}x${1} ${FullFolder}/$2 -resize ${1}x${1} \
	${3}/$2
    return
}

UpdatePhoto()
{
    # Usage: UpdatePhoto $StillSn $FileName $VidFile
    # Updates database record for photo $StillSn with name and
    # geometry of image $FileName and timestamp from video $VidFile
    # Assumes camera's timezone was UTC
    Geometry=$(gm identify -format '%w %h' $WebFolder/$2)
    WidthW=$(echo $Geometry | awk '{print $1}')
    HeightW=$(echo $Geometry | awk '{print $2}')
    Geometry=$(gm identify -format '%w %h' $ThumbFolder/$2)
    WidthT=$(echo $Geometry | awk '{print $1}')
    HeightT=$(echo $Geometry | awk '{print $2}')
    TimeStamp=$(VideoTimeStamp $3)
    Q="UPDATE photo SET filename = '$2', width = $WidthW,"
    Q="$Q height = $HeightW, thumbw = $WidthT, thumbh = $HeightT"
    if [ "z$TimeStamp" != "z" ]
    then
	Q="$Q, time = '$TimeStamp UTC'"
    fi
    Q="$Q WHERE photosn = $StillSn"
    $Query -c "$Q"
    return
}

VideoTimeStamp()
{
    # Usage: VideoTimeStamp $VidFile
    # Return timestamp (as a string) of video $FullFolder/$VidFile
    T="[[:digit:]][[:digit:]]:[[:digit:]][[:digit:]]:[[:digit:]]"
    T="$T[[:digit:]]"
    S=$(mplayer -really-quiet -vo null -ao null -identify -frames 0 \
	$FullFolder/$1 2>/dev/null | \
	grep ID_CLIP_INFO_VALUE | \
	grep $T | cut -d = -f 2)
    echo $S
    return
}

######################################################################
WebPicLength=768
ThumbnailLength=192
######################################################################
if [ $# -lt 2 ]
then
    echo Usage: replacedummystillfiles.sh database serviceroot \
	[ webpiclength [ thumbnaillength ]]
    exit
fi
Database=$1
Root=$2
if [ $# -gt 2 ]
then
    WebPicLength=$3
    if [ $# -gt 3 ]
    then
	ThumbnailLength=$4
    fi
fi
Query="psql -qtd $Database -U trove"

Q="SELECT foldername || '/' || vidfilename || '/' || stillsn"
Q="$Q FROM photo p, video v, photoset s"
Q="$Q WHERE filename LIKE '_@_%.png' AND v.stillsn = p.photosn"
Q="$Q AND s.photosetsn = p.photosetsn"
Q="$Q ORDER BY foldername, vidfilename"

Results=$($Query -c "$Q")
for Result in $Results
do
    Folder=$(echo $Result | cut -d / -f 1)
    VidFile=$(echo $Result | cut -d / -f 2)
    StillSn=$(echo $Result | cut -d / -f 3)
    BaseName=$(echo $VidFile | sed -e "s/\.[^.]*$//")
    OggFile=${BaseName}.ogg
    WebFolder=$Root/webpics/$Folder
    if [ -e $WebFolder/$OggFile ]
    then
	FullFolder=$Root/fullsize/$Folder
	oggThumb -t 0 -n $FullFolder/${BaseName}.jpg \
	    $WebFolder/$OggFile >/dev/null 2>&1
	if [ -e $FullFolder/${BaseName}.jpg ]
	then
	    ThumbFolder=${WebFolder}/thumbnails
	    ProcessImage ${BaseName}.jpg
	    UpdatePhoto $StillSn ${BaseName}.jpg $VidFile
	    RemoveDummyImages '_@_'${BaseName}.png
	    echo Installed new still image for $Folder/$VidFile
	else
	    echo Failed to make still image for $Folder/$VidFile
	fi
    else
	echo Found no Ogg file for $Folder/$VidFile, thus no still
    fi
done
