#!/bin/sh
######################################################################
# resetdev.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

resetdev.sh                      Copyright 2021 Theodore B. Ruegsegger

Resets the dev environment to mirror any reference phototrove database
(typically the production environment) except without photos or
videos, that is, with the albums/, fullsize/ and webpics/ folders
empty and these database tables cleared:
    album
    albumpage
    albumphoto
    archivedalbum
    deletedset
    derived_photo
    photo
    photoset
    tag_photo
    video

Also resets all user passwords in the dev environment to xyZZy!42
and makes showfunctions.php a public tool

Usage:

    resetdev.sh [ -h ] reference-dumpfile

Where:

    reference-dumpfile
	specifies the path of a dumpfile of the database to be used as
        the reference (also handles gzipped dumps).

Options:

    -h
        Display a help message listing options.

Examples:

    resetdev.sh /home/dumps/trove201102091634.dump.gz
	Copies user accounts, locations, and tags from the specified
	dumpfile

Must run as a user with sufficient sudo privilege to become the
webserver user "www-data", in order to write to the fullsize and
webpics subfolders.

Note: May need to run updateschema.sh if reference database schema
differs from trove_dev schema. In such a case, it might be simpler to
use a dump of trove_dev.

EOF
return  
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

######################################################################
TroveRoot=/home/web/webroot-ssl/trove
ProdDb=trove
DevDb=trove_dev
Owner=trove
PassWord='xyZZy!42'
Usage="resetdev.sh [ -h ] reference-dumpfile"
######################################################################
DevRoot=$TroveRoot/dev
QueryDev="psql -d $DevDb -U $Owner"

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

if [ $# -lt 1 ]
then
    echo $Usage
    exit
fi
DumpFile=$1

# Start with an empty database
echo "This will erase all photo data and all images and videos"
echo " from the dev environment ($DevDb)."
PromptYesNo "Is that OK"
if [ $Response = "N" ]
then
    echo "Ok, quitting."
    exit
fi

dropdb -U $Owner $DevDb
createdb -U $Owner $DevDb

# Clear image folders
sudo -u www-data sh -c "
rm -rf $DevRoot/albums/*
rm -rf $DevRoot/fullsize/*
rm -rf $DevRoot/webpics/*
"

# Transfer db records (all but those related to photos or videos)
Type=$(file -b $DumpFile | awk '{print $1}')
if [ "z$Type" = "zgzip" ]
then
    zcat $DumpFile
else
    cat $DumpFile
fi \
    | sed -e '/^COPY public.photo/,/^\\\.$/d' \
    -e '/^COPY public.archivedalbum/,/^\\\.$/d' \
    -e '/^COPY public.album/,/^\\\.$/d' \
    -e '/^COPY public.deletedset/,/^\\\.$/d' \
    -e '/^COPY public.derived_photo/,/^\\\.$/d' \
    -e '/^COPY public.tag_photo/,/^\\\.$/d' \
    -e '/^COPY public.video/,/^\\\.$/d' \
    | $QueryDev -q 2>&1 \
    | grep -v "NOTICE:  ALTER TABLE / ADD PRIMARY KEY" \
    | grep -v 'ERROR:  table ".*" does not exist' \
    | grep -v 'ERROR:  relation ".*" does not exist' \
    | grep -v 'ERROR:  index ".*" does not exist' \
    | grep -v 'ERROR:  sequence ".*" does not exist' \
    | grep -v 'ERROR:  language ".*" does not exist' \
    | grep -v 'ERROR:  schema "public" already exists' \
    | grep -v 'ERROR:  must be owner of schema public' \
    | grep -v 'ERROR:  must be member of role "postgres"' \
    | grep -v 'WARNING:  no privileges' \
    | sed -ne '/^ setval$/,/^(1 row)$/d'

# Reset all user passwords in the dev environment to $PassWord
CypherText=$(openssl passwd -1 $PassWord)
$QueryDev -q <<EOF
UPDATE reguser SET passwd_hash = '$CypherText',
 passwd_changed = CURRENT_DATE;
EOF
echo "Password for every account is \"$PassWord\""
$QueryDev -q <<EOF
UPDATE tool_priv SET privilegesn =
(SELECT privilegesn FROM privilege WHERE privname = 'public')
WHERE toolsn =
(SELECT toolsn FROM tool WHERE name = 'showfunctions.php');
EOF
echo "Everyone may now View all PHP function definitions"
