#!/bin/bash
######################################################################
# resettestdb.sh

# Resets the specified database to the integration testing data set.
# Asks for confirmation if not a "_tst" database.

######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################

# Usage:
#     resettestdb.sh database [ password [ dbuser ] ]
# Where:
#     password	is the initial password to be installed for all users
#		(default is xyZZy!42 as in the test scripts)
#     dbuser	is the database user (default is service name, the
#     		base name of the database)
######################################################################
Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo ()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

######################################################################
TroveRoot=/home/web/webroot-ssl/trove
PassWord='xyZZy!42'

if [ $# -lt 1 ]
then
    echo Usage: resettestdb.sh database [ password [ dbuser ] ]
    exit
elif [ $# -gt 1 ]
then
    PassWord=$2
fi
Database=$1
ServiceName=$(echo $Database | sed -e 's/_.*$//')
if [ $# -gt 2 ]
then
    DbUser=$3
else
    DbUser=$ServiceName
fi

CypherText=$(openssl passwd -1 $PassWord)

if echo $Database | grep -v '_tst' 2>/dev/null
then
    T="This will erase whatever's currently in the $Database"
    T="$T database."
    echo $T
    PromptYesNo "Are you certain you want to do this"
    if [ $Response = "N" ]
    then
	echo OK, aborting.
	exit
    fi
fi

if echo $0 | grep '/' >/dev/null
then
    ScriptDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    ScriptDir=$(pwd)
fi

DoSql="psql -q -d $Database -U $DbUser"
RunSql="$DoSql -f"

echo "Clearing database $Database..."

dropdb -U $DbUser $Database 2>&1 \
    | grep -v "ERROR:  database \"$Database\" does not exist"

createdb -U $DbUser $Database

echo "Repopulating database $Database with test data..."

sh $ScriptDir/gk/createbasictables.sh $Database
sh $ScriptDir/createapptables.sh $Database
# $RunSql $ScriptDir/local/loadinitialvalues.sql 2>&1 | grep -v NOTICE
# $RunSql $ScriptDir/local/loadinitialusers.sql 2>&1 | grep -v NOTICE
$RunSql $ScriptDir/gk/loadgktestdata.sql -v cyphertext=\'$CypherText\'
$RunSql $ScriptDir/resettestdb.sql -v cyphertext=\'$CypherText\'

if [ ! -d /home/public/photos/samplephotoset ]
then
    echo 'Fetching sample photo set from http://tbr.paunix.org/'
    echo 'This may take a few minutes...'
    wget -qrnH -P /home/public/photos/ \
	http://tbr.paunix.org/samplephotoset
fi
echo
echo "Created these user accounts:"
$DoSql -tc "SELECT userid FROM reguser WHERE userid != 'admin'
ORDER BY userid;"

$DoSql -tc "SELECT userid || ' may ' || lower(description)
FROM user_priv up, reguser r, privilege p
WHERE r.regusersn = up.regusersn AND p.privilegesn = up.privilegesn
AND userid != 'admin'
ORDER BY userid, privname;"

echo "Password for every account is \"$PassWord\""

Env=$(echo $Database | sed -ne 's/^.*_\([^_][^_]*$\)/\1/p')

if [ "z$Env" = "ztst" ]
then
    PromptYesNo "Clear out all photo image and album files"
    if [ $Response = "Y" ]
    then
	# Clear image folders
	TstRoot=$TroveRoot/tst
	for Dir in albums fullsize webpics
	do
	    sudo -u www-data sh -c "rm -rf $TstRoot/$Dir/*"
	done
    fi
else
    EnvRoot=$TroveRoot/$Env
    if [ -d $EnvRoot/fullsize ]
    then
	echo Service root for $Env/ environment appears to be in
	echo $EnvRoot
	PromptYesNo "Clear out all image and album files"
	if [ $Response = "Y" ]
	then
	    for Dir in albums fullsize webpics
	    do
		sudo -u www-data sh -c "rm -rf $EnvRoot/$Dir/*"
	    done
	else
	    echo "Ok, but remember to clear them out manually!"
	fi
    else
	echo "Remember to clear out image and album files if needed."
    fi
fi
