#!/bin/sh
######################################################################
# resettvl.sh

# Initializes a portable environment (for example, for a traveling
# laptop) for the PhotoTrove web application in preparation for
# independent operation

######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

resettvl.sh                      Copyright 2021 Theodore B. Ruegsegger

Initializes a portable environment (for example, for a traveling
laptop) for the PhotoTrove web application in preparation for
independent operation, after which the accumulated photos can be added
back to the production environment using the integrate.sh script.

Accomplishes this by resetting the tvl environment to mirror the
production environment (or any reference trove database) except
without photos or videos, that is, with the albums/, fullsize/ and
webpics/ folders empty and these database tables cleared:
    album
    albumpage
    albumphoto
    archivedalbum
    derived_photo
    photo
    photoset
    tag_photo
    video

Preserves the production environment's locations, tags and user
accounts (with passwords and privileges).

Usage:

    resettvl.sh [ -h ] dumpfile

Where dumpfile is a dump (gzipped is ok) of the latest version of the
production database.

Options:

    -h
        Display a help message listing options.

Must run as a user with sufficient sudo privilege to become the
webserver user "www-data", in order to write to the fullsize and
webpics subfolders.

EOF
return  
}

IsValueInTemplate1()
{
    # Return 1 if table $1, column $2 contains value $3, 0 if not
    # Usage: if [ $(IsValueInTemplate1 table column value) -eq 1 ]
    $QueryTemplate1 -t -c "$(QIsValueInTableCol $1 $2 $3)"
    return
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

QIsValueInTableCol()
{
    # Construct query for IsValueIn{Template1 | Trove}
    # Usage: $Query___ -c "$(IsValueInTableCol table column value)"
    echo "SELECT COUNT(*) FROM $1 WHERE $2 = '$3'"
    return
}

######################################################################
TroveRoot=/home/web/webroot-ssl/trove
TvlDb=trove_tvl
Owner=trove
Usage="resettvl.sh [ -h ] dumpfile"
######################################################################
TvlRoot=$TroveRoot/tvl
QueryTvl="psql -qd $TvlDb -U $Owner"
QueryTemplate1="psql -qd template1 -U postgres"

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done
shift $(( $OPTIND - 1 ))

if [ $# -lt 1 ]
then
    echo $Usage
    exit
fi
DumpFile=$1

if [ $(IsValueInTemplate1 pg_database datname $TvlDb) -eq 1 ]
then
    echo "This will erase all photo data and all images and videos"
    echo " from the tvl environment ($TvlDb)."
    PromptYesNo "Is that OK"
    if [ $Response = "N" ]
    then
	echo "Ok, quitting."
	exit
    else
	echo "Ok, erasing current database $TvlDb."
	dropdb -U $Owner $TvlDb
    fi
fi
createdb -U $Owner $TvlDb

# Clear image folders
sudo -u www-data sh -c "
rm -rf $TvlRoot/albums/*
rm -rf $TvlRoot/fullsize/*
rm -rf $TvlRoot/webpics/*
"

# Transfer db records (all but those related to photos or videos)
Type=$(file -b $DumpFile | awk '{print $1}')
if [ "z$Type" = "zgzip" ]
then
    zcat $DumpFile
else
    cat $DumpFile
fi \
    | sed -e '/^COPY public.photo/,/^\\\.$/d' \
    -e '/^COPY public.archivedalbum/,/^\\\.$/d' \
    -e '/^COPY public.album/,/^\\\.$/d' \
    -e '/^COPY public.derived_photo/,/^\\\.$/d' \
    -e '/^COPY public.tag_photo/,/^\\\.$/d' \
    -e '/^COPY public.video/,/^\\\.$/d' \
    | $QueryTvl -q 2>&1 \
    | grep -v "NOTICE:  ALTER TABLE / ADD PRIMARY KEY" \
    | grep -v 'ERROR:  table ".*" does not exist' \
    | grep -v 'ERROR:  relation ".*" does not exist' \
    | grep -v 'ERROR:  index ".*" does not exist' \
    | grep -v 'ERROR:  sequence ".*" does not exist' \
    | grep -v 'ERROR:  language ".*" does not exist' \
    | grep -v 'ERROR:  schema "public" already exists' \
    | grep -v 'ERROR:  must be owner of schema public' \
    | grep -v 'ERROR:  must be member of role "postgres"' \
    | grep -v 'WARNING:  no privileges' \
    | sed -ne '/^ setval$/,/^(1 row)$/d'

echo Independent environment \"trove_tvl\" initialized.
