#!/bin/sh
######################################################################
# scanupload.sh
######################################################################
# Copyright 2023 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

scanupload.sh               Copyright      2023 Theodore B. Ruegsegger

Variant of fotoupload.sh tailored to Wolverine SNaP slide-, negative-
and photo-scanner. Works the same except renames the image files to
use a supplied prefix and starting number.

Mounts any of various memory card types and copies all files to a new
subfolder (with a unique name based on the date and time) of
$UploadPath.

Usage:
    scanupload.sh [ -hs ][ -f firstname]

Example:
    scanupload.sh -f slide0097

Options:
    -f firstname
        Specify desired filename for first file as <prefix><number>,
        for example, slide0097. If omitted, script will prompt after
        loading images to temporary folder.

    -h
        Display a help message listing options.

    -s
	Display system and configuration requirements.

EOF
return  
}

PrintReqts()
{
cat <<EOF

Requirements:

scanupload.sh is adapted for memory cards that contain images in one
or more subfolders under a "dcim" or "DCIM" folder, which most digital
cameras create per the Digital Camera IMages specification:
http://en.wikipedia.org/wiki/Design_rule_for_Camera_File_system

Can handle CompactFlash/MicroDrive in PCMCIA adapter (appears as IDE
drive) as well as various cards in USB devices (appears as SCSI
drive).

The \$Devices variable lists devices to consider, currently:
$Devices
but the script ignores any devices with no corresponding mount point
under the /media/ folder; this avoids conflicts with other devices
like permanently-mounted SCSI disk drives.

Note that all mount points require an entry in /etc/fstab, for
example:

Device:		/dev/sdb1
Mount point:	/media/sdb1
/etc/fstab line:
/dev/sdb1	/media/sdb1	auto	user,noauto	0	0

EOF
return  
}

ParseFirstName()
{
    # Separates $FirstName into $Prefix and $FirstNum
    Prefix=$(echo $FirstName | sed -e 's/[0-9][0-9]*$//' \
	| sed -e 's/[^[:alnum:]_-]//g')
    FirstNum=$(echo $FirstName \
	| sed -e 's/^.*[^0-9]\([0-9][0-9]*\)$/\1/')
    if [ "z$Prefix" = "z" -o "z$FirstNum" = "z" ]
    then
	T="Name for first image must be a prefix followed by a"
	T="$T number, like img0094"
	echo $T
	PromptFirstName
	ParseFirstName
    fi
    return
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptFirstName()
{
    echo -n "Name for first image [prefixNNNN, e.g., img0459]? "
    read FirstName
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

RenameFiles()
{
    N=$FirstNum
    cd $TargetFolder
    for File in $(ls *.*)
    do
	if [ $N -gt 9999 -o $N -lt 1 ]
	then
	    N=1
	fi
	# Pad with zeroes to 4 digits
	NN=$(echo "000$N" | rev | cut -c -4 | rev)
	mv $File ${Prefix}${NN}.jpg
	N=$(( $N + 1 ))
    done
    cd ..
    return
}

######################################################################
UploadPath='/home/public/photos'
Devices="
hde1
mmcblk0p1
sda1
sdb1
sdc1
sdd1
sde1
"
Usage="scanupload.sh [ -hs ][ -f ]"
######################################################################
Host=$(hostname)

# Parse command line options
while getopts "f:hs" Option
do
    case $Option in
      f ) FirstName=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      s ) PrintReqts
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

MountPoints=""
for Device in $Devices
do
    if [ -d /media/$Device ]
    then
	if grep /media/$Device /etc/fstab >/dev/null
	then
	    MountPoints="$MountPoints $Device"
	fi
    fi
done
Count=$(echo $MountPoints | wc -w)
if [ $Count -eq 0 ]
then
    echo None of these devices: $Devices
    echo "has both a mount point in the /media/ folder"
    echo "and an entry in the /etc/fstab file."
    echo "Can't continue."
    exit
else
    Devices=$MountPoints
fi

DevicesPresent=""
for Device in $Devices
do
    if [ -b /dev/$Device ]
    then
	DevicesPresent="$DevicesPresent $Device"
    fi
done
Count=$(echo $DevicesPresent | wc -w)
if [ $Count -eq 0 ]
then
    echo Found none of these devices: $Devices
    echo "Can't continue."
    exit
elif [ $Count -gt 1 ]
then
    echo Found more than one device: $DevicesPresent
    echo "Can't continue (yet; in future, I'll ask which you want)."
    exit
else
    Device=$(echo $DevicesPresent)
    if [ ! -d /media/$Device/dcim -a ! -d /media/$Device/DCIM ]
    then
	mount /media/$Device
    fi
    if [ ! -d /media/$Device/dcim -a ! -d /media/$Device/DCIM ]
    then
	echo "Couldn't find a dcim or DCIM folder."
	echo "Might have failed to mount the memory device"
	echo "Can't continue."
	exit
    else
	DcimFolder=$(ls /media/$Device | grep -i dcim)
    fi
fi

# If we got this far, then /dev/$Device is mounted as /media/$Device
# and contains a dcim/DCIM folder $DcimFolder

cd $UploadPath
# Exclude the canonmsc folder; don't seem to need those files:
for ImageFolder in $(ls -d /media/$Device/$DcimFolder/* \
    | grep -vi 'canonmsc')
do
    if [ -d $ImageFolder -a \
	$(ls -1 $ImageFolder/ | sed -ne '1p')"ZZ" != "ZZ" ]
    then
	TargetFolder=$(date +%Y%m%d-%H%M%S)
	# Verify that folder doesn't already exist
	while [ -d "$TargetFolder" ]
	do
	    sleep 1
	    TargetFolder=$(date +%Y%m%d-%H%M%S)
	done
	echo Creating $TargetFolder
	mkdir $TargetFolder
	cp $ImageFolder/* $TargetFolder
	sync ; sync
	chmod 664 $TargetFolder/*
	ls -FAl $TargetFolder
	# check if folder contents match
	SrcContents=$(ls -l $ImageFolder | grep -v '^total ' \
	    | awk '{print $5,$NF}')
	TgtContents=$(ls -l $TargetFolder | grep -v '^total ' \
	    | awk '{print $5,$NF}')
	if [ "$SrcContents" = "$TgtContents" ]
	then
	    Count=$(ls -1 $TargetFolder | wc -l)
	    echo "$Count files uploaded to $Host."
	    if ! [ $FirstName ]
	    then
		PromptFirstName
	    fi
	    ParseFirstName
	    RenameFiles
	    PromptYesNo "OK to erase $ImageFolder on $Device"
	    if [ $Response = "Y" ]
	    then
		echo Erasing $ImageFolder
		rm -r $ImageFolder
	    else
		echo Leaving files in $ImageFolder
		echo "You'll need to clear it manually"
	    fi
	else
	    echo '****************************************'
	    echo '*           File Copy Failed!          *'
	    echo '****************************************'
	    echo
	    S="Be sure to correct the problem before erasing"
	    S="$S removable storage device $Device"
	    echo $S
	fi
    fi
done
sync ; sync
PromptYesNo "Unmount the removable storage device $Device"
if [ $Response = "Y" ]
then
    umount /media/$Device
fi
