#!/bin/sh
######################################################################
# scanxfer.sh
######################################################################
# Copyright 2023 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

scanxfer.sh    Copyright                   2023 Theodore B. Ruegsegger

Variant of fotoxfer.sh tailored to Wolverine SNaP slide-, negative-
and photo-scanner. Works the same except renames the image files to
use a supplied prefix and starting number.

Mounts any of various memory card types, copies each image folder to a
new subfolder (with a unique name based on the date and time) in a
temporary workspace on the local host, releases the memory card, then
copies all the image folders to the specified upload folder on a
remote host. Upon successful file transfer, deletes the temporary
workspace.

Usage:
    scanxfer.sh [ -hs ] [ -ed uploadpath ] [ -f firstname ] \
[ user@ ]remotehost

Where:

    remotehost is the IP address or resolvable name of the PhotoTrove
    server

    user is an account on remotehost with write access to the upload
    folder (defaults to same name as current user on local host)

    uploadpath is the full path to the upload folder on the remote
    host where the new image folder will be saved (defaults to
    /home/public/photos/)

Example:
    scanxfer.sh -ef slide0097 ted@pikachu

Options:
    -d uploadpath
	Explicitly specify upload folder on remote host, overriding
	default /home/public/photos/

    -e
        Erase and unmount memory card after local upload, without
        prompting. Default action is to ask first.

    -f firstname
        Specify desired filename for first file as <prefix><number>,
        for example, slide0097. If omitted, script will prompt after
        loading images to temporary folder.

    -h
        Display a help message listing options.

    -s
	Display system and configuration requirements.

Note:
    Saves a temporary copy of images in current working directory, so
    local filesystem must have adequate space for the uploaded images.

EOF
return  
}

PrintReqts()
{
cat <<EOF

Requirements:

scanxfer.sh is adapted for memory cards that contain images in one or
more subfolders under a "dcim" or "DCIM" folder, which most digital
cameras create per the Digital Camera IMages specification:
http://en.wikipedia.org/wiki/Design_rule_for_Camera_File_system

Can handle CompactFlash/MicroDrive in PCMCIA adapter (appears as IDE
drive) as well as various cards in USB devices (appears as SCSI
drive).

The \$Devices variable lists devices to consider, currently:
$Devices
but the script ignores any devices with no corresponding mount point
under the /media/ folder; this avoids conflicts with other devices
like permanently-mounted SCSI disk drives.

Note that all mount points require an entry in /etc/fstab, for
example:

Device:		/dev/sdb1
Mount point:	/media/sdb1
/etc/fstab line:
/dev/sdb1	/media/sdb1	auto	user,noauto	0	0

EOF
return  
}

FoundInList()
{
    # Returns True if $1 contains $2, else False
    if echo $1 | grep $2 >/dev/null
    then
	echo True
    else
	echo False
    fi
    return
}

FoundOnRemote()
{
    # Returns True if folder $1 exists in $UploadPath on $RemoteHost,
    # else False
    ssh $RemoteHost \
	"if [ -d $UploadPath/$1 ]
	 then echo True; else echo False; fi"
    return
}

NewUniqueTimestamp()
{
    # Returns a timestamp string that isn't already on remote host or
    # in list of folders waiting to be uploaded
    Timestamp=$(date +%Y%m%d-%H%M%S)
    # Check whether folder $Timestamp already exists on remote host
    ExistsOnRemote=$(FoundOnRemote $Timestamp)
    # Verify that folder $Timestamp isn't already queued locally
    IsInQueue=$(FoundInList "$TargetFolders" $Timestamp)
    # Increment Timestamp until new and unique
    while [ $ExistsOnRemote = "True" -o $IsInQueue = "True" ]
    do
	sleep 1
	Timestamp=$(date +%Y%m%d-%H%M%S)
	ExistsOnRemote=$(FoundOnRemote $Timestamp)
	IsInQueue=$(FoundInList "$TargetFolders" $Timestamp)
    done
    echo $Timestamp
    return
}

ParseFirstName()
{
    # Separates $FirstName into $Prefix and $FirstNum
    Prefix=$(echo $FirstName | sed -e 's/[0-9][0-9]*$//' \
	| sed -e 's/[^[:alnum:]_-]//g')
    # Trim leading zeroes (else number looks octal):
    FirstNum=$(echo $FirstName \
	| sed -e 's/^.*[^1-9]\([0-9][0-9]*\)$/\1/')
    if [ "z$Prefix" = "z" -o "z$FirstNum" = "z" ]
    then
	T="Name for first image must be a prefix followed by a"
	T="$T number, like img0094"
	echo $T
	PromptFirstName
	ParseFirstName
    fi
    return
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptFirstName()
{
    echo -n "Name for first image [prefixNNNN, e.g., img0459]? "
    read FirstName
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

RenameFiles()
{
    # Rename all files in target folder $1 with prefix $2 and 4-digit
    # serial number beginning with first number $3, then return next
    # number
    N=$3
    cd $TargetFolder
    for File in $(ls *.*)
    do
	if [ $N -gt 9999 -o $N -lt 1 ]
	then
	    N=1
	fi
	# Pad with zeroes to 4 digits
	NN=$(echo "000$N" | rev | cut -c -4 | rev)
	mv $File ${2}${NN}.jpg
	N=$(( $N + 1 ))
    done
    cd ..
    echo $N
    return
}

WarnFailedLocalCopies()
{
    # Report failure to copy files from removable device $1 to
    # temporary folder $2, preventing upload to remote host $3
    S="There was a problem copying files from one or more image"
    S="$S folders on removable device $1 to corresponding folders"
    S="$S under temporary folder $2."
    echo $S
    echo Leaving all files on device $1, still mounted as /media/$1.
    echo Nothing uploaded to remote host $3.
    echo "You'll need to fix this manually."
    return
}

WarnFailedLocalCopy()
{
    # Report failure to copy files from image folder $1 to target
    # folder $2
    echo '****************************************'
    echo '*           File Copy Failed!          *'
    echo '****************************************'
    echo
    echo There was a problem copying files from $1 to $2.
    S="Be sure to correct the problem before erasing removable"
    S="$S storage device $Device."
    echo $S
    return
}

WarnFailedRemoteCopies()
{
    # Report failure to copy files from folders under temporary
    # working folder $1 to remote host $2
    S="There was a problem copying files from one or more image"
    S="$S folders under temporary folder $(pwd)$1 to corresponding"
    S="$S folders on remote server $2."
    echo $S
    echo Leaving temporary folder $(pwd)$1 intact.
    echo "You'll need to fix this manually."
    return
}

WarnFailedRemoteCopy()
{
    # Report failure to copy files from temporary folder $1 to remote
    # host $2
    echo '****************************************'
    echo '*          Remote Copy Failed!         *'
    echo '****************************************'
    echo
    echo "There was a problem copying $1 to remote server $2."
    echo All the files are intact in the temporary folder $(pwd)/$1
    S="Be sure to correct the problem before erasing the temporary"
    S="$S folder."
    echo $S
    return
}

######################################################################
UploadPath='/home/public/photos'
Devices="
hde1
mmcblk0p1
sda1
sdb1
sdc1
sdd1
sde1
"
Usage="scanxfer.sh [ -hs ] [ -ed uploadpath ] [ -f firstname ]"
Usage="$Usage [ user@ ]remotehost"
######################################################################
LocalHost=$(hostname)

# Parse command line options
while getopts "d:ef:hs" Option
do
    case $Option in
      d ) UploadPath=$OPTARG
          ;;
      e ) EraseAfterCopy=True
          ;;
      f ) FirstName=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      s ) PrintReqts
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -lt 1 ]
then
    echo $Usage
    exit
fi
RemoteHost=$1

if ! (ping -c 1 -w 1 $RemoteHost >/dev/null)
then
    echo Remote host $RemoteHost is unreachable.
    echo "Can't continue."
    exit
fi
    
RemotePath=$RemoteHost:$UploadPath

if ! [ $FirstName ]
then
    PromptFirstName
fi
# Parse first name to get $Prefix and $FirstNum
ParseFirstName
	    
MountPoints=""
for Device in $Devices
do
    if [ -d /media/$Device ]
    then
	if grep /media/$Device /etc/fstab >/dev/null
	then
	    MountPoints="$MountPoints $Device"
	fi
    fi
done
Count=$(echo $MountPoints | wc -w)
if [ $Count -eq 0 ]
then
    echo None of these devices: $Devices
    echo "has both a mount point in the /media/ folder"
    echo "and an entry in the /etc/fstab file."
    echo "Can't continue."
    exit
else
    Devices=$MountPoints
fi

DevicesPresent=""
for Device in $Devices
do
    if [ -b /dev/$Device ]
    then
	DevicesPresent="$DevicesPresent $Device"
    fi
done
Count=$(echo $DevicesPresent | wc -w)
if [ $Count -eq 0 ]
then
    echo Found none of these devices: $Devices
    echo "Can't continue."
    exit
elif [ $Count -gt 1 ]
then
    echo Found more than one device: $DevicesPresent
    echo "Can't continue (yet; in future, I'll ask which you want)."
    exit
else
    Device=$(echo $DevicesPresent)
    if [ ! -d /media/$Device/dcim -a ! -d /media/$Device/DCIM ]
    then
	mount /media/$Device
    fi
    if [ ! -d /media/$Device/dcim -a ! -d /media/$Device/DCIM ]
    then
	echo "Couldn't find a dcim or DCIM folder."
	echo "Might have failed to mount the memory device"
	echo "Can't continue."
	exit
    else
	DcimFolder=$(ls /media/$Device | grep -i dcim)
    fi
fi

# If we got this far, then /dev/$Device is mounted as /media/$Device
# and contains a dcim/DCIM folder $DcimFolder

TempDir=$(mktemp -dp .)
if [ "z$TempDir" = "z" ]
then
    echo '***********************************************************'
    echo "Current directory not writable"
    echo "Can't continue."
    umount /media/$Device
    echo You may safely remove removable device $Device. To try again
    echo in a writable directory, remove and reinsert device $Device.
    echo '***********************************************************'
    exit
fi
cd $TempDir
FailedXfer=
# Exclude the canonmsc folder; don't seem to need those files:
ImageFolders=$(ls -d /media/$Device/$DcimFolder/* \
    | grep -vi 'canonmsc')

# Copy all image folders to $TempDir
FailedCopy=
TargetFolders=
for ImageFolder in $ImageFolders
do
    # Load only non-empty folders
    if [ -d $ImageFolder -a \
	$(ls -1 $ImageFolder/ | sed -ne '1p')"ZZ" != "ZZ" ]
    then
	echo Found image folder $ImageFolder
	TargetFolder=$(NewUniqueTimestamp)
	echo Creating $TargetFolder
	TargetFolders="$TargetFolders $TargetFolder"
	mkdir $TargetFolder
	cp $ImageFolder/* $TargetFolder
	sync ; sync
	chmod 664 $TargetFolder/*
	ls -FAl $TargetFolder
	# check that folder contents match
	SrcContents=$(ls -l $ImageFolder | grep -v '^total ' \
	    | awk '{print $5,$NF}')
	TgtContents=$(ls -l $TargetFolder | grep -v '^total ' \
	    | awk '{print $5,$NF}')
	if [ "$SrcContents" = "$TgtContents" ]
	then
	    Count=$(ls -1 $TargetFolder | wc -l)
	    echo "$Count files uploaded to local host $LocalHost."
	else
	    FailedCopy="$FailedCopy $TargetFolder"
	    WarnFailedLocalCopy $ImageFolder $TargetFolder
	fi
	echo
    fi
done

# If any local copy failed, warn and abort
if [ $FailedCopy ]
then
    WarnFailedLocalCopies $Device $TempDir $RemoteHost
    cd ..
    exit
fi

# Local copies all succeeded. Erase and unmount the storage device
# unless user wishes otherwise
if [ $EraseAfterCopy ]
then
    rm -r $ImageFolders
    umount /media/$Device
    echo You may safely remove removable device $Device.
else
    PromptYesNo "Erase and unmount removable device $Device"
    if [ $Response = "Y" ]
    then
	rm -r $ImageFolders
	umount /media/$Device
	echo You may safely remove removable device $Device.
    else
	echo Leaving all files on removable device $Device.
	echo "You'll need to clear it manually."
	PromptYesNo "Unmount $Device"
	if [ $Response = "Y" ]
	then
	    umount /media/$Device
	    echo You may safely remove removable device $Device.
	else
	    echo Leaving device mounted as /media/$Device
	fi
    fi
fi
echo

# Copy all target folders to remote host
FailedXfer=
for TargetFolder in $TargetFolders
do
    FirstNum=$(RenameFiles $TargetFolder $Prefix $FirstNum)
    echo Copying $TargetFolder to $RemotePath/
    if (rsync -rv $TargetFolder $RemotePath)
    then
	S="Successfully copied folder $TargetFolder to remote"
	S="$S server $RemoteHost."
	echo $S
    else
	FailedXfer=True
	WarnFailedRemoteCopy $TargetFolder $RemoteHost
    fi
    echo
done
sync ; sync

cd ..
if [ $FailedXfer ]
then
    WarnFailedRemoteCopies $TempDir $RemoteHost
else
    echo "Erasing temporary files."
    rm -r $TempDir
fi
