#!/bin/sh
######################################################################
# setup.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

setup.sh                         Copyright 2021 Theodore B. Ruegsegger

Initial setup script to establish the service software and
database(s).

Usage:
    setup.sh [-h] [-n]

Preconditions:

  o You have unpacked the entire package in a safe place (not the web
    server document tree).

  o You have edited the system configuration file
    $ScriptDir/local/def_common.sh
    to define your system.

  o The system administrator has set up the web server (preferably
    using SSL for secure connections), PHP, and PostgreSQL. PHP must
    include support for PostgreSQL. PostgreSQL must allow $User (the
    user running this script) and the web server user to access
    postgreSQL databases.

  o The system administrator has reserved the web server's document
    root, or a subdirectory therein, for the service's php code and
    data, with permissions allowing $User (the user running this
    script) to write files into it. According to the system
    configuration file, the service root is
    $ServiceRootPath

EOF
    ReportStatus "service root" "is" $BadServiceRoot
    ReportStatus "redirect root (non-ssl)" "is" $BadRedirect
    cat <<EOF

  o The system administrator has created the folder to which original
    photos can be uploaded from the camera, with permissions allowing
    $User (the user running this script) and optionally other users to
    write files into it. According to the system configuration file,
    the original photos folder is
    $OriginalPhotos

EOF
    ReportStatus "original photos folder" "is" $BadOrigsFolder
    DescribeAppReqts
    ReportAppReqtStatus
    cat <<EOF

Options:
    -h
        Display a help screen listing options.
    -n
        Noninteractive; don't prompt unless something is wrong

EOF
return  
}

CheckOrigsFolder()
{
    if ! [ -d $OriginalPhotos -a -w $OriginalPhotos \
	-a -x $OriginalPhotos ]
    then
	BadOrigsFolder=True
    fi
    return
}

CheckRedirect()
{
    RedirectPath=${WebServerDocumentRoot}$ServiceRoot
    if ! [ -d $RedirectPath -a -w $RedirectPath \
	-a -x $RedirectPath ]
    then
	BadRedirect=True
    fi
    return
}

CheckServiceRoot()
{
    ServiceRootPath=${WebServerSslDocumentRoot}$ServiceRoot
    if ! [ -d $ServiceRootPath -a -w $ServiceRootPath \
	-a -x $ServiceRootPath ]
    then
	BadServiceRoot=True
    fi
    return
}

InitialCaps ()
{
    RawString=$*
    CappedString=""
    for Word in $RawString
    do
	FirstCap=`echo $Word | cut -c 1 | tr "a-z" "A-Z" `
	Rest=`echo $Word | cut -c 2-`
	CappedString="$CappedString ${FirstCap}${Rest}"
    done
    CappedString=`echo $CappedString`
    return  
}

Prompt ()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=`echo $RawResponse | tr a-z A-Z`
    return
}

PromptYesNo ()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

ReportStatus()
{
    Description=$1
    Verb=$2
    if [ $3 ]
    then
	echo "    There is a problem with the $Description."
    else
	echo "    The $Description $Verb set up properly."
    fi
}


######################################################################
Usage="setup.sh [-h] [-n]"
######################################################################
User=$(whoami)
if echo $0 | grep '/' >/dev/null
then
    ScriptDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    ScriptDir=$(pwd)
fi

StartDir=$(pwd)
cd $ScriptDir

# Load definitions
. local/def_common.sh

# Establish the service root
CheckServiceRoot

# Establish the redirect root (non-ssl)
CheckRedirect

# Check presence and permissions of original photos folder
CheckOrigsFolder

# Check additional requirements
. ./appreqts.sh
TestAppReqts

# Parse command line options
while getopts "hn" Option
do
    case $Option in
	h ) PrintHelp
	    exit
	    ;;
	n ) Noninteractive=True
	    ;;
	\?) echo $Usage
	    exit
	    ;;
    esac
done

if [ ! $Noninteractive ]
then
    echo
    echo "Have you checked the help screen ($0 -h) to be sure you've"
    PromptYesNo "met all the preconditions"
    if [ $Response = "N" ]
    then
	PrintHelp
	exit
    fi
fi

echo
echo "Checking preconditions where possible."
ReportStatus "service root" "is" $BadServiceRoot
ReportStatus "redirect root (non-ssl)" "is" $BadRedirect
ReportAppReqtStatus
if [ "z${BadServiceRoot}$AppReqtFailed" != "z" ]
then
    echo
    echo "Please fix and run this script again."
    exit
fi

echo
echo "Creating the production database"
sh gk/createdatabases.sh -pw "$WebServerDbUser" $DbBaseName $DbUser \
    > /tmp/setuplog
echo
echo "Loading default data into the production database"
# There may be a wrapper for createbasictables.sql; if so, run that
# instead of running the SQL script directly:
if [ -e gk/createbasictables.sh ]
then
    sh gk/createbasictables.sh $DbBaseName $DbUser \
	>> /tmp/setuplog 2>&1
else
    # Script may use \i to load other SQL; ensure it knows where it is
    psql -d $DbBaseName -U $DbUser -f gk/createbasictables.sql \
    >> /tmp/setuplog 2>&1
fi

# There may be a wrapper for createapptables.sql; if so, run that
# instead of running the SQL script directly:
if [ -e ./createapptables.sh ]
then
    sh ./createapptables.sh $DbBaseName $DbUser >> /tmp/setuplog 2>&1
else
    # Script may use \i to load other SQL; ensure it knows where it is
    psql -d $DbBaseName -U $DbUser -f ./createapptables.sql \
	>> /tmp/setuplog 2>&1
fi

if [ -e local/loadinitialvalues.sql ]
then
    psql -d $DbBaseName -U $DbUser -f local/loadinitialvalues.sql \
	>> /tmp/setuplog 2>&1
fi

WebDir=$(cd .. ; cd web ; pwd)
echo
echo "Copying the php code"
echo "    from $WebDir"
echo "    to   $ServiceRootPath"
cp -R $WebDir/* $ServiceRootPath

WebDir=$(cd .. ; cd webredirect ; pwd)
echo
echo "Copying the php redirect code"
echo "    from $WebDir"
echo "    to   $RedirectPath"
cp -R $WebDir/* $RedirectPath

. ./appsetup.sh
ConfigureApp
cd $StartDir

