#!/bin/sh
######################################################################
# updateschema.sh
######################################################################
# Copyright 2022 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

updateschema.sh                  Copyright 2022 Theodore B. Ruegsegger

Adjusts a PhotoTrove database as needed to conform to latest
release. May be run at any time.

Usage:
    updateschema.sh [ -h ] database

EOF
    return  
}

CountTable()
{
    # Return 1 if table $1 exists in database, 0 if not
    # Usage: if [ $(CountTable "newtable") -eq 0 ]
    Table=$1
    Q="SELECT COUNT(*) FROM pg_tables WHERE tablename = '$Table'"
    psql -t -d $Database -U $DbUser -c "$Q"
    return
}

IsColInTable()
{
    # Return 1 if table $1 contains column $2, 0 if not
    # Usage: if [ $(IsColInTable "thetable" "newcolumn") -eq 0 ]
    Table=$1
    Column=$2
    psql -t -d $Database -U $DbUser -c "\d $Table" \
	| grep "^ *$Column " | wc -l
    return
}

######################################################################
LatestDBVersion="db20220809"
Usage="updateschema.sh [ -h ] database"
######################################################################
DbUser="trove"
# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

if [ $# -lt 1 ]
then
    echo $Usage
    exit
else
    Database=$1
fi

if echo $0 | grep '/' >/dev/null
then
    ThisDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    ThisDir=$(pwd)
fi

Query="psql -q -d $Database -U $DbUser"
######################################################################
# Stuff that can't be handled purely in SQL:
if [ $(CountTable "deletedset") -eq 0 ]
then
    $Query <<EOF
CREATE TABLE deletedset
(
    foldername text NOT NULL,
    PRIMARY KEY(foldername)
);
GRANT SELECT, INSERT, UPDATE, DELETE ON deletedset TO "www-data";
EOF
fi

# GAPPKit upgrade: Added table dbversion.
if [ $(CountTable "dbversion") -eq 0 ]
then
    $Query <<EOF
CREATE TABLE dbversion
(
    dbversion text NOT NULL,
    PRIMARY KEY(dbversion)
);
GRANT SELECT, INSERT, UPDATE, DELETE ON dbversion TO "www-data";
EOF
fi

if [ $(IsColInTable "photo" "latitude") -eq 0 ]
then
    $Query <<EOF
UPDATE photo
 SET aperture = NULL, shutterspeed = NULL, focallength = NULL
 WHERE aperture IS NOT NULL OR shutterspeed IS NOT NULL
 OR focallength IS NOT NULL;
ALTER TABLE photo
 RENAME COLUMN aperture TO latitude;
ALTER TABLE photo
 RENAME COLUMN shutterspeed TO longitude;
ALTER TABLE photo
 RENAME COLUMN focallength TO altitude;
ALTER TABLE photo
 ALTER COLUMN latitude TYPE DOUBLE PRECISION
  USING latitude::double precision,
 ALTER COLUMN longitude TYPE DOUBLE PRECISION
  USING longitude::double precision,
 ALTER COLUMN altitude TYPE real
  USING altitude::real;
EOF
fi

if [ $(IsColInTable "tag" "last_used") -eq 0 ]
then
    $Query <<EOF
ALTER TABLE tag
 ADD last_used timestamp;
UPDATE tag SET last_used = CURRENT_DATE;
EOF
fi

CurrentDBVersion=$(psql -qt -d $Database -U $DbUser \
    -c "SELECT dbversion FROM dbversion" | sed -e 's/ //g')
if [ "z$CurrentDBVersion" != "z$LatestDBVersion" ]
then
    ##################################################################
    # Stuff that can be handled purely in SQL
    psql -d $Database -U $DbUser -f $ThisDir/updateschema.sql
    ##################################################################
    # Update to latest database version
    if [ "z$CurrentDBVersion" = "z" ]
    then
	echo Current database version is undefined
	$Query <<EOF
INSERT INTO dbversion(dbversion) VALUES ('$LatestDBVersion');
EOF
    else
	echo Current database version is $CurrentDBVersion
	$Query <<EOF
UPDATE dbversion SET dbversion = '$LatestDBVersion';
EOF
    fi
    echo Updated database to latest version, $LatestDBVersion.
else
    echo Database is already at latest version, $LatestDBVersion.
fi
