#!/bin/sh
######################################################################
# upgradeversion.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

upgradeversion.sh                Copyright 2021 Theodore B. Ruegsegger

Given an earlier version of Photo Album Manager, upgrades the web code
and, if needed, the database schema.

  * Accommodates service root other than the default
    /home/web/webroot-ssl/foto which is the service root for any
    instance installed by Easy Foto Box.

  * Accommodates custom code under web/local/ (leaving it to the user
    to reconcile). If you haven't customized any files under
    web/local/ then use the -l option to update them automatically.

  * Performs a dump of the database before updating it.

  * Leaves all photo images untouched.

  * Does no harm if code and/or database is already up-to-date.

  * Checks for missing supporting software and warns of any packages
    that you must install before running Photo Album Manager.

Usage:
    upgradeversion.sh [ -h ] [ -s serviceroot ]

Options:

    -b database
        Explicitly specify a different database to upgrade (mainly for
        testing). Default is foto which is the production database for
        any instance installed by Easy Foto Box.

    -d dumpdir
        Specify the full path to dump the database (default
        /home/dumps).

    -h
        Display a help message listing options.

    -l
	Overwrite code under web/local/ (Don't do this if you have
	customized any of those files!)

    -s serviceroot
	Explicitly specify full path of service root (mainly for
        testing). Default is /home/web/webroot-ssl/foto which is the
        service root for any instance installed by Easy Foto Box.

EOF
return  
}

HandleLegacyVideos()
{
    Videos=$(psql -t -d $Database -U foto \
	-c "SELECT foldername || '/' || vidfilename FROM video")
    NumVideos=$(echo $Videos | wc -w)
    echo Database $Database currently contains $NumVideos videos.
    UpdateGeometries
    NeedFlv=""
    for Video in $Videos
    do
	FolderName=$(echo $Video | cut -d '/' -f1)
	VidFileName=$(echo $Video | cut -d '/' -f2)
	Ext=$(echo $VidFileName | sed -e 's/^.*\.\([^.]*\)$/\1/')
	BaseName=$(echo $VidFileName| sed -e "s/\.$Ext\$//")
	FlvFile=${BaseName}.flv
	if [ ! -e webpics/$FolderName/$FlvFile ]
	then
	    NeedFlv="$NeedFlv $Video"
	fi
    done
    NumNeedFlv=$(echo $NeedFlv | wc -w)
    if [ $NumNeedFlv -gt 0 ]
    then
	echo '*******************************************************'
	T="$NumNeedFlv videos lack a flash (.flv) version to"
	T="$T play in browser."
	echo '* '$T
	T="You will need to log in to the Photo Album Manager"
	echo '* '$T
	T="and run \"Find and import existing video files\""
	echo '* '$T
	T="to generate the missing flash versions."
	echo '* '$T
	echo '*******************************************************'
    fi

}

UpdateGeometries()
{
    NeedGeometry=$(psql -t -d $Database -U foto <<EOF
SELECT foldername || '/' || vidfilename FROM video
WHERE width IS NULL OR height IS NULL
EOF
	)
    NumNeedGeometry=$(echo $NeedGeometry | wc -w)
    if [ $NumNeedGeometry -gt 0 ]
    then
	echo $NumNeedGeometry videos lack width and height data.
	for Video in $NeedGeometry
	do
	    FolderName=$(echo $Video | cut -d '/' -f1)
	    VidFileName=$(echo $Video | cut -d '/' -f2)
	    Geometry=$(mplayer -really-quiet -vo null -ao null \
		-identify -frames 0 $ServiceRoot/fullsize/$Video \
		2>/dev/null | grep 'ID_VIDEO' | grep 'WIDTH\|HEIGHT')
	    VWidth=$(echo $Geometry \
		| sed -e 's/^.*WIDTH=\([0-9]*\)[^0-9]*.*$/\1/')
	    VHeight=$(echo $Geometry \
		| sed -e 's/^.*HEIGHT=\([0-9]*\)[^0-9]*.*$/\1/')
	    UpdateGeometry $FolderName $VidFileName $VWidth $VHeight
	done
	echo Updated $NumNeedGeometry videos with width and height.
    fi
}

UpdateGeometry()
{
    psql -d $Database -U foto <<EOF
UPDATE video SET width = $3, height = $4
WHERE foldername = '$1' AND vidfilename = '$2'
EOF
}

######################################################################
Database=foto
DumpDir=/home/dumps
ReplaceLocal=
ServiceRoot=/home/web/webroot-ssl/foto
ObsoleteFiles="
app_imagemagick.php
"
NewPackages()
{
    echo "ffmpeg ffmpeg"
    echo "gm graphicsmagick"
    echo "mplayer mplayer"
}
Usage="upgradeversion.sh [ -h ] [ -s serviceroot ]"
######################################################################
# Parse command line options
while getopts "b:d:hls:" Option
do
    case $Option in
      b ) Database=$OPTARG
          ;;
      d ) DumpDir=$OPTARG
          ;;
      h ) PrintHelp
          exit
          ;;
      l ) ReplaceLocal=True
          ;;
      s ) ServiceRoot=$OPTARG
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

if echo $0 | grep '/' >/dev/null
then
    LocalDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    LocalDir=$(pwd)
fi

TopDir=$(cd $LocalDir ; cd ../.. ; pwd)
TopScriptDir=$TopDir/scripts
WebDir=$TopDir/web

cp $WebDir/*.* $ServiceRoot/
for ObsoleteFile in $ObsoleteFiles
do
    rm -f $ServiceRoot/$ObsoleteFile
done
cp $WebDir/images/*.* $ServiceRoot/images/
if [ ! -d $ServiceRoot/flowplayer ]
then
    mkdir $ServiceRoot/flowplayer
fi
cp $WebDir/flowplayer/*.* $ServiceRoot/flowplayer/
if [ $ReplaceLocal ]
then
    cp $WebDir/local/*.* $ServiceRoot/local/
else
    NewFiles=
    Files=$(cd $WebDir/local ; ls *.*)
    for File in $Files
    do
	if ! [ -e $ServiceRoot/local/$File ]
	then
	    NewFiles="$NewFiles $File"
	elif ! diff $WebDir/local/$File $ServiceRoot/local/$File \
	    > /dev/null
	then
	    NewFiles="$NewFiles $File"
	fi
    done
    if [ "z$NewFiles" != "z" ]
    then
	echo "You indicated you have modified files in the folder"
	echo "$ServiceRoot/local/ so they haven't been updated."
	echo "The following files have changed:"
	echo
	for File in $NewFiles
	do
	    echo $File
	done
	echo
	echo "Please compare the versions in $ServiceRoot/local/ with"
	echo "the new versions in $WebDir/local and update as needed."
	echo "If you haven't made local changes but simply omitted"
	echo "the -l option, it's safe to run this script again with"
	echo "the -l option."
    fi
fi
if [ -d $DumpDir -a -w $DumpDir ]
then
    $TopScriptDir/dumpdb.sh -zd $DumpDir $Database foto
    sh $LocalDir/updateschema.sh $Database
else
    echo "Can't dump database."
    echo "Be sure folder $DumpDir exists and is writeable."
    echo "Then run this script again."
    exit
fi

HandleLegacyVideos

# Check for missing supporting packages
Missing=
Names=
for Program in $(NewPackages | awk '{print $1}')
do
    if ! which $Program >/dev/null
    then
	Missing="$Missing $Program"
	Name=$(NewPackages | grep "^$Program " | awk '{print $2}')
	Names="$Names $Name"
    fi
done
if [ "$Missing" ]
then
    Count=$(echo $Missing | wc -w)
    if [ $Count -eq 1 ]
    then
	T="this missing package"
    else
	T="these missing packages"
    fi
    echo "Before running Photo Album Manager, you must use your"
    echo "package-management tools to install $T:"
    for Name in $Names
    do
	echo $Name
    done
fi

exit
