<?php
/*********************************************************************
add1pic.php
Add (or replace) a single uploaded image to a photoset.
______________________________________________________________________
Copyright 2024 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");
require ("lib_graphicsmagick.php");
require ("lib_filesys.php");
require ("lib_photoset.php");
require ("lib_tag.php");
require ("local/loc_camera.php");

$T = "Current time zone of this server (".$_SERVER["SERVER_NAME"];
$T .= "), equal to UTC".LocalTimeZoneOffset;
$T .= ". For phones and cameras that can't set the time zone.";
define ("HintTimeZoneLocal", $T);
$T = "Coordinated Universal Time, aka Greenwich Mean Time (GMT).";
$T .= " Recommended unless you never travel.";
define ("HintTimeZoneUTC", $T);

$T = "Specify a file to upload, no larger than ".UploadMaxFileSize;
$T .= " bytes and without embedded spaces or special characters in";
$T .= " its name.";
define ("FileUploadPrompt", $T);

$T = Strong("*")." indicates value copied from photo already";
$T .= " existing in target photo set.";
define("StatsNoteText", $T);
$T = Strong("*")." indicates value copied from video already";
$T .= " existing in target photo set.";
define("VidStatsNoteText", $T);

function AddPhotoToSet($FileName, $PhotoSetSn, $TimeText, $LocationSn,
		       $OtherLoc, $Description, $ThumbnailLength,
		       $WebPicLength, $ParentPhotoSn, $PicExists,
		       $Sort, $TimeZone)
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    $TimeStamp = TimeString(trim($TimeText));
    $LocationSn = FinalLocationSn($LocationSn, $OtherLoc);
    $Description = trim($Description);
    list($WebSize, $ThumbSize)
	= AddImages($FolderName, $FileName, $WebPicLength,
		    $ThumbnailLength);
    if ($PicExists)
    {
	$PhotoSn = UpdatePhoto($PhotoSetSn, $FileName, $LocationSn,
			       $TimeStamp, $Description, $WebSize,
			       $ThumbSize);
	$Msg = "Updated existing photo $FileName in";
    } else
    {
	$PhotoSn = AddPhoto($PhotoSetSn, $FileName, $LocationSn,
			    $TimeStamp, $Description, $WebSize,
			    $ThumbSize, $TimeZone);
	$Msg = "Added photo $FileName to";
    }
    if ($ParentPhotoSn != "none")
    {
	DerivePhoto($PhotoSn, $ParentPhotoSn);
	// Fetch attributes from actual photo in db, since they'll
	// have changed
	$H = ResHeritableAttributes($PhotoSn);
	list ($LocationSn, $Description, $TimeStamp)
	    = pg_fetch_array($H, 0);
	$TimeStamp = TimeString($TimeStamp);
    }
    $S = TableSummary($Msg, $PhotoSn, $FolderName, $TimeStamp,
		      $LocationSn, $Description, $WebSize,
		      $ThumbSize);
    $S .= SelectFile($PhotoSetSn, $Sort, "", $FileName);
    return $S;
}

function AddVideoToSet($FileName, $PhotoSetSn, $TimeText,
		       $LocationSn, $OtherLoc, $Description,
		       $VidExists, $Sort, $TimeZone)
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    $TimeStamp = TimeString(trim($TimeText));
    $LocationSn = FinalLocationSn($LocationSn, $OtherLoc);
    $Description = trim($Description);
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
    // Install original-format video and its Ogg (Theora/Vorbis)
    // version
    $NewFile = "$FullUploads/".LoggedInUser();
    list($VWidth, $VHeight) = array_values(VideoSpecs($NewFile));
    $NewOggFile = "$FullUploads/".OggFile(LoggedInUser().".zzz");
    $OggFile = OggFile($FileName);
    rename($NewFile, "$FullFolder/$FileName");
    rename($NewOggFile, "$WebFolder/$OggFile");
    if ($VidExists)
    {
	// Replacing existing video
	list($VideoSn, $StillFileName)
	    = SerialAndStillName($PhotoSetSn, $FileName);
	// Update geometry
	$Q = "UPDATE video SET width = $VWidth, height = $VHeight";
	$Q .= " WHERE videosn = $VideoSn";
	Query($Q);
	$Msg = "Updated existing video $FileName in";
	// Update still's metadata
	$WebSize = Geometry("$WebFolder/$StillFileName");
	$ThumbSize = Geometry("$ThumbFolder/$StillFileName");
	$StillSn = UpdatePhoto($PhotoSetSn, $StillFileName,
			       $LocationSn, $TimeStamp, $Description,
			       $WebSize, $ThumbSize);
    } else
    {
	// Adding new video; check database for a matching still image
	$StillFileName = VideoThumbnail($FileName, $PhotoSetSn);
	$StillSn = ($StillFileName === False)
	    ? ThumbnailFromOgg($PhotoSetSn, $FolderName, $OggFile,
			       $LocationSn, $TimeStamp, $Description,
			       $TimeZone)
	    : PhotoSerialNumber($StillFileName, $PhotoSetSn);
	// add vid to db with still
	$Q = "INSERT INTO video (vidfilename, stillsn, width,";
	$Q .= " height)";
	$Q .= " VALUES ($1, $StillSn, $VWidth, $VHeight)";
	Query($Q, $FileName);
	// if still's timestamp is null, extract from video
	$Q = "SELECT 1 FROM photo";
	$Q .= " WHERE photosn = $StillSn AND time IS NULL";
	if (pg_num_rows(Query($Q)) > 0)
	{
	    if (strtotime($TimeStamp) !== InvalidDateTime)
	    {
		if ($TimeZone == "UTC") $TimeStamp .= " UTC";
		$Q = "UPDATE photo SET time = $1";
		$Q .= " WHERE photosn = $StillSn";
		Query($Q, $TimeStamp);
	    }
	}
	$Msg = "Added video $FileName to";
    }
    $GeoData = GPSFromFile("$FullFolder/$FileName");
    if ($GeoData !== False) UpdatePhotoGeoData($GeoData, $StillSn);
    $S = TableVidSummary($Msg, $FolderName, $FileName, $StillSn,
			 $GeoData);
    $S .= SelectFile($PhotoSetSn, $Sort, "", $FileName);
    return $S;
}

function CompareFileAndSet($PhotoSetSn, $Sort, $TimeZone)
{
    $UserId = LoggedInUser();
    if ($_FILES["UserFile"]["error"] != UPLOAD_ERR_OK)
    {
	$T = "Upload failed.<br />".FileUploadPrompt;
	$S = SelectFile($PhotoSetSn, $Sort, $T);
    } else
    {
	list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
	if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
	$NewFile = "$FullUploads/$UserId";
	if (move_uploaded_file($_FILES["UserFile"]["tmp_name"],
			       $NewFile) and filesize($NewFile) > 0)
	{
	    $FileName = $_FILES["UserFile"]["name"];
	    $Finfo = finfo_open(FILEINFO_MIME_TYPE);
	    $MediaType = MediaType($NewFile, $Finfo);
	    finfo_close($Finfo);
	    switch ($MediaType)
	    {
	    case "Video":
		$S = PageVideoVsSet($PhotoSetSn, $FileName, $Sort,
				     $FullUploads, $UserId, $NewFile,
				     $TimeZone);
		break;
	    case "Image":
		$S = PagePhotoVsSet($PhotoSetSn, $FileName, $Sort,
				     $FullUploads, $UserId, $NewFile,
				     $TimeZone);
		break;
	    default:
		$FailTitle = "Add File &quot;$FileName&quot;";
		$S = StartOfPage($FailTitle);
		$T = "Uploaded file &quot;$FileName&quot; seems to";
		$T .= " be neither a video nor an image.";
		$T .= " Please try again.";
		$S .= StrWarning($T);
		$S .= StrWarning("Couldn't add file.");
		UnlinkIfExists($NewFile);
		$T = "Problem handling file. Upload failed.<br />";
		$S = SelectFile($PhotoSetSn, $Sort,
				$T.FileUploadPrompt);
	    }
	}
	else
	{
	    UnlinkIfExists($NewFile);
	    $T = "Problem handling file. Upload failed.<br />";
	    $S = SelectFile($PhotoSetSn, $Sort, $T.FileUploadPrompt);
	}
    }
    return $S;
}

function DerivePhoto($ChildPhotoSn, $ParentPhotoSn)
{
    // Link a "parent" photo with its derived "child", if not already
    $SoleParent = ! ResParents($ChildPhotoSn);
    $Q = "INSERT INTO derived_photo (childphotosn, parentphotosn)";
    $Q .= " SELECT $ChildPhotoSn, $ParentPhotoSn WHERE NOT EXISTS";
    $Q .= " (SELECT * FROM derived_photo";
    $Q .= " WHERE childphotosn = $ChildPhotoSn";
    $Q .= " AND parentphotosn = $ParentPhotoSn)";
    Query($Q);
    // If sole parent, child inherits metadata
    if ($SoleParent)
    {
	$P = HeritableAttributes($ParentPhotoSn);
	$C = HeritableAttributes($ChildPhotoSn);
	$Q = "";
	$Count = 0;
	$Params = array();
	foreach($P as $Key => $PValue)
	{
	    $CValue = $C[$Key];
	    if ((is_null($CValue) or $CValue == 0)
		and !is_null($PValue) and $PValue !== 0)
	    {
		$Count++;
		$Params[] = $PValue;
		$Q .= ", $Key = $$Count";
	    }
	}
	if ($Q > "")
	{
	    $Q = "UPDATE photo SET".substr($Q, 1);
	    $Q .= " WHERE photosn = $ChildPhotoSn";
	    QueryArray($Q, $Params);
	}
    }
    return;
}

function HeritableAttributes($PhotoSn)
{
    // Return associative array (locationsn, description, time,
    // metadata) for a photo
    return pg_fetch_array(
	ResHeritableAttributes($PhotoSn),0, PGSQL_ASSOC);
}

function ImageStats($PhotoSetSn, $FileName, $Info)
{
    // Get photo from DB if it exists
    $Q = "SELECT locationsn, description, width, height, thumbw,";
    $Q .= " thumbh, time";
    $Q .= " FROM photo";
    $Q .= " WHERE photosetsn = $PhotoSetSn AND filename = $1";
    $Res = Query($Q, $FileName);
    $PicExists = (pg_num_rows($Res) > 0);
    if ($PicExists)
    {
	list($LocSn, $Description, $Width0, $Height0, $ThumbW0,
	     $ThumbH0, $TimeStamp0)
	    = pg_fetch_array($Res, 0);
	$LocDescNote = "*";
	$WebPicLength = max($Width0, $Height0);
	$ThumbnailLength = max($ThumbW0, $ThumbH0);
	$SizeNote = "*";
	$ButtonLabel = "Overwrite existing image file $FileName";
    } else
    {
	$LocDescNote = "";
	$LocSn = Null;
	$Description = Null;
	$WebPicLength
	    = ScaledWebPicLength($Info["width"], $Info["height"]);
	$ThumbnailLength = ThumbnailLength;
	$SizeNote = "";
	$ButtonLabel = "Add new image file $FileName";
	$TimeStamp0 = False;
    }
    return array($PicExists, $LocDescNote, $LocSn, $Description,
		 $WebPicLength, $ThumbnailLength, $SizeNote,
		 $ButtonLabel, $TimeStamp0);
}

function MenuParents($ResParentPool, $Prompt="None, not derived")
{
    // Drop-down menu of other photos in photoset that might be
    // parents
    $S = HtmlSelectBegin("ParentPhotoSn");
    $S .= HtmlSelectOption("none", $Prompt, True);
    for ($Row = 0 ;  $Row < pg_num_rows($ResParentPool) ; $Row++)
    {
	List($ParentPhotoSn, $ParentFileName)
	    = pg_fetch_array($ResParentPool, $Row);
	$S .= HtmlSelectOption($ParentPhotoSn, $ParentFileName);
    }
    $S .= HtmlSelectEnd();
    return $S;
}

function PagePhotoVsSet($PhotoSetSn, $FileName, $Sort, $FullUploads,
			$UserId, $NewFile, $TimeZone)
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    $Info = ImageInfo($FullUploads, $UserId, $TimeZone);
    $HasStatsFromNew = (strtotime($Info["time"]) !== InvalidDateTime);
    $TimeString = ($HasStatsFromNew) ? $Info["time"] : Null;
    $StatsNote = "";
    list($PicExists, $LocDescNote, $LocSn, $Description,
         $WebPicLength, $ThumbnailLength, $SizeNote, $ButtonLabel,
         $TimeString0) = ImageStats($PhotoSetSn, $FileName, $Info);
    if ($PicExists and !$HasStatsFromNew)
    {
        $StatsNote = "*";
        $TimeString = $TimeString0;
    }
    $GeoData = (is_numeric($Info["latitude"]))
      ? array($Info["latitude"], $Info["longitude"], 
	      $Info["altitude"])
      : False;
    ClearExifRotation($NewFile);
    $Width = $Info["width"];
    $Height = $Info["height"];
    $WebPicLength = min($WebPicLength, max($Width, $Height));
    $S = StrSubTitle("New Photo to Add:");
    $S .= StrFormBeginRecurse();
    $S .= TableStats($FileName, $PicExists, $ThumbnailLength,
		     $WebPicLength, $StatsNote, $LocDescNote,
		     $SizeNote, $LocSn, $Description, $TimeString,
		     ResParentPool($PhotoSetSn, $FileName), $GeoData,
		     $NewFile);
    $S .= HtmlHidden("FileName", $FileName);
    $S .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $S .= HtmlHidden("PicExists", $PicExists);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("TimeZone", $TimeZone);
    $S .= "<br />\n";
    $T = HtmlSubmitButton("AddPhoto", $ButtonLabel);
    $T .= "&nbsp; &nbsp; &nbsp;";
    $T .= HtmlSubmitButton("Cancel", "Cancel");
    $S .= HtmlPara($T, "center");
    $S .= StrFormEnd();

    // Make webpic (but no larger than fullsize)
    $NewWebPic = "$FullUploads/{$UserId}-web";
    list($WidthW, $HeightW) =
	array_values(ResizePath($WebPicLength, $NewFile, $NewWebPic));
    $S .= HtmlImage(LatestFile($NewWebPic), "uploaded file", $HeightW,
		    $WidthW, "1", "uploaded file");
    $T = "Photos/<span style=\"background-color: ".HighlightBg."\">";
    $T .= "Videos</span> now in Photo Set $FolderName:";
    $S .= StrSubTitle($T);
    if ($PicExists)
    {
	// Display existing image at web size:
	list ($FullFolder, $WebFolder, $ThumbFolder)
	    = ImageFolder($FolderName);
	$S .= HtmlImage("$WebFolder/$FileName", "current $FileName",
			$Height0, $Width0, "1");
    }
    $S .= ThumbGallery($FileName, $PhotoSetSn, $FolderName, $Sort);
    return $S;
}

function PageVideoVsSet($PhotoSetSn, $FileName, $Sort, $FullUploads,
			$UserId, $NewFile, $TimeZone)
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    list($TimeStamp, $WithZone) = VideoTimeStamp($NewFile);
    if (strtotime($TimeStamp) !== InvalidDateTime
	and $TimeZone == "UTC and ! $WithZone")
	$TimeStamp .= " UTC";
    list ($VidExists, $StillFileName, $VWidth0, $VHeight0,
	  $Description, $TimeStamp0, $LocSn, $LocDescNote)
	= VideoStats($PhotoSetSn, $FileName, $ThumbFolder);
    if ($VidExists)
    {
	$StatsNote = "*";
	if ($TimeStamp == "") $TimeStamp = $TimeStamp0;
    } else $StatsNote = "";
    $S = StrSubTitle("New Video to Add:");
    $S .= StrFormBeginRecurse();
    $S .= TableVidStats($PhotoSetSn, $ThumbFolder, $FileName,
			$VidExists, $StillFileName, $StatsNote,
			$Description, $TimeStamp, $LocSn,
			$LocDescNote, $NewFile);
    $S .= HtmlHidden("FileName", $FileName);
    $S .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $S .= HtmlHidden("VidExists", $VidExists);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("TimeZone", $TimeZone);
    $S .= "<br />\n";
    $T = $VidExists ? "Overwrite existing" : "Add new";
    $T = HtmlSubmitButton("AddVideo", "$T video file $FileName");
    $T .= "&nbsp; &nbsp; &nbsp;";
    $T .= HtmlSubmitButton("Cancel", "Cancel");
    $S .= HtmlPara($T, "center");
    $S .= StrFormEnd();
    // Convert uploaded video to Ogg (Theora/Vorbis) and display
    $OggFile = "$FullUploads/".OggFile(LoggedInUser().".zzz");
    // Don't abort just because it takes a while
    set_time_limit (0);
    ConvertToOgg($NewFile, $OggFile);
    $S .= VideoBlock($OggFile);
    $T = "Photos/<span style=\"background-color: ".HighlightBg."\">";
    $T .= "Videos</span> now in Photo Set $FolderName:";
    $S .= StrSubTitle($T);
    if ($VidExists)
    {
	// Display Ogg theora version of existing video:
	$OggFile = "$WebFolder/".OggFile($FileName);
	$S .= VideoBlock($OggFile);
    }
    $S .= ThumbGallery($FileName, $PhotoSetSn, $FolderName, $Sort);
    return $S;
}

function ResParentPool($PhotoSetSn, $FileName)
{
    // List possible parents for a derived photo, that is, all others
    // in same photoset
    $Q = "SELECT photosn, filename FROM photo";
    $Q .= " WHERE photosetsn = $PhotoSetSn";
    $Q .= " AND filename != $1";
    $Q .= " ORDER BY filename";
    return Query($Q, $FileName);
}

function ResHeritableAttributes($PhotoSn)
{
    global $PhotoMetaData;
    // Return database result record (locationsn, description, time,
    // metadata) for a photo
    $Q = "SELECT locationsn, description, time";
    foreach ($PhotoMetaData as $Col)
    {
	$Q .= ", $Col";
    }
    $Q .= " FROM photo WHERE photosn = $PhotoSn";
    return Query($Q);
}

function ResParents($ChildPhotoSn)
{
    // Return all photos from which $ChildPhotoSn is derived
    // ($ChildPhotoSn's "parents")
    $Q = "SELECT foldername, filename";
    $Q .= " FROM derived_photo dp, photo p, photoset s";
    $Q .= " WHERE childphotosn = $ChildPhotoSn";
    $Q .= " AND photosn = parentphotosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY foldername, filename";
    $Res = Query($Q);
    return (pg_num_rows($Res) > 0) ? $Res : False;
}

function RowStats($Label, $Content)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong($Label));
    $S .= StrCell($Content);
    $S .= StrRowEnd();
    return $S;
}

function RowSummary($Label, $Content)
{
    $S = StrRowBegin();
    $S .= StrCell("$Label:", "2");
    $S .= StrCell($Content);
    $S .= StrRowEnd();
    return $S;
}

function RowSummary2($Label, $SubLabel1, $Content1, $SubLabel2,
		     $Content2)
{
    $S = StrRowBegin();
    $S .= StrCell($Label, "", "2");
    $S .= StrCell("$SubLabel1:");
    $S .= StrCell($Content1);
    $S .= StrRowEnd();
    $S .= StrCell("$SubLabel2:");
    $S .= StrCell($Content2);
    $S .= StrRowBegin();
    $S .= StrRowEnd();
    return $S;
}

function SelectFile($PhotoSetSn, $Sort, $Warning="", $FileName="")
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    $S = "";
    if ( $Warning !== "" ) $S .= StrWarning($Warning, WarningColor);
    $S .= BoxPhotoSetSelect($PhotoSetSn, $Sort);
    $T = ParsedHumanNum(UploadMaxFileSize, True);
    $UploadSizeLimit = ceil($T[1]);
    $S .= StrFormBeginPostUpload(ThisScript(), UploadMaxFileBytes);
    $S .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= TableSelectFile($FolderName);
    $T = "Photos/<span style=\"background-color: ".HighlightBg."\">";
    $T .= "Videos</span> now in Photo Set $FolderName:";
    $S .= StrSubTitle($T);
    $S .= ThumbGallery($FileName, $PhotoSetSn, $FolderName, $Sort);
    $S .= StrFormEnd();
    return $S;
}

function SerialAndStillName($PhotoSetSn, $FileName)
{
    // If $FileName exists in photo set $PhotoSetSn and is a video,
    // return array(video's serial number, filename of linked still
    // image), else array(False, False)
    $Q = "SELECT videosn, p.filename AS stillfilename";
    $Q .= " FROM video v, photo p";
    $Q .= " WHERE p.photosetsn = $PhotoSetSn AND stillsn = photosn";
    $Q .= " AND vidfilename = $1";
    $Res = Query($Q, $FileName);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_array($Res, 0) : array(False, False);
}

function TableSelectFile($FolderName)
{
    $S = StrTableBegin(0, "", "", BoxStyle);
    $S .= StrRowBegin();
    $T = "Select photo or video (max ".UploadMaxFileSize." bytes)";
    $T .= " to add to photo set &quot;$FolderName&quot;:<br>\n";
    $T .= HtmlInputPostUpload("UserFile", 40);
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell("Camera's time is set to:");
    $T = "<label title=\"".HintTimeZoneUTC."\">";
    $T .= HtmlRadioButton("TimeZone", "UTC", True);
    $T .= "UTC</label> <label title=\"".HintTimeZoneLocal."\">";
    $T .= HtmlRadioButton("TimeZone", "Local");
    $T .= LocalTimeZone."</label>";
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("Upload", "Upload File"), "2", "",
		  "text-align: center");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function TableStats($FileName, $PicExists, $ThumbnailLength,
		    $WebPicLength, $StatsNote, $LocDescNote,
		    $SizeNote, $LocSn, $Description, $TimeString,
		    $ResParentPool, $GeoData, $NewFile)
{
    $S = StrTableBegin("0", "", "", BoxStyle);
    $C = $FileName;
    if ($PicExists)
	$C .= Strong(HtmlColor(WarningColor,
			       " (name exists in target photo set)"));
    $S .= RowStats("Filename:", $C);
    $S .= RowStats("${StatsNote}When taken:",
		   HtmlTextInput("TimeText", 25,
				 TimeString($TimeString,
					    DateTimeFormatFull, "")));

    $T = MenuLocation($LocSn);
    if ($GeoData !== False) $T .= "<br />".GeoLocation($GeoData);
    $S .= RowStats("${LocDescNote}Where taken:", $T);

    
    
    /* $S .= RowStats("${LocDescNote}Where taken:", */
    /* 		   MenuLocation($LocSn)); */
    $S .= RowStats("If &quot;other&quot;, specify:",
		   HtmlTextInput("OtherLoc", "35"));
    
    $S .= RowStats("${LocDescNote}Description:",
		   HtmlTextArea ("Description", 40, 3, $Description));
    $C = "Thumbnails ";
    $C .= HtmlTextInput("ThumbnailLength", 4, $ThumbnailLength);
    $C .= " Large images ";
    $C .= HtmlTextInput("WebPicLength", 4, $WebPicLength);
    $S .= RowStats("${SizeNote}Image length (pixels):", $C);
    $S .= RowStats("Parent photo, if any",
		   MenuParents($ResParentPool));
    if ($StatsNote.$LocDescNote.$SizeNote > "")
    {
	$S .= StrRowBegin();
	$S .= StrCell(StatsNoteText, "3");
	$S .= StrRowEnd();
    }
    $S .= StrTableEnd();
    return $S;
}

function TableSummary($Msg, $PhotoSn, $FolderName, $TimeStamp,
		      $LocationSn, $Description, $WebSize, $ThumbSize)
{
    $S = "<div style=\"border: solid ".NoticeColor."; padding: 2mm;";
    $S .= " background-color: ".TableColor."; display: table-cell;";
    $S .= " margin-bottom: 2mm\">\n";
    $S .= HtmlPara("$Msg photo set $FolderName with these values:");
    $S .= StrTableBegin("0");
    $S .= RowSummary("When taken", $TimeStamp);
    $T = Location($LocationSn);
    $G = GeoCoordinates($PhotoSn);
    if ($G !== False) $T .= "<br />".$G;
    $S .= RowSummary("Where taken", $T);
    $S .= RowSummary("Description", $Description);
    $S .= RowSummary2("Web-size image", "width", $WebSize["width"],
		      "height", $WebSize["height"]);
    $S .= RowSummary2("Thumbnail", "width", $ThumbSize["width"],
		      "height", $ThumbSize["height"]);
    $Res = ResParents($PhotoSn);
    if ($Res)
    {
	$T = "";
	$NumRows = pg_num_rows($Res);
	for ($Row = 0 ; $Row < $NumRows ; $Row++)
	{
	    list($FolderName, $FileName) = pg_fetch_array($Res, $Row);
	    $T .= "<br />\n$FolderName/$FileName";
	}
	$S .= RowSummary("Derives from", substr($T, 7));
    }
    $Res = ResDerivedPhotos($PhotoSn);
    if ($Res)
    {
	$T = "";
	$NumRows = pg_num_rows($Res);
	for ($Row = 0 ; $Row < $NumRows ; $Row++)
	{
	    list($FolderName, $FileName) = pg_fetch_array($Res, $Row);
	    $T .= "<br />\n$FolderName/$FileName";
	}
	$S .= RowSummary("Has derivatives", substr($T, 7));
    }
    $S .= StrTableEnd();
    $S .= "</div><br />\n";
    return $S;
}

function TableVidStats($PhotoSetSn, $ThumbFolder, $FileName,
		       $VidExists, $StillFileName, $StatsNote,
		       $Description, $TimeStamp, $LocSn,
		       $LocDescNote, $NewFile)
{
    $S = StrTableBegin("1", TableColor);
    $C = $FileName;
    if ($VidExists)
	$C .= Strong(HtmlColor(WarningColor,
			       " (name exists in target photo set)"));
    $S .= RowStats("Filename:", $C);
    if ($StillFileName)
    {
	$C = $StillFileName;
	$F = "*";
    } else
    {
	$C = VideoThumbnail($FileName, $PhotoSetSn);
	if (! $C)
	{
	    $C = FileNameBase($FileName);
	    $C .= ".jpg [ extracted automatically ]";
	}
	$F = "";
    }
    $S .= RowStats("{$F}Still Image:", $C);
    $S .= RowStats("${StatsNote}When taken:",
		   HtmlTextInput("TimeText", 25,
				 TimeString($TimeStamp)));
    $T = MenuLocation($LocSn);
    $GeoData = GPSFromFile($NewFile);
    if ($GeoData !== False) $T .= "<br />".GeoLocation($GeoData);
    $S .= RowStats("${LocDescNote}Where taken:", $T);
    $S .= RowStats("If &quot;other&quot;, specify:",
		   HtmlTextInput("OtherLoc", "35"));
    $S .= RowStats("${LocDescNote}Description:",
		   HtmlTextArea ("Description", 40, 3, $Description));
    if ($StatsNote.$LocDescNote > "")
    {
	$S .= StrRowBegin();
	$S .= StrCell(VidStatsNoteText, "3");
	$S .= StrRowEnd();
    }
    $S .= StrTableEnd();
    return $S;
}

function TableVidSummary($Msg, $FolderName, $FileName, $StillSn,
			 $GeoData)
{
    $Q = "SELECT description, width, height, thumbw, thumbh, time,";
    $Q .= " filename, l.name AS locname";
    $Q .= " FROM photo p LEFT OUTER JOIN location l";
    $Q .= " ON (l.locationsn = p.locationsn)";
    $Q .= " WHERE photosn = $StillSn";
    list($Description, $Width, $Height, $Thumbw, $Thumbh, $TimeStamp,
	 $StillFileName, $LocName) = pg_fetch_array(Query($Q), 0);
    $S = "<div style=\"border: solid ".NoticeColor."; padding: 2mm;";
    $S .= " background-color: ".TableColor."; display: table-cell;";
    $S .= " margin-bottom: 2mm\">\n";
    $T = "$Msg photo set $FolderName with still image $StillFileName";
    $T .= " and these values:";
    $S .= HtmlPara($T);
    $S .= StrTableBegin("0");
    $S .= RowSummary("When taken", TimeString($TimeStamp));
    if ($GeoData !== False)
	$LocName .= "<br />".GeoLocation($GeoData);
    $S .= RowSummary("Where taken", $LocName);
    $S .= RowSummary("Description", $Description);
    $S .= StrTableEnd();
    $S .= "</div><br />\n";
    return $S;
}

function ThumbGallery($NewFileName, $PhotoSetSn, $FolderName, $Sort)
{
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $Res = ResPhotosInSet($PhotoSetSn, $Sort);
    $NumPics = pg_num_rows($Res);
    $Imgs = "";
    for ($Row = 0 ; $Row < $NumPics ; $Row++)
    {
	list($PhotoSn, $Z1, $FileName, $Z2, $Description, $Z3, $Z4,
	     $ThumbW, $ThumbH, $Z5, $Z6, $VidFileName)
	    = pg_fetch_array($Res, $Row);
	$T = $FileName;
	$T .= ($Description > "") ? (": $Description") : "";
	$I = "<img src=\"$ThumbFolder/$FileName\" height=\"$ThumbH\"";
	$I .= " width=\"$ThumbW\"";
	if ($FileName== $NewFileName)
	{
	    $T = "current $T";
	    $Color = WarningColor;
	} elseif ($VidFileName > "" )
	{
	    if ($VidFileName == $NewFileName)
	    {
		$Color = WarningColor;
		$T = "current $T";
	    } else $Color =  HighlightBg;
	    $T .= " (Video)";
	} else $Color = BgColor;
	$T = HtmlCleanTitle($T);
	$I .= " style=\"border: thick solid $Color\" title=\"$T\" />";
	$Imgs .= $I;
    }
    return "<p style=\"text-align: left\">$Imgs</p>\n";
}

function ThumbnailFromOgg($PhotoSetSn, $FolderName, $OggFile,
			  $LocationSn, $TimeStamp, $Description,
			  $TimeZone)
{
    /* Create still image for video and return its serial number */
    // Extract frame from Ogg file as thumbnail
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $Thumb = FileNameBase($OggFile).".jpg";
    $Cmd = "oggThumb -t 0 -n $FullFolder/$Thumb $WebFolder/$OggFile";
    exec("$Cmd >/dev/null 2>&1");
    list($Width, $Height)
	= array_values(Geometry("$FullFolder/$Thumb"));
    // Make webpic
    $WebSize = Resize(min(WebPicLength, max($Width, $Height)),
		      $Thumb, $FullFolder, $WebFolder);
    // Make thumbnail
    $ThumbSize = Resize(min(ThumbnailLength, max($Width, $Height)),
			$Thumb, $FullFolder, $ThumbFolder);
    // Add to db and return photo serial no
    return AddPhoto($PhotoSetSn, $Thumb, $LocationSn, $TimeStamp,
		    $Description, $WebSize, $ThumbSize, $TimeZone);
}

function UpdatePhoto($PhotoSetSn, $FileName, $LocationSn, $TimeStamp,
		     $Description, $WebSize, $ThumbSize)
{
    // Update existing photo from new image, return photosn
    global $PhotoMetaData;
    $Description = trim($Description);
    $Count = 6;
    $Params = array(
	$PhotoSetSn, $FileName, $WebSize["width"], $WebSize["height"],
	$ThumbSize["width"], $ThumbSize["height"]);
    $C = "width = $3, height = $4, thumbw = $5, thumbh = $6";
    if ($LocationSn !== False)
    {
	$Count++;
	$Params[] = $LocationSn;
	$C .= ", locationsn = $$Count";
    }
    if ($TimeStamp !== NullTimeFlag)
    {
	$Count++;
	$Params[] =$TimeStamp ;
	$C .= ", time = $$Count";
    }
    if ($Description != '')
    {
	$Count++;
	$Params[] = $Description;
	$C .= ", description = $$Count";
    }
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $Info = ImageInfo($FullFolder, $FileName, $TimeZone);
    foreach ($PhotoMetaData as $Key)
    {
        if ($Info[$Key] != "")
        {
            $Count++;
            $Params[] = $Info[$Key];
            $C .= ", $Key = $$Count";
        }
    }
    $Q = "UPDATE photo SET $C";
    $Q .= " WHERE photosetsn = $1 AND filename = $2";
    QueryArray($Q, $Params);
    return PhotoSerialNumber($FileName, $PhotoSetSn);
}

function VideoStats($PhotoSetSn, $FileName, $ThumbFolder)
{
    // Get video from DB if it exists
    $Q = "SELECT p.filename AS stillfilename, description, time,";
    $Q .= " locationsn, v.width as vwidth, v.height as vheight";
    $Q .= " FROM video v, photo p";
    $Q .= " WHERE p.photosetsn = $PhotoSetSn";
    $Q .= " AND vidfilename = $1 AND p.photosn = v.stillsn";
    $Res = Query($Q, $FileName);
    $VidExists = (pg_num_rows($Res) > 0);
    if ($VidExists)
    {
	list($StillFileName, $Description, $TimeStamp, $LocSn,
	     $VWidth, $VHeight) = pg_fetch_array($Res, 0);
	$LocDescNote = "*";
    } else
    {
	$LocDescNote = "";
	$VWidth = Null;
	$VHeight = Null;
	// Check in case the still image is already in photo set
	$StillFileName = VideoThumbnail($FileName, $PhotoSetSn);
	if ($StillFileName)
	{
	    $Q = "SELECT locationsn, description, time FROM photo";
	    $Q .= " WHERE photosetsn = $PhotoSetSn";
	    $Q .= " AND filename = $1";
	    list ($LocSn, $Description, $TimeStamp)
		= pg_fetch_array(Query($Q, $StillFileName), 0);
	} else
	{
	    $LocSn = Null;
	    $Description = Null;
	    $TimeStamp = False;
	}
    }
    return array($VidExists, $StillFileName, $VWidth, $VHeight,
		 $Description, $TimeStamp, $LocSn, $LocDescNote);
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "Description",
    "FileName",
    "FolderName",
    "LocationSn",
    "OtherLoc",
    "ParentPhotoSn",
    "PhotoSetSn",
    "PicExists",
    "Sort",
    "ThumbnailLength",
    "TimeText",
    "TimeZone",
    "VidExists",
    "WebPicLength"
    );

$FromBefore = array(
    "AddPhoto"
    );
$PostVarList = array(
//    "FolderName",
    "MAX_FILE_SIZE",
    "PhotoSetSn",
    "Sort",
    "TimeZone",
    "UserFile"
   );

foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S = StartOfPage("Add a Photo or Video to an Existing Photo Set");

if (!isset($Sort)) $Sort = "Time";
if (!isset($PhotoSetSn) or $PhotoSetSn == "none")
    $S .= BoxPhotoSetSelect("", $Sort);
elseif (isset($_POST["Upload"]))
    $S .= CompareFileAndSet($PhotoSetSn, $Sort, $TimeZone);
elseif (isset($FormVars["AddPhoto"]))
    $S .= AddPhotoToSet($FileName, $PhotoSetSn, $TimeText,
			$LocationSn, $OtherLoc, $Description,
			$ThumbnailLength, $WebPicLength,
			$ParentPhotoSn, $PicExists, $Sort, $TimeZone);
elseif (isset($FormVars["AddVideo"]))
    $S .= AddVideoToSet($FileName, $PhotoSetSn, $TimeText,
			$LocationSn, $OtherLoc, $Description,
			$VidExists, $Sort, $TimeZone);
else
{
    if (! isset($FileName)) $FileName = "";
    $S .= SelectFile($PhotoSetSn, $Sort, "", $FileName);
}
$S .= "</div></body></html>\n";
echo $S;
?>
