<?php
/*********************************************************************
applydates.php
Apply Dates Module for PhotoTrove
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require ("lib_inputval.php");
require_once ("lib_layout.php");
require ("lib_tag.php");
require ("lib_photoset.php");
require ("gk/gk_jscalendar.php");
require_once ("local/loc_camera.php");

$T = "Sometimes photos have no timestamps, for example, digital";
$T .= " scans of slides, negatives or prints. In such cases, the";
$T .= " approximate date may be known, though not the precise time.";
$T .= " This tool applies a date (with time 00:00:00";
$T .= " local) to selected photos.<br style=\"margin-bottom: 1em\"/>";
$T .= "First, identify all photos for which you want to set the";
$T .= " dates. Use the &quot;Tag selected photos&quot; feature on";
$T .= " the Search or Edit Photo Set pages to assign a particular";
$T .= " tag to those photos. On this page, select that tag to";
$T .= " display them all.<br style=\"margin-bottom: 1em\"/>";
$T .= "For each date you wish to set, select the individual photos";
$T .= " to which it applies. Then use the calendar to specify a";
$T .= " date, or enter the date as text, and Apply.";
$T .= "<br style=\"margin-bottom: 1em\"/>";
$T .= "The date will be set for each selected photo and the tag";
$T .= " removed.";
define("Introduction", $T);
define("IntroStyle", BoxStyle."; width: 50%; text-align: left");

function AppliedDateUntagged($NumRows, $Date, $TagSn)
{
    $S = "";
    $Count = 0;
    if (strtotime($Date) !== InvalidDateTime)
    {
	$FormVars = (FormMethod == "get") ? $_GET : $_POST;
	for ($Row=0 ; $Row < $NumRows ; $Row++ )
	{
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
	    {
            Query("BEGIN");
            $PhotoSn = $FormVars["PS$Row"];
            $Q = "UPDATE photo SET time = $2";
            $Q .= " WHERE photosn = $1";
            Query($Q, $PhotoSn, $Date);
            UnTagPhoto($TagSn, $PhotoSn);
            Query("COMMIT");
            $S .= ", ".PhotoPointer($PhotoSn);
            $Count ++;
	    }
	}
	if ($S > "")
	{
	    $S = substr($S, 1);
	    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
	    $T = "Applied date &quot;$Date&quot; to $Count photos:";
	    $S = "$T.$S<br />and removed tag &quot;$TagName&quot;.";
	} else
	{
	    $S = "No photos selected; couldn't apply date";
	    $S .= " &quot;$Date&quot;.";
	}
    } else
    {	$S = "Invalid date &quot;$Date&quot;; couldn't apply it to";
	$S .= " selected photos.";
    }
    return HtmlPara($S);
}

function BoxTagSelect($TagSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SearchByTag", True);
    $S .= StrTableBegin("0", "", "", BoxStyle);
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Select tag:&nbsp;"));
    $S .= StrCell(MenuTag($TagSn, "TagSn", True, False));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function DateActionOptions($Date)
{
    // Vertical array of actions that may be applied to selected
    // photos in thumbnail gallery
    $S = StrTableBegin("0");
    $S .= OptionSetDate($Date);
    $S .= StrTableEnd();
    return $S;
}

function OptionSetDate($Date)
{
    $S = StrRowBegin();
    $S .= StrCell("Assign this date<br />\nto selected photos:");
    $C = DateInput("Date", 10, StrDate($Date, PubDateFormat),
		   "", False, CalPubDateFormat);
    $C .= HtmlSubmitButton("ApplyDate", "Apply");
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function PageTaggedPhotos($TagSn, $Date, $SelectAll=False,
			  &$RangeMarks=False)
{
    /*****************************************************************
    Displays page of tagged photos with widgets and photo thumbnails.
    Calls ImageCells to display thumbnails with checkboxes set
    according to $SelectAll and $RangeMarks.
    *****************************************************************/
    $Res = ResTaggedPhotos($TagSn);
    $NumRows = pg_num_rows($Res);
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $Plural = ($NumRows > 1) ? "s" : "";
    $KeyDescription = " tagged with &quot;$TagName&quot;";
    $N = "Found $NumRows photo$Plural$KeyDescription.";
    if (! $Res or $NumRows == 0)
	$S = StrNoticeBox("Found no photos$KeyDescription.");
    else
    {
/*
Leave this stuff in in case too many hits is a problem
	if (is_numeric($Limit) && $NumRows > $Limit)
	{
	    $S = WarnTooManyHits(
		$WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
		$InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
		$TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
		$DescIs, $TimeIs, $Sort, $N);
	    $EndsOnly = True;
	} else
	{
	    $T = ($Sort == "Time") ? "time" : "place and then time";
	    if ($NumRows > 1) $N .= "<br />Sorting by $T.";
	    $S = StrNoticeBox($N);
	}
*/
	$S = StrNoticeBox("$N<br />Sorting by folder and filename.");

	$State = PackedState(array($TagSn));
	$S .= StrFormBeginRecurse();
	$S .= HtmlHidden("State",
			 htmlentities($State));
	$S .= HtmlHidden("NumRows", $NumRows);
/*
Leave this stuff in in case too many hits is a problem
	if ($EndsOnly)
	{
	    $W = "Careful: All $NumRows matching photos are";
	    $W .= " selected!";
	    $S .= StrWarningBox($W, "50%");
	} else
*/
	$S .= StrSelectionOptions();
	$S .= DateActionOptions($Date);

	$S .= "<br>\n";
/*
Leave this stuff in in case too many hits is a problem
	if ($EndsOnly) $S .= EndsOnly($Res, $NumRows);
	else
	{
*/
	    if ($SelectAll) $RangeMarks = True;
	    $S .= ImageCells($Res, $NumRows, $RangeMarks, False,
			     False, True);
/*
Leave this stuff in in case too many hits is a problem
	}
*/
	$S .= StrFormEnd();
    }
    return $S;
}

function ResTaggedPhotos($TagSn)
{
    $Q = "SELECT tp.photosn, foldername, filename, p.locationsn,";
    $Q .= " description, p.width, p.height, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay;
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " vidfilename, 1, 1, 1, TRUE as istagged";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY lower(foldername || filename)";
    return Query($Q, $TagSn);
}

function SelectAll($Date, $State, $Selected)
{
    /*****************************************************************
    Sets or clears selection checkboxes on all images according to
    $SelectAll, then redisplays search results page.
    *****************************************************************/
    list ($TagSn) = UnPackedState($State);
    return PageTaggedPhotos($TagSn, $Date, $Selected);
}

function SelectRange($Date, $State, $NumThumbs, $FormVars)
{
    /*****************************************************************
    If $NumThumbs > 0, builds an ordered array of selected thumbnails
    which specify starting and stopping points of ranges, then calls
    PageTaggedPhotos to redisplay photo set page with checkboxes set
    for all images in range(s).
    *****************************************************************/
    $RangeMarks = array();
    for ($Row=0 ; $Row < $NumThumbs ; $Row++ )
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
            $RangeMarks[] = $FormVars["PS$Row"];
    if (count($RangeMarks) > 0)
    {
	list ($TagSn) = UnPackedState($State);
	$T= PageTaggedPhotos($TagSn, $Date, False, $RangeMarks);
	$S = HtmlPara($RangeMarks).$T;
    } else
   {
	$S = HtmlPara("No selection range specified.");
	$S .= SelectAll($Date, $State, False);
    }
    return $S;
}

//********************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "Date",
    "NumRows",
    "SelectAll",
    "State",
    "TagSn"
    );
$GetVarList = array(
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (! isset($Date)) $Date = False;
if (! isset($TagSn)) $TagSn = False;
$S = StartOfPage("Apply Dates to Photos", CalendarDefinitions);
if (isset($State)) list ($TagSn) = UnPackedState($State);
if (isset($TagSn) and ! is_numeric($TagSn))
    $S .= StrWarning("Select a tag");
$S .= BoxTagSelect($TagSn);
if (is_numeric($TagSn))
{
    if (isset($SelectAll) or isset($FormVars["ClearAll"]))
        $S .= SelectAll($Date, $State, isset($SelectAll));
    elseif (isset($FormVars["SelectRange"]))
        $S .= SelectRange($Date, $State, $NumRows, $FormVars);
    elseif (isset($FormVars["ApplyDate"]))
    {
        $S .= AppliedDateUntagged($NumRows, $Date, $TagSn);
        $S .= SelectAll($Date, $State, False);
    } else
          $S .= PageTaggedPhotos($TagSn, $Date);
}
else $S .= "<div style=\"".IntroStyle."\">".Introduction."</div>\n";
$S .= "</div></body></html>\n";
echo $S;

?>
