<?php
/*********************************************************************
capturegps.php
Capture Geo-Data Module for PhotoTrove
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require ("lib_inputval.php");
require_once ("lib_layout.php");
require ("lib_tag.php");
require ("lib_photoset.php");
require ("local/loc_camera.php");

$T = "Many cameras and smart phones now use the Global Positioning";
$T .= " System (GPS) to tag photos and videos with latitude,";
$T .= " longitude, and altitude. Beginning February 2019,";
$T .= " PhotoTrove captures this information from any photos or";
$T .= " videos newly added to the collection, records it in the";
$T .= " database, and displays it along with the named location.";
$T .= "<br style=\"margin-bottom: 1em\"/>\n";
$T .= "This tool captures GPS information for photos and videos";
$T .= " already in the collection that lack it, probably because";
$T .= " they were added before PhotoTrove did it automatically.";
$T .= " It accomplishes this by examining the metadata in the";
$T .= " actual image or video; this can take some time for a large";
$T .= " number of them.";
$T .= "<br style=\"margin-bottom: 1em\"/>\n";
$T .= "To specify which photos to examine, use the &quot;Tag";
$T .= " selected photos&quot; feature on the Search or Edit Photo";
$T .= " Set pages to assign a particular tag to all the photos in";
$T .= " question, and only to those, then select that tag on this";
$T .= " page.";
define("Introduction", $T);
define("IntroStyle", BoxStyle."; width: 50%; text-align: left");
define ("IntroBlock",
	"<div style=\"".IntroStyle."\">".Introduction."</div>\n");

define ("MaxCellChars", 20);

function BoxTagSelect($TagSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SearchByTag", True);
    $S .= StrTableBegin("0", "", "", BoxStyle);
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Select tag:&nbsp;"));
    $S .= StrCell(MenuTag($TagSn, "TagSn", True, False));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function CaptureGeoData($TagSn)
{
    $Res = ResTaggedPicsToUpdate($TagSn);
    $Tuples = pg_num_rows($Res);
    $CaptureCount = 0;
    $TimeStart = time();
    if ($Tuples > 0)
    {
    	for ($Tuple = 0 ; $Tuple < $Tuples ; $Tuple++)
	{
	    list($PhotoSn, $FolderName, $PhotoFileName, $VidFileName)
		= pg_fetch_array($Res, $Tuple);
	    list ($FullFolder, $WebFolder, $ThumbFolder)
		= ImageFolder($FolderName);
 	    $Path = ($VidFileName > "")
		? "$FullFolder/$VidFileName"
		: "$FullFolder/$PhotoFileName";
	    $GeoData = GPSFromFile($Path);
	    if ($GeoData !== False)
	    {
		UpdatePhotoGeoData($GeoData, $PhotoSn);
		$CaptureCount ++;
	    }
	}
    }
    $TimeFinish = time();
    $T = "Captured Geo-data for $CaptureCount photos/videos in ";
    $T .= HmsTime($TimeFinish - $TimeStart).".";
    $S = StrWarning($T, NoticeColor);
    return $S.PageTaggedPhotos($TagSn, True);
}

function CellFilePath($PhotoSn, $FolderName, $FileName, $VidFileName)
{
    if ($VidFileName > "")
    {
	$FileName = $VidFileName;
	$Style = "background: ".HighlightBg;
    } else $Style = "";
    $T = "<a href=\"".ImageUrl($PhotoSn)."\" target=\"_blank\">";
    $T .= "$FolderName/$FileName</a>";
    return StrCell($T, "", "", $Style);
}

function CellGeolocation($Latitude, $Longitude, $Altitude)
{
    $T = (is_numeric($Latitude))
	? GeoLocation(array($Latitude, $Longitude, $Altitude))
	: "[unknown]";
    return StrCell($T);
}

function CellLimitedLength($Field)
{
    if (strlen($Field) > MaxCellChars)
	$Field =
	    substr($Field, 0, MaxCellChars - 1)."&hellip;";
    return StrCell($Field);
}

function FormCaptureButton($TagSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $T = "Capture geo-data for tagged photos";
    $S .= HtmlSubmitButton("Capture", $T);
    $S .= StrFormEnd();
    return $S;
}

function FormUntag($TagSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= HtmlSubmitButton("Untag", "Untag All");
    $S .= StrFormEnd();
    return $S;
}

function HeadingPhotoCount($NumHits, $Res, $TagSn, $CaptureComplete)
{
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    if ($NumHits > 0)
    {
	$First = pg_fetch_result($Res, 0, "timestamp");
	$Plural = ($NumHits > 1) ? "s" : "";
	$T = "$NumHits photo$Plural tagged &quot;$TagName&quot;";
	if ($NumHits > 1)
	{
	    $Last = pg_fetch_result($Res, $NumHits-1, "timestamp");
	    $T .= " between $First and $Last";
	} else
	{
	    $T .= " at $First:";
	}
	$S = StrCell(Strong($T), 3);
	$T = $CaptureComplete
	    ? FormUntag($TagSn)
	    : FormCaptureButton($TagSn);
	$S .= StrCell($T, 2, "", "padding: 0.5ex");
    } else
    {
	$T = "No photos tagged &quot;$TagName&quot;";
	$S = StrCell(Strong($T), 5);
    }
    return $S;
}

function PageTaggedPhotos($TagSn, $CaptureComplete=False)
{
    $Res = ResTaggedPhotos($TagSn);
    $Tuples = pg_num_rows($Res);
    $S = StrTableBegin("1", TableColor, "", "border: thin solid");
    $S .= StrRowBegin();
    $S .= HeadingPhotoCount($Tuples, $Res, $TagSn, $CaptureComplete);
    $S .= StrRowEnd();
    if ($Tuples > 0)
    {
    	$S .= TaggedPhotoHeaders();
    	for ($Tuple = 0 ; $Tuple < $Tuples ; $Tuple++)
    	    $S .= TaggedPhotoRow($Res, $Tuple);
    }
    $S .= StrTableEnd();
    return $S;
}
    
function ResTaggedPhotos($TagSn)
{
    $Q = " SELECT tp.photosn, foldername, filename, description, ";
    $Q .= ClauseDateForDisplay." AS timestamp";
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " v.vidfilename, latitude, longitude, altitude";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $TagSn);

}

function ResTaggedPicsToUpdate($TagSn)
{
    $Q = " SELECT tp.photosn, foldername, filename, v.vidfilename";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " AND latitude IS NULL";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $TagSn);
}

function TaggedPhotoHeaders()
{
    $Headers = array(
	"File",
	"Date",
	"Description",
	"Location",
	"Geolocation"
	);
    $S = StrRowBegin();
    foreach ($Headers as $Header)
    {
	$S .= StrHeaderCell($Header);
    }
    $S .= StrRowEnd();
    return $S;
}

function TaggedPhotoRow($Res, $Tuple)
{
    list($PhotoSn, $FolderName, $FileName, $Description, $TimeStamp,
	 $LocName, $VidFileName, $Latitude, $Longitude, $Altitude)
	= pg_fetch_array($Res, $Tuple);
    $S = StrRowBegin();
    $S .= CellFilePath($PhotoSn, $FolderName, $FileName,
		       $VidFileName);
    $S .= StrCell($TimeStamp);
    $S .= CellLimitedLength(HtmlCleanTitle($Description));
    $S .= CellLimitedLength($LocName);
    $S .= CellGeolocation($Latitude, $Longitude, $Altitude);
    $S .= StrRowEnd();
    return $S;
}

function UntaggedAll($TagSn)
{
    $Q = "DELETE FROM tag_photo WHERE tagsn = $1";
    Query($Q, $TagSn);
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $S = "Removed tag &quot;$TagName&quot; from all photos.";
    return StrWarning($S, NoticeColor);
}

function MainPage($TagSn)
{
    $S = BoxTagSelect($TagSn);
    if (is_numeric($TagSn))
	$S .= PageTaggedPhotos($TagSn);
    else
	$S .= IntroBlock;
    return $S;
}

//********************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "TagSn"
    );
$GetVarList = array(
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S = StartOfPage("Capture Geo-Data");
if (isset($FormVars["Capture"]))
{
    $S .= CaptureGeoData($TagSn);
} elseif (isset($FormVars["Untag"]))
{
    $S .= UntaggedAll($TagSn);
    $S .= MainPage($TagSn);
} elseif (isset($TagSn) and ! is_numeric($TagSn))
{
    $S .= StrWarning("Select a tag");
    $S .= MainPage($TagSn);
} else
{
    if (! isset($TagSn)) $TagSn = "";
    $S .= MainPage($TagSn);
}
$S .= "</div></body></html>\n";
echo $S;

?>
