#!/bin/sh
# createphotoset.sh
######################################################################
# Copyright 2023 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
#
# Usage:

# createphotoset.sh -s Source -f FolderName -t ThumbnailLength \
# -l WebPicLength 
#
#   * Started in background by importphotoset.php
#   * Copies selected source folder to fullsize repository in web
#     tree, owned by webserver
#   * Creates web-sized photos and thumbnails in webpics folder
#   * Creates Ogg (Theora/Vorbis) versions and thumbnails for videos
#   * Makes no database entries -- PHP script does them after this
#     finishes)

######################################################################
AddDummy()
{
    # Usage: AddDummy $BaseName
    # Copy a dummy image (PNG) as a thumbnail for video file $1.???
    for Folder in $FullFolder $WebFolder $ThumbFolder
    do
	cp images/dummyvideothumbnail.png $Folder/_@_$1.png
    done
    return
}

FileSize()
{
    ls -s --block-size=1 ${FullFolder}/$1 | awk '{print $1}'
    return
}

# Used to check conversion times for calibration
NowHundths()
{
    # Current time to nearest hundredth second, times 100
    Now=$(date +%s.%N)
    Int=$(echo $Now | sed -e 's/\..*$//')
    Thousths=$(echo $Now | \
	sed -e 's/^.*\.\([[:digit:]][[:digit:]][[:digit:]]\).*$/\1/')
    Hundths=$(( (1$Thousths - 995) / 10 ))
    echo $(( $Int * 100 + $Hundths ))
    return
}

# Used to check conversion times for calibration
Delta()
{
    # Current time - $1 with decimal point added before last 2 digits
    Str=$(( $(NowHundths) - $1 ))
    Int=$(( $Str / 100 ))
    Frac=$(echo 0$(( $Str % 100 )) \
	| sed -e 's/^.*\([[:digit:]][[:digit:]]\)$/\1/')
    echo $Int.$Frac
    return
}

ProcessImage()
{
    # Usage: ProcessImage $File
    # Set EXIF Orientation tag to "Horizontal (normal)' if rotated
    # Generate reduced-size webpic and thumbnail
    # Generate webpic (limit webpic size to full size)

#ZStartP=$(NowHundths)
    if (exiftool ${FullFolder}/$1 | grep Orientation \
	    | grep -v 'Horizontal (normal)' >/dev/null)
    then
	exiftool -Orientation='Horizontal (normal)' \
		 -overwrite_original ${FullFolder}/$1 >/dev/null
    fi
    
    LongShort=$(gm identify -format '%w\n%h\n' ${FullFolder}/$1 \
	| sort -rn | tr "\n" " ")
    FullLength=$(echo $LongShort | awk '{print $1}')
    FullShort=$(echo $LongShort | awk '{print $2}')
    # Adjust WebPicLength for extra-wide or extra-tall images
    WebLength=$(ScaledWebPicLength $FullLength $FullShort \
				   $WebPicLength)
    # But not greater than fullsize length
    if [ $WebLength -gt $FullLength ]
    then
	TargetLength=$FullLength
    else
	TargetLength=$WebLength
    fi
    Resize $TargetLength $1 $WebFolder
    # Generate thumbnail image (limit thumbnail size to full size)
    if [ $ThumbnailLength -gt $FullLength ]
    then
	TargetLength=$FullLength
    else
	TargetLength=$ThumbnailLength
    fi
    Resize $TargetLength $1 $ThumbFolder
    PicCount=$(( $PicCount + 1 ))
    PicBytes=$(( $PicBytes + $(FileSize $1) ))
    echo $PicCount $PicBytes > $WebFolder/zzpicsdone
#echo $1 $(Delta $ZStartP) >> $WebFolder/zzpictimes
    return
}

ProcessVideo()
{
    # Usage: ProcessVideo $File
    # For each video, make an ogg (theora/vorbis) version under
    # $WebFolder, then, if no camera-generated thumbnail is present,
    # extract a thumbnail image from the video or, failing that, add a
    # dummy placeholder stillfile.

#ZStartV=$(NowHundths)
    BaseName=$(echo $1 | sed -e "s/\.[^.]*$//")
    OggFile=${BaseName}.ogg
    Geometry=$(/usr/share/mplayer/midentify.sh $FullFolder/$File | \
	grep 'ID_VID' | grep 'WIDTH\|HEIGHT\|_0_ROTATE')
    Width=$(echo $Geometry | \
	sed -ne 's/^.*ID_VIDEO_WIDTH=\([[:digit:]]*\).*$/\1/p')
    Height=$(echo $Geometry | \
	sed -ne 's/^.*ID_VIDEO_HEIGHT=\([[:digit:]]*\).*$/\1/p')
    Rotation=$(echo $Geometry | \
	sed -ne 's/^.*ID_VID_0_ROTATE=\(\(+\|-\)*\([[:digit:]]*\)\).*$/\1/p')
    if [ "z$Rotation" = "z" ]
    then
	Rotation=$(exiftool -m -p \$Rotation $FullFolder/$File \
			    2>/dev/null)
	if [ "z$Rotation" = "z" ]
	then
	    Rotation=0
	fi
    fi
    Rights=$(( $Rotation / 90 ))
    if [ $(($Rights/2*2)) -ne $Rights ]
    then
	Temp=$Width
	Width=$Height
	Height=$Temp
    fi
    if [ $Width -gt $Height ]
    then
	Aspect="L"
    else
	Aspect="P"
    fi
    Options="-c:v libtheora -q:v 7 -c:a libvorbis -q:a 4 -y"
    # Avoid known bug in ffmpeg:
    Options=" $Options -max_muxing_queue_size 9999"
    if [ $Width -gt 1000 -o $Height -gt 1000 ]
    then
	if [ $Aspect = "L" ]
	then
	    W=960
	    H=$(( $Height * 960 / $Width ))
	else
	    H=960
	    W=$(( $Width * 960 / $Height ))
	fi
	Options="$Options -s ${W}x$H"
    fi
    ffmpeg -i $FullFolder/$1 $Options $WebFolder/$OggFile \
	   >/dev/null 2>&1

    # If no thumbnail present, make one
    if ! (echo " "$ImageList | grep -i " $BaseName\." >/dev/null)
    then
	if [ -e $WebFolder/$OggFile ]
	then
	    oggThumb -t 0 -n $FullFolder/${BaseName}.jpg \
		$WebFolder/$OggFile >/dev/null 2>&1
	fi
	if [ -e $FullFolder/${BaseName}.jpg ]
	then
	    ProcessImage ${BaseName}.jpg
	else
	    AddDummy $BaseName
	fi
    fi
    VidCount=$(( $VidCount + 1 ))
    VidBytes=$(( $VidBytes + $(FileSize $1) ))
    echo $VidCount $VidBytes > $WebFolder/zzvidsdone
#echo $1 $(Delta $ZStartV) >> $WebFolder/zzvidtimes
    return
}

Resize()
{
    # Usage: Resize $PicLength $Pic $Folder
    # Puts a $PicLength-sized copy of the fullsize $Pic into $Folder
    gm convert -size ${1}x${1} ${FullFolder}/$2 -resize ${1}x${1} \
	${3}/$2
    return
}

ScaledWebPicLength()
{
    # Usage: ScaledWebPicLength $Long $Short $DefaultLength
    # Where: $Long is the longer of width/height, $Short the shorter,
    # $DefaultLength is the default $WebPicLength
    # For extra-wide or extra-tall (aspect ratio > 3:2) images, adjust
    # longer webpic dimension so that shorter is 3/4 WebPicLength
    # Returns longer webpic dimension
    
    if [ $2 -ge $(( $1 * 2 / 3 )) ]
    then
	echo $3
    else
	echo $(( $1 * $3 * 3 / $2 / 4 ))
    fi
    return
}

######################################################################
while getopts "s:f:t:w:" Option
do
    case $Option in
	s ) Source=$OPTARG
            ;;
	f ) FolderName=$OPTARG
            ;;
	t ) ThumbnailLength=$OPTARG
            ;;
	w ) WebPicLength=$OPTARG
            ;;
    esac
done

FullFolder=fullsize/$FolderName
WebFolder=webpics/$FolderName
ThumbFolder=${WebFolder}/thumbnails

PicCount=0
VidCount=0
PicBytes=0
VidBytes=0

cp ${Source}/* $FullFolder/
chmod 644 $FullFolder/*

StartFolder=$(pwd)
cd $FullFolder

# Check MIME types of all files, assign to $ImageList or $VideoList
ImageList=""
VideoList=""
FileList=$(ls)
for File in $FileList
do
    MimeType=$(file -b --mime-type $File)
    First5=$(echo $MimeType | cut -c1-5)
    case $First5 in
	image )
	    ImageList="$ImageList $File"
            ;;
	video )
	    VideoList="$VideoList $File"
            ;;
	* )
	    if [ "Z$MimeType" = "Zapplication/ogg" -o \
		 "Z$MimeType" = "Zapplication/octet-stream" ]
	    then
		VideoList="$VideoList $File"
	    fi
    esac
done

cd $StartFolder

# Indicate that conversion has started
touch $WebFolder/zzconvstart

# Process each image
for File in $ImageList
do
    ProcessImage $File
done

# Process each video
for File in $VideoList
do
    ProcessVideo $File
done

sync; sync

# Indicate completion:
touch $WebFolder/zzcomplete
#rm $WebFolder/zzpicsdone
#rm $WebFolder/zzvidsdone
