<?php
/*********************************************************************
gk_inputval.php

Input validation functions
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

By convention, these tests assume the $Value argument has already been
trimmed of leading and trailing spaces.

Intended application: Use a single function Errors(...) to validate
all values user has submitted. This function will test each input
value using the appropriate ErrIf... function, appending the results
to a single string variable $E, then conclude with something like:

    if ($E != "")
    {
	$T = "Problems with input:";
	$E = Strong($T)."\n<ul>$E\n</ul>";
	$E = StrWarningBox($E);
    }
    return $E;
*********************************************************************/
function BulletItem($Text)
{
    // return list item enclosing text ended in period
    return "\n<li>$Text.</li>";
}

function ErrIfAbsent($Label, $Value, $NoneOk=False)
{
    // Return error bullet if $Value is blank or "none" (as from a
    // default menu option)
    if ($Value != "" and ($Value !== "none" or $NoneOk))
	$E = "";
    else
    {
	$E = "You must specify a";
	if (in_array(substr(strtolower($Label), 0, 1),
		     array("a", "e", "i", "o", "u"))) $E .= "n";
	$E .= " $Label";
	$E = BulletItem($E);
    }
    return $E;
}

function ErrIfAbsentOrInvalid($Label, $Value, $Table, $Col,
			      $NoneOk=False)
{
    // Return error bullet if $Value is blank or "none" (as from a
    // default menu option) or not present in $Col of $Table
    $E = ErrIfAbsent($Label, $Value, $NoneOk);
    if ($E == "")
	if (! IsInDatabase($Table, $Col, $Value))
	    $E = BulletItem("Invalid $Label &quot;$Value&quot;");
    return $E;
}

function ErrIfAbsentOrNotNew($Label, $Value, $Table, $Col,
			     $NoneOk=False)
{
    // Return error bullet if $Value is blank or "none" (as from a
    // default menu option) or already present in $Col of $Table
    $E = ErrIfAbsent($Label, $Value, $NoneOk);
    if ($E == "")
	if (IsInDatabase($Table, $Col, $Value))
	{
	    $T = "$Label &quot;$Value&quot; already exists";
	    $E = BulletItem($T);
	}
    return $E;
}

function ErrIfBadDateTime($Label, $Value, $BlankOk=False)
{
    // Return error bullet if $Value is not a valid date/time
    if (($BlankOk and $Value == "")
	or strtotime($Value) !== InvalidDateTime)
	$E = "";
    else
    {
	$T = "Can't parse $Label &quot;$Value&quot; as a date";
	$E = BulletItem($T);
    }
    return $E;
}

function ErrIfBeyondRealRange($Label, $Value, $Lo, $LoOk, $Hi,
			      $HiOk=True, $BlankOk=False)
{
    // Return error bullet if $Value is out of range
    if (($BlankOk and $Value == "")
	or (is_numeric($Value) and 
	    ($Value > $Lo or ($LoOk and $Value == $Lo))
	    and
	    ($Value < $Hi or ($HiOk and $Value == $Hi))))
	$E = "";
    else
    {
	$E = "$Label must be ";
	$E .= (($LoOk) ? "at least " : "greater than ")."$Lo and ";
	$E .= (($HiOk) ? "at most " : "less than ")."$Hi";
	$E = BulletItem($E);
    }
    return $E;
}

function ErrIfNotNatural($Label, $Value, $ZeroOk=False)
{
    // Return error bullet if $Value is not a positive integer
    return Is_Natural($Value, $ZeroOk)
	? ""
	: BulletItem("$Label must be a positive integer");
    return $E;
}

function ErrIfNotPosReal($Label, $Value, $ZeroOk=False,
			 $BlankOk=False)
{
    // Return error bullet if $Value is not a positive real
    return (($BlankOk and $Value == "")
	    or (is_numeric($Value)
		and ($Value > 0 or ($ZeroOk and $Value == 0))))
	? ""
	: BulletItem("$Label must be a ".($ZeroOk
					  ? "nonnegative"
					  : "positive")." number");
}

function Is_Natural($Value, $ZeroOk=False)
{
  return (is_numeric($Value) and $Value == intval($Value) and
	    ($Value > 0 or ($ZeroOk and $Value == 0)));
}
