<?php
/*********************************************************************
importphotoset.php

Import a photo set from a folder of original photos
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

Starts background process createphotoset.sh, which
    Copies selected source folder to fullsize repository
    in web tree, owned by webserver
    Creates web-sized set in webpics folder 
    Creates thumbnails folder for each web-size set
    Makes database entries

Includes showsetprogress.php, which
    Displays progress bar until background process completes

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require_once ("lib_layout.php");
require_once("lib_tag.php");

$T = "Current time zone of this server (".$_SERVER["SERVER_NAME"];
$T .= "), equal to UTC".LocalTimeZoneOffset;
$T .= ". For phones and cameras set to local time";
$T .= " (handles daylight vs. standard automatically).";
define ("HintTimeZoneLocal", $T);
$T = "An arbitrary time zone you specify, either by its abbreviation";
$T .= " or by its offset from UTC";
define ("HintTimeZoneOther", $T);
$T = "Coordinated Universal Time, aka Greenwich Mean Time (GMT).";
$T .= " Recommended unless you never travel.";
define ("HintTimeZoneUTC", $T);

function CreateInBackground($Source, $FolderName, $ThumbnailLength,
                            $WebPicLength)
{
    // Create folders in filesystem and assemble GNU command to create
    // fullsize, webpic and thumbnail images in background
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    mkdir ($FullFolder, 0755);
    mkdir ($WebFolder, 0755);
    mkdir ($ThumbFolder, 0755);
    // Initialize progress files
    foreach (array("pic", "vid") as $Media)
    {
        $File = fopen("$WebFolder/zz{$Media}sdone", "w");
        fwrite($File, "0 0");
        fclose($File);
    }
    // Convert arrays (of image and video filename extensions) to
    // strings to pass as arguments to shell script

    $LogFile = "$WebFolder/processlog";
    $C = "sh createphotoset.sh -s ".OriginalPhotos."/$Source";
    $C .= " -f $FolderName -t $ThumbnailLength -w $WebPicLength";
    $C .= " > $LogFile 2>&1 &";
    return $C;
}

function ExistsAnyFolder($FolderName)
{
    // True if any $FolderName already exists anywhere in the
    // repository
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    return (file_exists ($FullFolder) or file_exists ($WebFolder));
}

function ExistsPhotoSet($FolderName)
{
    // True if a photoset named $FolderName is in the database
    return IsInDatabase("photoset", "foldername", $FolderName, True);
}

function FormCreatePhotoSet($Source, $FolderName, $TimeZone,
                            $ThumbnailLength, $WebPicLength)
{
    $S = StrFormBeginRecurse();

    $S .= HtmlHidden("Source", $Source);
    $S .= HtmlHidden("FolderName", $FolderName);
    $S .= StrTableBegin(0, "", "", BoxStyle."; width: 50%");
    $S .= RowSetName($FolderName);
    $S .= RowCameraTimeZone($TimeZone);
    $S .= RowsImageSizes($ThumbnailLength, $WebPicLength);
    $S .= StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("Create", "Create Photo Set"), "2",
                  "", "text-align: center");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function FormMenuSource($Selected="", $AutoSubmit=True, $Prompt="")
{
    if ($Prompt == "")
        $Prompt = "&mdash; Select folder of original photos &mdash;";
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SelectSource", "1");
    $S .= MenuSource($Selected, $AutoSubmit, $Prompt);
    $S .= StrFormEnd();
    return $S;
}

function FormUploadArchive()
{
    $T = ParsedHumanNum(UploadMaxFileSize, True);
    $UploadSizeLimit = ceil($T[1]);
    $FolderName = date('Ymd-His');
    while (file_exists(OriginalPhotos."/$FolderName"))
    {
        sleep(1);
        $FolderName = date('Ymd-His');
    }
    $S = StrFormBeginPostUpload(ThisScript(), UploadMaxFileBytes);
    $S .= StrTableBegin(0, "", "", BoxStyle);
    $S .= StrRowBegin();
    $T = "Select a file (.zip, .tar or .tgz, max ";
    $T .= UploadMaxFileSize." bytes)<br>\ncontaining photos to";
    $T .= " upload:<br>\n";
    $T .= HtmlInputPostUpload("UserFile", 60);
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $T = "Specify a folder name for the files:<br>\n";
    $T .= HtmlTextInput("FolderName", 20, $FolderName);
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $T = HtmlSubmitButton("Upload", "Upload File");
    $T .= "&nbsp; &nbsp; &nbsp;".HtmlSubmitButton("CancelUpload",
						  "Cancel");
    $S .= StrCell($T, "", "", "text-align: center");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function IsDeletedPhotoSet($FolderName)
{
    // True if $FolderName occurs in deletedset table
    return IsInDatabase("deletedset", "foldername", $FolderName,
                        True);
}

function IsFolderNameNew($FolderName)
{
    // True if $FolderName isn't in repository or database (even as a
    // deleted photo set)
    return (! ExistsAnyFolder($FolderName)
            and ! ExistsPhotoSet($FolderName)
            and ! IsDeletedPhotoSet($FolderName));
}

function MediaCountsByType($Directory)
{
    /* Return array(category=>ext=>count) with numbers of {images,
     * videos, other, [ unreadable ]} in specified directory. Omits
     * hidden files */
    $MediaCountsByType = array();
    $Dir = dir($Directory);
    $Finfo = finfo_open(FILEINFO_MIME_TYPE);
    while ($File = $Dir->read())
    {
        if (! Invisible($File))
        {
            $Ext = FileNameExtension($File);
            $MediaType = MediaType("$Directory/$File", $Finfo);
            $MediaCountsByType[$MediaType][$Ext]++;
        }
    }
    finfo_close($Finfo);
    $Dir->close();
    return $MediaCountsByType;
}

function MenuSource($Selected="", $AutoSubmit=True, $Prompt="")
{
    // Drop-down menu of folders under OriginalPhotos/
    if ($Prompt == "")
        $Prompt = "&mdash; Select folder of original photos &mdash;";
    $Dirs = Subdirectories(OriginalPhotos);
    rsort ($Dirs);
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin("Source", $Attribs);
    if ($Selected == "" or $Selected == "none")
        $S .= HtmlSelectOption("none", $Prompt, True);
    foreach ($Dirs as $Dir)
        if (IsFolderNameNew($Dir))
            $S .= HtmlSelectOption($Dir, $Dir, ($Dir == $Selected));
    $S .= HtmlSelectEnd();
    if ($AutoSubmit) $S = StrWithGoButton($S, "Go");
    return $S;
}

function OriginalStats($Source, $MediaCounts)
{
    $S = "<div style=\"".BoxStyle."; width: 25%;";
    $S .= " text-align: left\">\n";
    $S .= HtmlPara(Strong($Source)." contains files:");
    $S .= "<dl>\n";
    foreach (array("Image", "Video", "Other", "[ unreadable ]")
             as $Cat)
        if (array_key_exists($Cat, $MediaCounts))
            $S .= StatsByCat($Cat, $MediaCounts[$Cat]);
    $S .= "</dl>\n";
    $S .= "</div>\n";
    return $S;
}

function PageCreatePhotoSet($Source, $MediaCounts, $FolderName,
                            $TimeZone, $ThumbnailLength,
                            $WebPicLength)
{
    $S = FormMenuSource($Source);
    $S .= OriginalStats($Source, $MediaCounts);
    $S .= "<h3>Settings for new photo set</h3>\n";
    $S .= FormCreatePhotoSet($Source, $FolderName, $TimeZone,
                             $ThumbnailLength, $WebPicLength);
    $S .= SamplePic("thumbnail", $ThumbnailLength);
    $S .= SamplePic("large image", $WebPicLength);
    return $S;
}

function PageImportOptions()
{
    $S = StrSubTitle("&hellip;from a folder on the server:");
    $S .= FormMenuSource();
    $S .= StrSubTitle("or");
    $S .= StrSubTitle("&hellip;by uploading a zip or tar file:");
    $S .= FormUploadArchive();
    return $S;
}

function PagePhotoSetSettings($Source, $TimeZone)
{
    $SourcePath = OriginalPhotos."/$Source";
    if (!file_exists($SourcePath))
        $S = WarnNoSuchPath($SourcePath);
    elseif (! is_readable($SourcePath))
        $S = PageUnreadables($Source, False);
    else
    {
        $MediaCounts = MediaCountsByType($SourcePath);
        $FolderName = $Source;
        if (! IsFolderNameNew($FolderName))
            $S = WarnFolderNotNew($FolderName);
        elseif (sizeof($MediaCounts) > 0)
        {
            if (array_key_exists("[ unreadable ]", $MediaCounts))
                $S = PageUnreadables($Source, $MediaCounts);
            else
                $S = PageCreatePhotoSet(
                    $Source, $MediaCounts, $FolderName, $TimeZone,
                    ThumbnailLength, WebPicLength);
        } else $S = WarnNoImages($Source);
    }
    return $S;
}

function PageUnreadables($Source, $MediaCounts)
{
    $ServerUser = trim(shell_exec("whoami"));
    $Path = (is_link(OriginalPhotos))
          ? realpath(readlink(OriginalPhotos))
          : OriginalPhotos;
    $T = "Folder &quot;$Path/$Source&quot; ";
    $T .= ($MediaCounts === False) ? "is unreadable."
       : "contains one or more unreadable files.";
    $T .= " This is almost certainly a file permissions problem.";
    $T .= " Please correct this by:<ol><li>Setting the folder and";
    $T .= " its files world-readable or at least readable by the";
    $T .= " &quot;$ServerUser&quot; group (the account under which";
    $T .= " the web server runs).</li><li>Setting the folder itself";
    $T .= " world- (or $ServerUser-) executable.";
    $S = FormMenuSource($Source);
    $S .= StrWarningBox($T, "50%");
    if ($MediaCounts !== False)
        $S .= OriginalStats($Source, $MediaCounts);
    return $S;
}

function RowCameraTimeZone($TimeZone)
{
    $S = StrRowBegin();
    $S .= StrCell("Camera's time zone is set to:");
    $C = "<label title=\"".HintTimeZoneUTC."\">";
    $C .= HtmlRadioButton("TimeZone", "UTC", ($TimeZone == "UTC"));
    $C .= "UTC</label> <label title=\"".HintTimeZoneLocal."\">";
    $C .= HtmlRadioButton("TimeZone", "Local",
                          ($TimeZone == LocalTimeZone));
    $C .= LocalTimeZone."</label>";
    /*****************************************************************
      Defer until we find a way to validate time zone inputs:
      $C .=" <label title=\"";
      $C .= HintTimeZoneOther."\">";
      $C .= HtmlRadioButton("TimeZone", "other");
      $C .= "other: </label>";
      $C .= HtmlTextInput("OtherTZ", "3", "", "", "text-align: right");
    *****************************************************************/
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSetName($FolderName)
{
    $S = StrRowBegin();
    $S .= StrCell("Photo Set Name:");
    $S .= StrCell($FolderName);
    $S .= StrRowEnd();
    return $S;
}

function RowsImageSizes($ThumbnailLength, $WebPicLength)
{
    $S = StrRowBegin();
    $T = "Maximum size of displayed images (they won't exceed the";
    $T .= " originals):";
    $S .= StrCell($T);
    $S .= StrCell("&nbsp;");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell("Thumbnail images:");
    $S .= StrCell(HtmlTextInput("ThumbnailLength", "5",
                                $ThumbnailLength, "",
                                "text-align: right")." pixels");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell("Large images:");
    $S .= StrCell(HtmlTextInput("WebPicLength", "5", $WebPicLength,
                                "", "text-align: right")." pixels");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell("See below for previews of selected image sizes");
    $S .= StrCell(HtmlSubmitButton("ChangeSizes", "Update previews"));
    $S .= StrRowEnd();
    return $S;
}

function SamplePic($Description, $Length)
{
    $L = "Sample ${Description}, $Length pixels wide";
    $S = HtmlPara("$L:");
    $S .= HtmlImage("images/blue75x100.jpg", $L, 0.75 * $Length,
                    $Length, "1");
    return $S;
}

function StatsByCat($Cat, $Exts)
{
    ksort($Exts);
    $S = "<dt>$Cat:</dt>\n";
    $T = "";
    foreach ($Exts as $Ext => $FileCount)
    {
        $T .= ", $FileCount&nbsp;$Ext";
    }
    $S .= "<dl>".substr($T, 2)."</dl>\n";
    return $S;
}

function UploadArchive($FolderName, $TimeZone)
{
    $Status = $_FILES["UserFile"]["error"];
    if ($Status != UPLOAD_ERR_OK)
    {
        switch ($Status)
        {
        case UPLOAD_ERR_INI_SIZE:
            $S = StrWarning("File is too big!");
            break;
        default:
            $S = StrWarning("Upload failed with error code $Status");
        }
        $S .= PageImportOptions();
    } elseif (! IsFolderNameNew($FolderName))
    {
        $S = WarnFolderNotNew($FolderName);
        $S .= PageImportOptions();
    } else
    {
        list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
        if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
        $NewPath = "$FullUploads/".LoggedInUser();;
        if (move_uploaded_file($_FILES["UserFile"]["tmp_name"],
                               $NewPath)
            and filesize($NewPath) > 0)
        {
            if (is_readable($NewPath))
            {
                $CWD = getcwd();
                mkdir("originalphotos/$FolderName");
                $NewFolder = "originalphotos/$FolderName";
                $FullPath = "$CWD/$NewPath";
                $Finfo = finfo_open(FILEINFO_MIME_TYPE);
                $MimeType = finfo_file($Finfo, $NewPath);
                switch ($MimeType)
                {
                case "application/zip":
                    shell_exec("cd $NewFolder ; unzip $FullPath");
                    break;
                case "application/gzip":
                    shell_exec("cd $NewFolder ; tar xzf $FullPath");
                    break;
                case "application/x-tar":
                    shell_exec("cd $NewFolder ; tar xf $FullPath");
                    break;
                default:
                    $T = "Can't unpack file type \"$MimeType\"";
                    $S = StrWarning($T);
                    UnlinkIfExists($NewPath);
                    $S .= PageImportOptions();
                    return $S;
                }
                UnlinkIfExists($NewPath);
                $S = PagePhotoSetSettings($FolderName, $TimeZone);
            } else
            {
                UnlinkIfExists($NewPath);
                $S = StrWarning("Couldn't read uploaded file");
                $S .= PageImportOptions();
            }
        } else
        {
            UnlinkIfExists($NewPath);
            $S = StrWarning("Upload failed");
            $S .= PageImportOptions();
        }
    }
    return $S;
}

function WarnFolderNotNew($FolderName)
{
    $W = "Photo Set $FolderName already exists or is in progress.";
    $S = StrWarningBox($W, "50%");
    $S .= FormMenuSource($FolderName);
    return $S;
}

function WarnNoImages($Source)
{
    $S = StrWarning("$Source contains no images");
    $S .= FormMenuSource($Source);
    return $S;
}

function WarnNoSuchPath($SourcePath)
{
    $S = StrWarning("Path $SourcePath doesn't exist!");
    $S .= FormMenuSource();
    return $S;
}

//********************************************************************
set_time_limit(0); // Waive the timeout; it takes a while

// Fetch input variables
$FormVarList = array(
    "FolderName",
    "Source",
    "ThumbnailLength",
    "TimeZone",
    "WebPicLength"
);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S = HeadAndTitle($SystemTitle." Select Original Photos");
$S .= PageTitleWithButton("Select Original Photos", BtnMainMenu());
$S .= "<div align=\"center\">\n";

if (! isset($TimeZone)) $TimeZone = DefaultCameraTimeZone;
if (isset($FormVars["SelectSource"]))
{
    // $Source also set, but may no longer exist as a source folder
    if ($Source == "none")
    {
        $S .= StrWarning("You must select a folder:");
        $S .= FormMenuSource();
    } else
        $S .= PagePhotoSetSettings($Source, $TimeZone);
} elseif (isset($FormVars["ChangeSizes"]))
{
    $MediaCounts = MediaCountsByType(OriginalPhotos."/$Source");
    $S .= PageCreatePhotoSet($Source, $MediaCounts, $FolderName,
                             $TimeZone, $ThumbnailLength,
                             $WebPicLength);
} elseif (isset($FormVars["Create"]))
{
    if (! IsFolderNameNew($FolderName))
        $S .= WarnFolderNotNew($FolderName);
    else
    {
        $Cmd = CreateInBackground($Source, $FolderName,
                                  $ThumbnailLength, $WebPicLength);
        /*************************************************************
          To prevent PHP from hanging while converting, use method
          proposed by nop at netissat dot bg, 10-Apr-2003 07:49 on PHP
          exec ref page http://us4.php.net/manual/en/function.exec.php
        *************************************************************/
        session_write_close();
        exec ($Cmd);
        $StartTime = time();
        $_SERVER["SCRIPT_NAME"] = "showsetprogress.php";
        require("showsetprogress.php");
        exit;
    }
} elseif (isset($_POST["Upload"]))
    $S .= UploadArchive(trim($_POST["FolderName"]), $TimeZone);
else
    $S .= PageImportOptions();
$S .= "</div></body></html>\n";
echo $S;

?>
