<?php
/*********************************************************************
lib_filesys.php
Filesystem Access Definitions for Application
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
function ConvertToOgg($SourceFile, $OggFile)
{
    /*****************************************************************
    Converts video in $SourceFile to Ogg (Theora/Vorbis) $OggFile,
    scaling if needed
    *****************************************************************/
    list($VWidth, $VHeight, $Format, $Codec, $AutoRotate)
        = array_values(VideoSpecs($SourceFile));
    // if $AutoRotate is an odd multiple of 90, swap dimensions
    $Rights = $AutoRotate / 90;
    if (intdiv($Rights, 2) * 2 != $Rights)
    {
        $Temp = $VWidth;
        $VWidth = $VHeight;
        $VHeight = $Temp;
    }
    $Aspect = $VWidth > $VHeight ? "L" : "P";
    $Options = "-c:v libtheora -q:v 7 -c:a libvorbis -q:a 4 -y";
    // Avoid known bug in ffmpeg:
    $Options .= " -max_muxing_queue_size 9999";
    if ($VWidth > 1000 or $VHeight > 1000)
    {
        if ($Aspect == "L")
        {
            $W = 960;
            $H = $VHeight * 960 / $VWidth;
        } else
        {
            $H = 960;
            $W = $VWidth * 960 / $VHeight;
        }
        $Options .= " -s {$W}x{$H}";
    }
    exec("ffmpeg -i $SourceFile $Options $OggFile");
    return;
}

function DummyThumbnail($BaseName)
{
    // Return the artificial filename generated as a thumbnail for a
    // video file that has no other thumbnail
    return "_@_$BaseName.png";
}

function ImageFolder($FolderName)
{
    $FullFolder = "fullsize/$FolderName";
    $WebFolder = "webpics/$FolderName";
    $ThumbFolder = "$WebFolder/thumbnails";
    return array
    (
        $FullFolder,
        $WebFolder,
        $ThumbFolder
    );
    return;
}

function LatestFile($Path, $Php=True)
{
    return $Php
	? "$Path?mtime=".filemtime($Path)
	: $Path;
}

function MediaType($File, $Finfo)
{
    // return media type {Image, Video, Other, [ unreadable ]}
    if (is_readable($File))
    {
        $MimeType = finfo_file($Finfo, $File);
        switch (substr($MimeType, 0, 5))
        {
        case "image":
            $Type = "Image";
            break;
        case "video":
            $Type = "Video";
            break;
        default:
            $Type = ($MimeType == "application/ogg"
                     or $MimeType == "application/octet-stream")
                ? "Video" : "Other";
        }
    } else $Type = "[ unreadable ]";	
    return $Type;
}

function OggFile($File)
{
    // Return the filename of the (possibly nonexistent) Ogg
    // (Theora/Vorbis) version corresponding to the video $File
    return FileNameBase($File).".ogg";
}

function VideoSpecs($VideoFile)
{
    /*****************************************************************
    Returns width, height, format, codec and, if present, autorotation
    of a video file

    Uses mplayer's identify function (via wrapper script midentify.sh)
    to extract information.
    *****************************************************************/
    $AutoRotate = False; // in case not present
    $Cmd = "/usr/share/mplayer/midentify.sh $VideoFile";
    $Cmd .= " | grep 'ID_VID'";
    $Cmd .= " | grep 'WIDTH\|HEIGHT\|FORMAT\|CODEC\|_0_ROTATE'";
    exec($Cmd, $Result);
    foreach ($Result as $Line)
    {
        switch (strtok($Line, "="))
        {
        case "ID_VIDEO_WIDTH":
            $Width = strtok("=");
            break;
        case "ID_VIDEO_HEIGHT":
            $Height = strtok("=");
            break;
        case "ID_VIDEO_FORMAT":
            $Format = strtok("=");
            break;
        case "ID_VIDEO_CODEC":
            $Codec = strtok("=");
            break;
        case "ID_VID_0_ROTATE":
            $AutoRotate = strtok("=");
            break;
        }
    }
    //if ($AutoRotate === False)
    if (! is_numeric($AutoRotate))
    {
        unset($Result);
	$Cmd = "exiftool -m -p '\$Rotation' $VideoFile 2>/dev/null";
	exec($Cmd, $Result);
	if (count($Result) > 0)
	{
	    $AutoRotate = $Result[0];
	    if (! is_numeric($AutoRotate))
	        $AutoRotate = 0;
	} else
	    $AutoRotate = 0;
    }
    return array
        ("width" => $Width,
         "height" => $Height,
         "format" => $Format,
         "codec" => $Codec,
         "autorotate" => $AutoRotate);
}

function VideoThumbnail($File, $PhotoSetSn)
{
    // Return the filename of the thumbnail image corresponding to the
    // video $File, if it exists in photo table, else false.
    $BaseName = FileNameBase($File);
    $Q = "SELECT filename FROM photo";
    $Q .= " WHERE photosetsn = $1 AND";
    $Q .= " (filename LIKE $2 || '.%'";
    $Q .= " OR filename = '_@_' || $2 || '.png')";
    return pg_fetch_result(Query($Q, $PhotoSetSn, $BaseName), 0);
}

function VideoTimeStamp($VideoFile)
{
    /*****************************************************************
    Returns timestamp (as a string) of a video file

    Uses mplayer's identify function (via wrapper script midentify.sh)
    to extract information. Returns only the time without timezone
    adjustment, from line beginning ID_CLIP_INFO_VALUE3=
    *****************************************************************/
    $Cmd = "/usr/share/mplayer/midentify.sh $VideoFile";
    $Cmd .= " | grep ID_CLIP_INFO_VALUE";
    $Cmd .= " | grep [[:digit:]][[:digit:]]:[[:digit:]][[:digit:]]";
    $Cmd .= ":[[:digit:]][[:digit:]] | cut -d = -f 2";
    exec($Cmd, $TimeStamps);
    // certain files may have multiple lines containing timestamps:
    $ValidTimeStamp = False;
    foreach ($TimeStamps as $TimeStamp)
    {
        if (strtotime($TimeStamp) !== InvalidDateTime)
        {
            $ValidTimeStamp = $TimeStamp;
            // Check if timestamp includes timezone
            $WithZone = (preg_match("/\+|-\d\d:?\d\d$/", $TimeStamp)
                         == 1);
            if ($WithZone) break;
        }
    }
    return array($ValidTimeStamp, $WithZone);
}

//********************************************************************
require ("gk/gk_filesys.php");


?>
