<?php
/*********************************************************************
loc_camera.php

Camera-Specific Definitions for Photo Collection Manager
______________________________________________________________________
Copyright 2024 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/

/*********************************************************************
Attributes (besides geometry and timestamp) of photos captured by
camera, which must also exist in photo table.
Edit as new attributes are added
*********************************************************************/
$PhotoMetaData = array
(
    "latitude",
    "longitude",
    "altitude"
);

function ClearExifRotation($NewFile)
{
    /*****************************************************************
    Set EXIF Orientation tag to "Horizontal (normal)' if rotated.

    Use exiftool to check for Orientation tag and correct if needed.
    *****************************************************************/
    $Cmd = "if (exiftool $NewFile | grep Orientation";
    $Cmd .= " | grep -v 'Horizontal (normal)' >/dev/null);";
    $Cmd .= " then exiftool -Orientation='Horizontal (normal)'";
    $Cmd .= " -overwrite_original $NewFile >/dev/null;";
    $Cmd .= " fi";
    exec($Cmd);
    return;
}

function DeleteGPSFromFile($FilePath)
{
    /*****************************************************************
    Remove tags related to geolocation from embedded metadata (EXIF in
    photos, various formats in videos). Use ffmpeg for *.ogg videos,
    use exiftool for:

    Images:
        Apple iPhone IMG_*.JPG
        Android phone IMG_*.jpg

    Videos:
        Apple iPhone IMG_*.MOV
        Android phone VID_*.mp4

    *****************************************************************/
    $MimeType = mime_content_type($FilePath);
    if (mime_content_type($FilePath) == "video/ogg")
    {
        $UserId = LoggedInUser();
        list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
        if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
        $NewFile = "$FullUploads/$UserId.ogg";
        $Cmd = "ffmpeg -i $FilePath  -map 0 -map_metadata -1 -c copy";
        $Cmd .= " $NewFile 2>/dev/null";
        exec($Cmd);
        rename($NewFile, $FilePath);
    } else
    {
        $Cmd = "exiftool -overwrite_original -gpsaltitude= ";
        $Cmd .= "-gpslatitude= -gpslongitude= -gpsposition= ";
        $Cmd .= "-gpscoordinates= $FilePath >/dev/null";
        exec($Cmd);
    }
    return;
}

function GPSDiv($Frac)
{
    // Parse GPS values from EXIF data, given as fractions
    list($Num, $Denom) = sscanf($Frac, "%f/%f");
    return $Num/$Denom;
}

function GPSFromExifData($ExifData)
{
    /*****************************************************************
    Given the result of PHP's exif_read_data(), returns an array of
    latitude, longitude, altitude, if present, from a photo or video
    file, as signed decimal degrees and signed meters, else False
    *****************************************************************/
    if (array_key_exists("GPS", $ExifData))
    {
	$GPSLatitudeRef = $ExifData["GPS"]["GPSLatitudeRef"];
	$GPSLatitude = $ExifData["GPS"]["GPSLatitude"];
	$GPSLongitudeRef = $ExifData["GPS"]["GPSLongitudeRef"];
	$GPSLongitude = $ExifData["GPS"]["GPSLongitude"];
	$GPSAltitudeRef = $ExifData["GPS"]["GPSAltitudeRef"];
	$GPSAltitude = $ExifData["GPS"]["GPSAltitude"];
	$Latitude = GPSDiv($GPSLatitude[0])
	  + GPSDiv($GPSLatitude[1])/60.0
	  + GPSDiv($GPSLatitude[2])/3600.0;
	if ($GPSLatitudeRef == "S") $Latitude = -$Latitude;
	$Longitude = GPSDiv($GPSLongitude[0])
	  + GPSDiv($GPSLongitude[1])/60.0
	  + GPSDiv($GPSLongitude[2])/3600.0;
	if ($GPSLongitudeRef == "W") $Longitude = -$Longitude;
	$Altitude = GPSDiv($GPSAltitude);
	if ($GPSAltitudeRef == 1 or $GPSAltitudeRef == 3)
	    $Altitude = -$Altitude;
        return array("latitude" => $Latitude,
                     "longitude" => $Longitude,
                     "altitude" => $Altitude);
    } else return False;
}

function GPSFromFile($FilePath)
{
    /*****************************************************************
    Returns an array of latitude, longitude, altitude, if present,
    from a video file, as signed decimal degrees and signed meters,
    else False.

    Works for photos, too, but wastes time rereading EXIF data as
    photos will already have it in $Info array from function
    ImageCaptionExtract.
    *****************************************************************/
    $Cmd = "exiftool -m -c '%.8f' -p '\$gpslatitude \$gpslongitude'";
    $Cmd .= " $FilePath 2>/dev/null";
    exec($Cmd, $Result);
    if (count($Result) > 0 && strlen($Result[0]) > 0)
    {
        list($Latitude, $LatHem, $Longitude, $LonHem)
            = sscanf($Result[0], "%f %s %f %s");
        if ($LatHem == "S") $Latitude = -$Latitude;
        if ($LonHem == "W") $Longitude = -$Longitude;
        unset($Result);
        $Cmd = "exiftool -m -c '%.8f' -p '\$gpsaltitude'";
        $Cmd .= " $FilePath 2>/dev/null";
        exec($Cmd, $Result);
        if (count($Result) > 0 && strlen($Result[0]) > 0)
        {
            list($Altitude, $AltRef) = sscanf($Result[0], "%f m %s");
            if ($AltRef == "Below") $Altitude = -$Altitude;
        } else
        {
            $Altitude = "NULL"; // php NULL is flakey, use a string
        }
        return array("latitude" => $Latitude,
                     "longitude" => $Longitude,
                     "altitude" => $Altitude);
    } else return False;
}

function ImageCaptionExtract($Image, $TimeZone)
{
    /*****************************************************************
    All images: Always returns width, height. Returns IsPanorama,
    timestamp, latitude, longitude, altitude if available.

    Uses PHP's exif_read_data function to extract information. In
    particular, "DateTimeOriginal" is formatted thus:
    2009:07:05 16:02:06
    *****************************************************************/
    list ($Info["width"], $Info["height"], $ImageType)
        = getimagesize($Image);
    if ($ImageType == IMAGETYPE_JPEG
        or $ImageType == IMAGETYPE_TIFF_II
        or $ImageType == IMAGETYPE_TIFF_MM)
    {
        $ExifData = exif_read_data($Image, 0, True);
	if (array_key_exists("EXIF", $ExifData))
	{
	    $DateTime = $ExifData["EXIF"]["DateTimeOriginal"];
	    $CustomRendered = $ExifData["EXIF"]["CustomRendered"];
	    $Info["IsPanorama"] = (is_numeric($CustomRendered)
				   and $CustomRendered == 6);
	}
	elseif (array_key_exists("IFD0", $ExifData))
	    $DateTime = $ExifData["IFD0"]["DateTime"];
	else
	    $DateTime = "";
	// DateTimeOriginal with no value returns " ":
        $DateTime = trim($DateTime);
        if ($DateTime != "")
        {
            list($Date, $Time) = sscanf($DateTime, "%s %s");
            $Date = str_replace(":", "/", $Date);
            $Info["time"] = "$Date $Time";
            if ($TimeZone == "UTC" and trim($Info["time"]) > "")
                $Info["time"] .= " UTC";
        }
	
	//        $GeoData = GPSFromFile($Image);
        $GeoData = GPSFromExifData($ExifData);
        if ($GeoData !== False)
            $Info = array_merge($Info, $GeoData);
    }
    return $Info;
}

function ImageInfo($FullFolder, $FileName, $TimeZone)
{
    // Returns all image information as array.
    // Nikon-specific: may call NikonInfoExtract to parse info.txt.
    // Cell-phone-specific: may call PhoneTimeStamp to get timestamp.

    // First, extract info from caption within image:
    $Info = ImageCaptionExtract("$FullFolder/$FileName", $TimeZone);
    if (! array_key_exists('time', $Info))
    {
        // Only geometry; use info.txt file if exists in $FullFolder
        $Geometry = array("height" => $Info["height"],
                          "width" => $Info["width"]);
        $InfoRest
            = NikonInfoExtract($FileName, $FullFolder, $TimeZone);
        if (is_null($InfoRest["time"]))
            $InfoRest["time"] = PhoneTimeStamp($FileName,
                                               $FullFolder);
        $Info = array_merge($Geometry, $InfoRest);
    }
    return $Info;
}

function NikonInfoExtract($FileName, $Folder, $TimeZone)
{
    /*****************************************************************
    Nikon-specific: Parses Nikon info.txt file, which has lines like
    this:

    DSCN0035.JPG
    SHUTTER      : 1/120sec
    APERTURE     : F3.7
    FOCAL LENGTH : f27.3mm(X1.0)
    DATE         : 2004.03.22 12:57

    Returns array("time"); in future: array("time", "gpslocation").
    Values are null if not found.
    No longer includes "aperture", "focal" or "speed".
    *****************************************************************/
    $Labels = array(
        "DATE *: "
	);
    $InfoTxt = "$Folder/info.txt";
    $Info = array("time" => Null);
    if (file_exists($InfoTxt))
    {
        $StripCRs = "cat $InfoTxt | tr -d \"\r\"";
        $FindBlock = "sed -ne \"/^".strtoupper($FileName)."/,/^$/p\"";
        $L = "";
        foreach ($Labels as $Label) $L .= "\\\\|$Label";
        $FindLines = "sed -ne \"/".substr($L, 3)."/p\"";
        $CmdLine = "$StripCRs | $FindBlock | $FindLines";
        $Cmd = exec($CmdLine, $Result);
        if (sizeof($Result) != 0)
        {
            foreach ($Result as $Line)
            {
                switch (strtok($Line, " \n\t"))
                {
                case "DATE":
                    $Info["time"]
                        = strtr(AllAfter(": ", $Line), ".", "/");
                    if ($TimeZone == "UTC"
                        and trim($Info["time"]) > "")
                        $Info["time"] .= " UTC";
                    break;
                }
            }
        }
    }
    return $Info;
}

function PhoneTimeStamp($FileName, $Folder)
{
    /*****************************************************************
    Cell-phone-specific: Parses phonetimestamps.txt file generated by
    scripts/local/phoneupload.sh. File has lines like this:

    image185.jpg 2005-11-13 17:52:46.000000000

    Returns timestamp, or null if phonetimestamps.txt doesn't exist or
    doesn't contain an entry for $FileName.

    Note: no $TimeZone argument; cell phone is always on local time
    *****************************************************************/
    $TimeStampFile = "$Folder/phonetimestamps.txt";
    $Time = Null;
    if (file_exists($TimeStampFile))
    {
        $CmdLine = "sed -ne 's/^$FileName //p' $TimeStampFile";
        $Cmd = exec($CmdLine, $Result);
        if (sizeof($Result) != 0) $Time = $Result[0];
    }
    return $Time;
}

?>
