<?php
/*********************************************************************
movepics.php
Move photos between sets Module for PhotoTrove
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require ("lib_inputval.php");
require_once ("lib_layout.php");
require ("lib_tag.php");
require ("lib_photoset.php");
require_once ("local/loc_camera.php");

$T = "A photo set is the most basic grouping for photos and videos.";
$T .= " Analogous to rolls of film, photo sets contain sequences of";
$T .= " images taken during a single time period or at a single";
$T .= " location. Photo sets can be fragmented: vigorous culling of";
$T .= " similar photos can leave a photo set all but empty; a trip";
$T .= " may start in the middle of one set and go on to span several";
$T .= " later ones; some cameras start a fresh folder after a set";
$T .= " number of shots or for every panoramic sequence. In such";
$T .= " cases, some housekeeping can help organize photo sets more";
$T .= " logically and efficiently.<br style=\"margin-bottom: 1em\"/>";
$T .= "Use the &quot;Tag selected photos&quot; feature on the Search";
$T .= " or Edit Photo Set pages to assign a particular tag to those";
$T .= " photos, and only those, that you want to move to a";
$T .= " particular target photo set. Then, on this page, select that";
$T .= " tag, specify the desired photo set, and move the photos.";
define("Introduction", $T);
define("IntroStyle", BoxStyle."; width: 50%; text-align: left");

function BoxSetSelect($TagSn, $Sort, $PhotoSetSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= StrTableBegin(0, "", "", BoxStyle);
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Select destination photo set:"));
    $S .= "<td align=\"right\">";
    $S .= MenuPhotoSet($PhotoSetSn, True);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= RowSearchSortOrder($Sort, True);
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function BoxTagSelect($TagSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SearchByTag", True);
    $S .= StrTableBegin("0", "", "", BoxStyle);
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Select tag:&nbsp;"));
    $S .= StrCell(MenuTag($TagSn, "TagSn", True, False));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function CompareTaggedAndSet($TagSn, $Sort, $PhotoSetSn, $NumRows)
{
    if (! is_numeric($PhotoSetSn))
    	$PhotoSetSn = NextPhotoSetSn(LastTaggedFolderName($TagSn));
    $S = BoxSetSelect($TagSn, $Sort, $PhotoSetSn);
    if (is_numeric($PhotoSetSn))
    {
	$E = ErrorNameInTargetSet($TagSn, $PhotoSetSn);
	if ($E > "") $S .= $E;
	else
	{
	    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
	    $NumTaggedInSet = CountTaggedInSet($TagSn, $PhotoSetSn);
	    if ($NumTaggedInSet > 0 and $NumTaggedInSet < $NumRows)
		$S .= WarnTaggedInSet($NumTaggedInSet, $TagName);
	    if ($NumTaggedInSet < $NumRows)
		$S .= FormMoveButton($TagSn, $PhotoSetSn, $Sort);
	    else $S .= FormUntag($TagSn, $TagName);
	}
	$S .= ThumbGallery($PhotoSetSn, $Sort, $TagSn);
    }
    return $S;
}

function CountTaggedInSet($TagSn, $PhotoSetSn)
{
    /* Return number of photos in $PhotoSetSn tagged with $TagSn */
    $Q = "SELECT COUNT(*) FROM tag_photo tp, photo p";
    $Q .= " WHERE tp.tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND p.photosetsn = $2";
    return pg_fetch_result(Query($Q, $TagSn, $PhotoSetSn), 0);
}

function ErrorDuplicateName($TagSn)
{
    /* Return an error bullet for any filename that occurs more than
     * once in the set of tagged photos */
    $Q = "SELECT DISTINCT filename FROM tag_photo tp, photo p";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn AND EXISTS";
    $Q .= "(SELECT 1 FROM tag_photo tp1, photo p1";
    $Q .= " WHERE tagsn = $1 AND p1.photosn = tp1.photosn";
    $Q .= " AND p1.photosn != p.photosn AND filename = p.filename)";
    $Res = Query($Q, $TagSn);
    $NumRows = pg_num_rows($Res);
    $E = "";
    if ($NumRows > 0)
    {
	for ($Row = 0 ; $Row < $NumRows ; $Row++)
	    $E .= "\n<li>".(pg_fetch_result($Res, $Row, 0))."</li>";
	$T = "Can't proceed; duplicate file names among tagged";
	$T .= " photos:\n";
	$E = StrWarningBox(Strong($T)."<ul>$E\n</ul>", "50%");;
    }
    return $E;
}

function ErrorNameInTargetSet($TagSn, $PhotoSetSn)
{
    /* Return an error bullet for any tagged photo not already in the
     * target photo set but with a filename that occurs in the target
     * photo set */
    $Q = "SELECT filename FROM tag_photo tp, photo p";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND photosetsn != $2 AND EXISTS";
    $Q .= "(SELECT 1 FROM photo WHERE photosetsn = $2";
    $Q .= " AND filename = p.filename)";
    $Res = Query($Q, $TagSn, $PhotoSetSn);
    $NumRows = pg_num_rows($Res);
    $E = "";
    if ($NumRows > 0)
    {
	for ($Row = 0 ; $Row < $NumRows ; $Row++)
	    $E .= "\n<li>".(pg_fetch_result($Res, $Row, 0))."</li>";
	$T = "Can't proceed; one or more tagged photos have file";
	$T .= " names that already exist in destination photo set:\n";
	$E = StrWarningBox(Strong($T)."<ul>$E\n</ul>", "50%");;
    }
    return $E;
}

function FormMoveButton($TagSn, $PhotoSetSn, $Sort)
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("TagSn", $TagSn);
    $T = "Move tagged photos to photo set &quot;$FolderName&quot;";
    $S .= HtmlSubmitButton("Move", $T);
    $S .= StrFormEnd();
    return $S;
}

function FormUntag($TagSn, $TagName)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= StrTableBegin(0, "", "", BoxStyle."; width:50%");
    $S .= StrRowBegin();
    $T = "Now that you've moved them, you may wish to remove the";
    $T .= " &quot;$TagName&quot; tag from all the photos.";
    $S .= StrCell($T);
    $S .= StrCell(HtmlSubmitButton("Untag", "Untag All"));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function HeadingPhotoCount($NumHits, $TagName, $Res, $TagSn)
{
    if ($NumHits > 0)
    {
	$First = pg_fetch_result($Res, 0, "timestamp");
	$Plural = ($NumHits > 1) ? "s" : "";
	$S = "$NumHits photo$Plural tagged &quot;$TagName&quot;";
	if ($NumHits > 1)
	{
	    $Last = pg_fetch_result($Res, $NumHits-1, "timestamp");
	    $S .= "<br />\nbetween $First and $Last:";
	} else
	{
	    $S .= " at $First:";
	}
    } else $S = "No photos tagged &quot;$TagName&quot;.";
    $S = StrNoticeBox($S);
    return $S;
}

function LastTaggedFolderName($TagSn)
{
    /* Return highest foldername among photo sets containing photos
     * tagged with $TagSn */
    $Q="SELECT DISTINCT(foldername)";
    $Q .= " FROM tag_photo tp, photo p, photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY foldername DESC";
    return pg_fetch_result(Query($Q, $TagSn), 0);
}

function MovedPics($TagSn, $PhotoSetSn, $Sort)
{
    $E = ErrorDuplicateName($TagSn);
    $E .= ErrorNameInTargetSet($TagSn, $PhotoSetSn);
    if ($E == "")
    {
	MoveImages($TagSn, $PhotoSetSn);
	MovePhotosInDb($TagSn, $PhotoSetSn);
    }
    return PageSelectSet($TagSn, $Sort, $PhotoSetSn);
}

function MoveImages($TagSn, $PhotoSetSn)
{
    $DestFolder
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    list($DestFulls, $DestWebPics, $DestThumbs)
	= ImageFolder($DestFolder);
    $Res = ResTaggedPicsToMove($TagSn);
    $NumRows = pg_num_rows($Res);
    for ($Row = 0 ; $Row < $NumRows ; $Row++)
    {
	list($PhotoSn, $SrcFolder, $FileName, $VidFileName)
	    = pg_fetch_array($Res, $Row);
	if ($SrcFolder != $DestFolder)
	{
	    list($SrcFulls, $SrcWebPics, $SrcThumbs)
		= ImageFolder($SrcFolder);
	    rename("$SrcFulls/$FileName", "$DestFulls/$FileName");
	    rename("$SrcWebPics/$FileName", "$DestWebPics/$FileName");
	    rename("$SrcThumbs/$FileName", "$DestThumbs/$FileName");
	    if ($VidFileName > "")
	    {
		rename("$SrcFulls/$VidFileName",
		       "$DestFulls/$VidFileName");
		$OggFile = OggFile($VidFileName);
		rename("$SrcWebPics/$OggFile",
		       "$DestWebPics/$OggFile");
	    }
	}
    }
    return;
}

function MovePhotosInDb($TagSn, $PhotoSetSn)
{
    $Q = "UPDATE photo p SET photosetsn = $2 WHERE EXISTS";
    $Q .= "(SELECT 1 FROM tag_photo tp";
    $Q .= " WHERE tagsn = $1 AND tp.photosn = p.photosn)";
    Query($Q, $TagSn, $PhotoSetSn);
    return;
}

function PageSelectSet($TagSn, $Sort, $PhotoSetSn)
{
    $Res = ResTaggedPhotos($TagSn);
    $NumRows = pg_num_rows($Res);
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $S = HeadingPhotoCount($NumRows, $TagName, $Res, $TagSn);
    if ($NumRows > 0)
    {
	$E = ErrorDuplicateName($TagSn);
	if ($E > "") $S .= $E;
	$S .= PhotoStats($Res, 0, True);
	if ($NumRows > 1)
	    $S .= PhotoStats($Res, $NumRows - 1, False);
	if ($E == "")
	    $S .= CompareTaggedAndSet($TagSn, $Sort, $PhotoSetSn,
				      $NumRows);
    }
    return $S;
}

function ResSetContents($PhotoSetSn, $Sort, $TagSn)
{
    /*****************************************************************
    Return attributes of all photos in set $PhotoSetSn, ordered by
    $Sort:
    ($PhotoSn, $FileName, $Description, $ThumbW, $ThumbH,
    $VidFileName, $IsTagged)
    *****************************************************************/
    $Q = "SELECT photosn, filename, description, thumbw, thumbh";
    $Q .= ", vidfilename, EXISTS ";
    $Q .= "(SELECT 1 FROM tag_photo WHERE photosn = p.photosn";
    $Q .= " AND tagsn = $2) AS istagged,";
    $Q .= " COALESCE(l.name, '".NullLocationFlag."') AS locname";
    $Q .= " FROM photoset s, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= " WHERE s.photosetsn = p.photosetsn";
    $Q .= " AND p.photosetsn = $1";
    $Q .= " ORDER BY ";
    $Q .= ($Sort == "Time") ? "time" : "locname, time";
    $Q .= ", lower(filename)";
    return Query($Q, $PhotoSetSn, $TagSn);
}

function ResTaggedPhotos($TagSn)
{
    $Q = " SELECT tp.photosn, foldername, filename, p.locationsn,";
    $Q .= " description, 1, 1, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay." AS timestamp";
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " v.vidfilename";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $TagSn);
}

function ResTaggedPicsToMove($TagSn)
{
    $Q = " SELECT tp.photosn, foldername, filename, v.vidfilename";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $TagSn);
}

function ThumbGallery($PhotoSetSn, $Sort, $TagSn)
{
    $FolderName
	= ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $Res = ResSetContents($PhotoSetSn, $Sort, $TagSn);
    $NumPics = pg_num_rows($Res);
    $T = "Photo set $FolderName currently contains these $NumPics";
    $T .= " photos/<span style=\"background-color: ".HighlightBg;
    $T .= "\">videos</span>:";
    $S = StrSubTitle($T);
    $Imgs = "";
    for ($Row = 0 ; $Row < $NumPics ; $Row++)
    {
	list($PhotoSn, $FileName, $Description, $ThumbW, $ThumbH,
	     $VidFileName, $IsTagged) = pg_fetch_array($Res, $Row);
	$T = $FileName;
	$T .= ($Description > "") ? (": $Description") : "";
	$I = "<img src=\"$ThumbFolder/$FileName\" height=\"$ThumbH\"";
	$I .= " width=\"$ThumbW\"";
	if ($IsTagged == "t")
	{
	    $T = "tagged $T";
	    $Color = WarningColor;
	} elseif ($VidFileName > "" )
	{
	    $Color =  HighlightBg;
	    $T .= " (Video)";
	} else $Color = BgColor;
	$T = HtmlCleanTitle($T);
	$I .= " style=\"border: thick solid $Color\" title=\"$T\" />";
	$Imgs .= $I;
    }
    return "$S<p style=\"text-align: left\">$Imgs</p>\n";
}

function UntaggedAll($TagSn)
{
    $Q = "DELETE FROM tag_photo WHERE tagsn = $1";
    Query($Q, $TagSn);
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $S = "Removed tag &quot;$TagName&quot; from all photos.";
    return StrWarning($S, NoticeColor);
}

function WarnTaggedInSet($NumTaggedInSet, $TagName)
{
    $S = "$NumTaggedInSet image";
    if ($NumTaggedInSet > 1) $S .= "s";
    $S .= " tagged &quot;$TagName&quot; ";
    $S .= ($NumTaggedInSet > 1) ? "are" : "is";
    $S .= " already in this photo set. Are you sure you mean this";
    $S .= " one?";
    $S = StrWarningBox($S, "50%");
    return $S;
}

//********************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "PhotoSetSn",
    "Sort",
    "TagSn"
    );
$GetVarList = array(
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (!isset($PhotoSetSn)) $PhotoSetSn = False;
if (!isset($Sort)) $Sort = "Time";
if (!isset($TagSn)) $TagSn = False;
$S = StartOfPage("Move Photos Between Sets");
if (isset($FormVars["Move"]))
    MovedPics($TagSn, $PhotoSetSn, $Sort);
elseif (isset($FormVars["Untag"]))
    $S .= UntaggedAll($TagSn);
elseif (isset($TagSn) and ! is_numeric($TagSn))
    $S .= StrWarning("Select a tag");
$S .= BoxTagSelect($TagSn);
if (is_numeric($TagSn))
    $S .= PageSelectSet($TagSn, $Sort, $PhotoSetSn);
else $S .= "<div style=\"".IntroStyle."\">".Introduction."</div>\n";
$S .= "</div></body></html>\n";
echo $S;

?>
