<?php
/*********************************************************************
showsetprogress.php
Create Photo Set Progress Display for PhotoTrove

______________________________________________________________________
Copyright 2024 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

Can be started independently with optional parameter:
	$FolderName	Photoset name
Included by importphotoset.php with parameters already set:
	$FolderName	Photoset name
	$StartTime	Time process started, for completion estimate
	$TimeZone	Camera's time zone
*********************************************************************/
if (! isset($Cmd)) // omit if coming straight from importphotoset.php
{
    require_once("lib_common.php");
    require("lib_filesys.php");
    require_once("lib_layout.php");
    require_once("lib_tag.php");
}
require ("lib_photoset.php");
require ("local/loc_camera.php");

function AddPicToDb($PhotoSetSn, $FileName, $FullFolder, $WebFolder,
                    $ThumbFolder, $TimeZone)
{
    global $PhotoMetaData;
    list($Width, $Height)
        = array_values(Geometry("$WebFolder/$FileName"));
    list($ThumbW, $ThumbH)
        = array_values(Geometry("$ThumbFolder/$FileName"));
    $Count = 6;
    $Params = array(
        $PhotoSetSn, $FileName, $Width, $Height, $ThumbW, $ThumbH);
    $C = "photosetsn, filename, width, height, thumbw, thumbh";
    $V = "$1, $2, $3, $4, $5, $6";
    $Info = ImageInfo($FullFolder, $FileName, $TimeZone);
    $Time = $Info["time"];
    if(trim($Time) > "")
    {
        $Count++;
        $Params[] = $Time;
        $C .= ", time";
        $V .= ", $$Count";
    }
    foreach ($PhotoMetaData as $Key)
    {
        if (array_key_exists($Key, $Info) && ($Info[$Key] != "")
            && ($Info[$Key] != "NULL"))
        {
            $Count++;
            $Params[] = $Info[$Key];
            $C .= ", $Key";
            $V .= ", $$Count";
        }
    }
    $Q = "INSERT INTO photo ($C) VALUES ($V)";
    QueryArray($Q, $Params);
    if ($Info["IsPanorama"])
        TagPhoto(TagSn("panorama"),
                 PhotoSerialNumber($FileName, $PhotoSetSn));
    return;
}

function AddSetToDb($FolderName, $TimeZone)
{
    // Analyzes photos and videos in a folder and makes database
    // entries. Returns numbers of photos and videos added.
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $Media = MediaInFolder($FullFolder);
    $Images = $Media["Image"];
    $Videos = $Media["Video"];
    Query("BEGIN");
    Query("INSERT INTO photoset (foldername) VALUES ($1)",
          $FolderName);
    $PhotoSetSn = SerialNumber("photoset", "foldername", $FolderName);
    if (is_array($Images))
        foreach ($Images as $FileName)
            AddPicToDb($PhotoSetSn, $FileName, $FullFolder,
                       $WebFolder, $ThumbFolder, $TimeZone);
    if (is_array($Videos))
        foreach ($Videos as $VidFileName)
            AddVidToDb($PhotoSetSn, $VidFileName, $FullFolder,
                       $TimeZone);
    Query("COMMIT");
    return array(is_null($Images) ? 0 : count($Images),
                 is_null($Videos) ? 0 : count($Videos));
}

function AddVidToDb($PhotoSetSn, $VidFileName, $FullFolder, $TimeZone)
{
    $StillFileName = VideoThumbnail($VidFileName, $PhotoSetSn);
    $StillSn = PhotoSerialNumber($StillFileName, $PhotoSetSn);
    list($Width, $Height)
        = array_values(VideoSpecs("$FullFolder/$VidFileName"));
    $Q = "INSERT INTO video (vidfilename, stillsn, width, height)";
    $Q .= " VALUES ($1, $2, $3, $4)";
    Query($Q, $VidFileName, $StillSn, $Width, $Height);
    // if still's timestamp is null, extract from video
    $Q = "SELECT 1 FROM photo WHERE photosn = $1 AND time IS NULL";
    if (pg_num_rows(Query($Q, $StillSn)) > 0)
    {
        list($TimeStamp, $WithZone)
            = VideoTimeStamp("$FullFolder/$VidFileName");
        if (strtotime($TimeStamp) !== InvalidDateTime)
        {
            if ($TimeZone == "UTC" and ! $WithZone)
                $TimeStamp .= " UTC";
            $Q = "UPDATE photo SET time = $2 WHERE photosn = $1";
            Query($Q, $StillSn, TimeString($TimeStamp));
        }
    }
    $GeoData = GPSFromFile("$FullFolder/$VidFileName");
    if ($GeoData !== False) UpdatePhotoGeoData($GeoData, $StillSn);
    return;
}

function ConversionComplete($WebFolder)
{
    return file_exists("$WebFolder/zzcomplete");
}

function DatabaseUpdated($FolderName)
{
    $Q = "SELECT * FROM photoset WHERE foldername = $1";
    return (pg_num_rows(Query($Q, $FolderName)) > 0); 
}

function FolderTotals($Directory)
{
    // Return counts and total sizes of images and videos in
    // $Directory
    $Media = MediaInFolder($Directory);
    $Images = array_key_exists("Image", $Media)
            ? $Media["Image"]
            : array();
    $Videos = array_key_exists("Video", $Media)
            ? $Media["Video"]
            : array();
    return array(
        count($Images), TotalSize($Directory, $Images),
        count($Videos), TotalSize($Directory, $Videos)
	);
}

function FormMenuFolder($Selected="", $AutoSubmit=True, $Prompt="")
{
    if ($Prompt == "")
        $Prompt = "&mdash; Select folder of photos &mdash;";
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SelectFolder", "1");
    $S .= MenuFolder($Selected, $AutoSubmit, $Prompt);
    $S .= StrFormEnd();
    return $S;
}

function MediaInFolder($Directory)
{
    /* Return array(Image, Video, Other, [ unreadable ]) where each
     * element is an array of filenames */
    $Media = array();
    $Dir = dir($Directory);
    $Finfo = finfo_open(FILEINFO_MIME_TYPE);
    while ($File = $Dir->read())
    {
        if (! Invisible($File))
        {
            $MediaType = MediaType("$Directory/$File", $Finfo);
            $Media[$MediaType][] = $File;
        }
    }
    finfo_close($Finfo);
    $Dir->close();
    return $Media;
}

function MenuFolder($Selected="", $AutoSubmit=True,
                    $Prompt="&mdash; Select folder of photos &mdash;")
{
    // Drop-down menu of folders under "fullsize/"
    $Dirs = Subdirectories("fullsize");
    rsort ($Dirs);
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin("FolderName", $Attribs);
    if ($Selected == "" or $Selected == "none")
        $S .= HtmlSelectOption("none", $Prompt, True);
    foreach ($Dirs as $Dir)
        $S .= HtmlSelectOption($Dir, $Dir, ($Dir == $Selected));
    $S .= HtmlSelectEnd();
    if ($AutoSubmit) $S = StrWithGoButton($S, "Go");
    return $S;
}

function PhotosInSet($FolderName)
{
    $Q = "SELECT COUNT(*) FROM photoset s, photo p";
    $Q .= " WHERE s.foldername = $1 AND p.photosetsn = s.photosetsn";
    return pg_fetch_result(Query($Q, $FolderName), 0, 0);
}

function TotalSize($Directory, $FileList)
{
    // $FileList is array($FileName) of filenames in $Directory
    // Returns total size in bytes of all specified files
    $Size = 0;
    if (is_array($FileList))
        foreach ($FileList as $File)
            $Size += filesize("$Directory/$File");
    return $Size;
}

function VideosInSet($FolderName)
{
    $Q = "SELECT COUNT(*) FROM photoset s, photo p, video v";
    $Q .= " WHERE s.foldername = $1 AND p.photosetsn = s.photosetsn";
    $Q .= " AND v.stillsn = p.photosn";
    return pg_fetch_result(Query($Q, $FolderName), 0, 0);
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "BytesAllPics",
    "BytesAllVids",
    "ConvStart",
    "CountDone",
    "EndTimeUnix",
    "FolderName",
    "NumImages",
    "NumVideos",
    "StartTime",
    "TimeZone"
);
// Auto-reload uses GET:
$GetVarList = array(
    "BytesAllPics",
    "BytesAllVids",
    "ConvStart",
    "CountDone",
    "EndTimeUnix",
    "FolderName",
    "NumImages",
    "NumVideos",
    "StartTime",
    "TimeZone"
);
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (!isset($FolderName) or $FolderName == "none")
{
    $S = HeadAndTitle("$SystemTitle Conversion Progress");
    $S .= PageTitleWithButton("Conversion Progress",
                              BtnMainMenu());
    $S .= "<div align=\"center\">\n";
    if (isset($FolderName) && $FolderName == "none")
        $S .= StrWarning("You must select a folder");
    $S .= FormMenuFolder();
    $S .= "</div></body></html>\n";
    echo $S;
    exit;
}

$Source = OriginalPhotos."/$FolderName";
list($FullFolder, $WebFolder, $ThumbFolder)
    = ImageFolder($FolderName) ;    
$LogFile = "$WebFolder/processlog";

/* Determine total number and size of images and videos to ingest, for
 * progress measure. If $Source gone, use $FullFolder contents. */
if (! isset($NumImages))
    list($NumImages, $BytesAllPics, $NumVideos, $BytesAllVids)
        = FolderTotals(file_exists($Source) ? $Source : $FullFolder);
$BytesAllPicsHuman = StrHumanNum($BytesAllPics, 3, True)."B";
$BytesAllVidsHuman = StrHumanNum($BytesAllVids, 3, True)."B";
$BytesAll = $BytesAllPics + $BytesAllVids;
$BytesAllHuman = StrHumanNum($BytesAll, 3, True)."B";

// Determine time process started, for time-to-complete estimate
if (!isset($StartTime)) $StartTime = filectime($FullFolder);

if (ConversionComplete($WebFolder))
{
    $ConvDoneTime = filectime("$WebFolder/zzcomplete");
    $S = HeadAndTitle("$SystemTitle Conversion Complete");
    $S .= PageTitleWithButton("Conversion Complete", BtnMainMenu());
    $S .= "<div align=\"left\">\n";
    $ConvTime = $ConvDoneTime - $StartTime;

    $T = "Imported ";
    if ($NumImages > 0)
        $T .= "$NumImages photos ($BytesAllPicsHuman) ";
    if ($NumVideos > 0)
    {
        if ($NumImages > 0) $T .= "and ";
        $T .= "$NumVideos videos ($BytesAllVidsHuman) ";
    }
    $T .= "to photoset $FolderName in ";
    $T .= HmsTime($ConvTime);
    $S .= HtmlPara($T);
    if (!DatabaseUpdated($FolderName))
    {
        // Make database entries, return how many
        list($DbImages, $DbVideos)
            = AddSetToDb($FolderName, $TimeZone);
        touch("$WebFolder/zzdbdone");
        $DBDoneTime = time();
    } else
    {
        $DbImages = PhotosInSet($FolderName);
        $DbVideos = VideosInSet($FolderName);
        $DBDoneTime = filectime("$WebFolder/zzdbdone");
    }

    $T = "Added ";
    if ($DbImages > 0)
        $T .= "$DbImages photos ";
    if ($DbVideos > 0)
    {
        if ($DbImages > 0) $T .= "and ";
        $T .= "$DbVideos videos ";
    }
    $T .= "to database for photo set";
    $T .= " &quot;$FolderName&quot; in ";
    $T .= HmsTime($DBDoneTime - $ConvDoneTime);
    $S .= HtmlPara($T);

    // Display log contents, if any
    if (filesize($LogFile) > 0)
    {
        $S .= StrBoxBegin("1", "left", "#ffffff");
        $S .= HtmlPara(Strong("Process Log:"));
        $S .= "<pre>\n";
        $S .= file_get_contents($LogFile);
        $S .= "</pre>\n";
        $S .= StrBoxEnd();
    } else $S .= HtmlPara("No log messages.");

    // Danger: do not delete completion indicator file
    /* 	UnlinkIfExists("$WebFolder/zzcomplete"); */
    /* UnlinkIfExists("$WebFolder/zzconvstart"); */

    $S .= StrTableBegin("0");
    $S .= StrRowBegin();
    $T = StrFormBegin("importphotoset.php");
    $T .= HtmlPara(HtmlSubmitButton("ImportNewSet",
                                    "Import a new set of originals"));
    $T .= StrFormEnd();
    $S .= StrCell($T);
    $T = StrFormBegin("editset.php");
    $PhotoSetSn = SerialNumber("photoset", "foldername", $FolderName);
    $T .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $T .= HtmlPara(HtmlSubmitButton("Edit", "Edit this photo set"));
    $T .= StrFormEnd();
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrTableEnd();

    $S .= StrWarning("Track progress of a different photoset");
    $S .= "</div><div align=\"center\">\n";
    $T = "&mdash; Select a different folder of photos &mdash;";
    $S .= FormMenuFolder("", True, $T);
} else
{
    list($PicCount, $PicSize) =
        explode(" ",
                trim(file_get_contents("$WebFolder/zzpicsdone")));
    $PicSizeHuman = StrHumanNum($PicSize, 3, True)."B";
    list($VidCount, $VidSize) =
        explode(" ",
                trim(file_get_contents("$WebFolder/zzvidsdone")));
    if (!isset($CountDone)) $CountDone = 0;
    $VidSizeHuman = StrHumanNum($VidSize, 3, True)."B";

    $RefreshDelay = 60;
    if ($PicCount + $VidCount > 0)
    {
        if (!isset($ConvStart) or $ConvStart == "")
            $ConvStart = filectime("$WebFolder/zzconvstart");
        // Videos take less time per MB to process than images
        $Progress = ($PicSize + VidToPicTimeRatio * $VidSize)
            / ($BytesAllPics + VidToPicTimeRatio * $BytesAllVids);
        if ($PicCount + $VidCount == $CountDone)
        {
            // Still working on same item, probably a large video;
            // keep same end time as before, adjust $SecsLeft
            $SecsLeft = $EndTimeUnix - time();
            // ...within reason
            if ($SecsLeft < $RefreshDelay/2)
            {
                $SecsLeft = $RefreshDelay/2;
                $EndTimeUnix = time() + $SecsLeft;
            }
        } else
        {
            $CountDone = $PicCount + $VidCount;
            $SecsLeft = (time() - $ConvStart) * ( 1 - $Progress )
                      / $Progress;
            $EndTimeUnix = time() + $SecsLeft;
        }
        $EndTime = date("H:i:s", $EndTimeUnix);
        if ($SecsLeft < $RefreshDelay)
            $RefreshDelay = floor($SecsLeft + 5);
    }
    // Page refreshes automatically every $RefreshDelay seconds until
    // completion test is satisfied
    $T = "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"$RefreshDelay;";
    $T .= " URL=showsetprogress.php?FolderName=$FolderName";
    $T .= "&NumImages=$NumImages&BytesAllPics=$BytesAllPics";
    $T .= "&NumVideos=$NumVideos&BytesAllVids=$BytesAllVids";
    $T .= "&StartTime=$StartTime&ConvStart=$ConvStart";
    $T .= "&CountDone=$CountDone&EndTimeUnix=$EndTimeUnix";
    $T .= "&TimeZone=$TimeZone\">";
    $S = HeadAndTitle("$SystemTitle Conversion Progress", $T);
    $S .= PageTitleWithButton("Conversion Progress", BtnMainMenu());
    $S .= "<div align=\"center\">\n";
    $T = "Importing ";
    if ($NumImages > 0)
        $T .= "$NumImages photos ($BytesAllPicsHuman) ";
    if ($NumVideos > 0)
    {
        if ($NumImages > 0) $T .= "and ";
        $T .= "$NumVideos videos ($BytesAllVidsHuman) ";
    }
    $T .= "to create photoset $FolderName";
    $S .= HtmlPara($T);
    if ($PicCount > 0)
    {
        if ($NumImages > 0)
        {
            $T = "Imported $PicCount photos ($PicSizeHuman) so far,";
            $T .= " out of $NumImages";
            $S .= HtmlPara($T);
        }
        if ($NumVideos > 0)
        {
            $T = "Imported $VidCount videos ($VidSizeHuman) so far,";
            $T .= " out of $NumVideos";
            $S .= HtmlPara($T);
        }
        $S .= HtmlPara("Estimated Time Left: ".HmsTime($SecsLeft));
        $S .= HtmlPara("Estimated Finish at: ".$EndTime);
        $S .= ProgressBar($Progress);
    } else
    {
        $NumItems = $NumImages + $NumVideos;
        $T = "Copying $NumItems originals ($BytesAllHuman) to";
        $T .= " repository.";
        $S .= HtmlPara($T);
        $S .= HtmlPara("Preparing to generate web images&hellip;");
        $S .= HtmlPara("Too early to estimate time left.");
    }
    $S .= HtmlPara("Checking again after $RefreshDelay seconds.");
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("BytesAllPics", $BytesAllPics);
    $S .= HtmlHidden("BytesAllVids", $BytesAllVids);
    $S .= HtmlHidden("ConvStart", $ConvStart);
    $S .= HtmlHidden("FolderName", $FolderName);
    $S .= HtmlHidden("NumImages", $NumImages);
    $S .= HtmlHidden("NumVideos", $NumVideos);
    $S .= HtmlHidden("StartTime", $StartTime);
    $S .= HtmlHidden("TimeZone", $TimeZone);
    $S .= HtmlPara(HtmlSubmitButton("Submit", "Refresh now"));
    $S .= StrFormEnd();
}

$S .= "</div></body></html>\n";
echo $S;

?>
