<?php
/*********************************************************************
stitchpano.php

Assist user desiring to assemble a panorama from component images
within a photo set.
______________________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");
require ("lib_filesys.php");
require ("lib_photoset.php");
require ("lib_tag.php");
require ("lib_graphicsmagick.php");
require_once ("local/loc_camera.php");

$T = "Specify a file to upload, no larger than ".UploadMaxFileSize;
$T .= " bytes and without embedded spaces or special characters in";
$T .= " its name.";
define ("FileUploadPrompt", $T);

define ("TagNameComponent", "component");
define ("TagNamePano", "panorama");
define ("TagNamePanoToDo", "panorama-to-do");

$T = "A panorama is an assembly of multiple overlapping images to";
$T .= " create a single image with an extra-large field of view.";
$T .= " Typically, you create a panorama with these steps:\n";
$T .= "<ol>\n";
$T .= "<li>Record the component images, rotating the camera in";
$T .= " place to each position, ideally using a tripod. Many";
$T .= " cameras offer features to help align successive images with";
$T .= " consistent focus and exposure.</li>\n";
$T .= "<li>Identify the panorama components within the photo set;";
$T .= " they may be segregated from the rest in a separate folder";
$T .= " (Nikon) or with distinctive filenames (Canon).</li>\n";
$T .= "<li>Assign each component the tag";
$T .= " &quot;".TagNamePanoToDo."&quot; to keep track as they";
$T .= " accumulate.</li>\n";
$T .= "<li class=\"hi\">Download the tagged components from the";
$T .= " PhotoTrove server to your personal computer.</li>\n";
$T .= "<li>Using a tool like";
$T .= " <a href=\"http://hugin.sourceforge.net/\"";
$T .= " target=\"_blank\">Hugin";
$T .= " (http://hugin.sourceforge.net/)</a>, stitch the components";
$T .= " together into a single continuous view.</li>\n";
$T .= "<li>Using a photo editor like ";
$T .= "<a href=\"http://www.gimp.org/\" target=\"_blank\">Gimp";
$T .= " (http://www.gimp.org/)</a>, crop, adjust color levels and";
$T .= " adjust as necessary.</li>\n";
$T .= "<li class=\"hi\">Upload the finished panorama to the photo";
$T .= " set, scaling the webpic to the default height and referring";
$T .= " to the last component in the series for the location,";
$T .= " description and timestamp.</li>\n";
$T .= "<li class=\"hi\">Assign each finished panorama the tag";
$T .= " &quot;panorama&quot; for future retrieval and replace the";
$T .= " &quot;".TagNamePanoToDo."&quot; tag with the";
$T .= " &quot;".TagNameComponent."&quot; tag for each of its";
$T .= " components.</li>\n";
$T .= "</ol>\n";
$T .= "This tool can help with the";
$T .= " <span class=\"hi\">highlighted</span> steps.";
define("Introduction", $T);

$T = BoxStyle."; width: 62%; text-align: left; margin-bottom: 2mm";
define("IntroStyle", $T);

define("SortDefault", "Time");

$T = "<style type=\"text/css\">.hi { background: ".HighlightBg;
$T .= " }</style>";
define ("Styles", $T);

function BoxSelectSetsPanoToDo($PhotoSetSn, $Sort)
{
    $Res  = ResPhotoSetsPanoToDo();
    if (pg_num_rows($Res) > 0)
    {
        $S = StrFormBeginRecurse();
        $S .= HtmlHidden("SelectPhotoSet", True);
        $S .= StrTableBegin(0, "", "", BoxStyle);
        $S .= RowSelectPhotoSets($Res, $PhotoSetSn);
        $S .= RowSearchSortOrder($Sort, True);
        $S .= StrTableEnd();
        $S .= StrFormEnd();
    } else
    {
        $S = StrTableBegin(0, "", "", BoxStyle);
        $S .= "<tr><td>\n";
        $S .= "No folder contains photos tagged &quot;";
        $S .= TagNamePanoToDo."&quot;.";
        $S .= StrBoxEnd();
    }
    return $S;
}

function ComponentCell($Res, $Row, $NumRows, $Selected, &$IsPano)
{
    // Cell with thumbnail image and active checkbox
    $S = "";
    if ($Row % 4 == 0)
    {
        if ($Row > 0) $S .= StrRowEnd();
        $S .= StrRowBegin();
    }
    list ($PhotoSn, $FolderName, $FileName, $Description, $ThumbW,
          $ThumbH, $Time, $LocName, $IsPano, $IsComponent)
        = pg_fetch_array($Res, $Row);
    $IsPano = ($IsPano == 't');
    $IsComponent = ($IsComponent == 't');
    list($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    if ($IsPano) $T = "";
    else
    {
        $T = "<label>".HtmlCheckbox("Sel$Row", $Selected);
        $T .= HtmlHidden("PS$Row", $PhotoSn);
    }
    $T .= "$FileName:<br />\n";
    $Path = "$ThumbFolder/$FileName";
    $T .= HtmlImage(LatestFile($Path),
                    "$FileName $Time ($LocName) $Description",
                    $ThumbH, $ThumbW, 1);
    if (! $IsPano) $T .= "</label>";
    $Style = ($IsComponent) ? "background: ".HighlightBg : "";
    $S .= StrCell($T, "", "", $Style);
    return $S;
}

function ComponentCells($Res, $NumRows, &$RangeMarks)
{
    /*****************************************************************
    Display thumbnails of all potential panorama components in photo
    set, in rows of four.
    If $RangeMarks is False, clear all thumbnail selection checkboxes.
    If it's True (and Boolean, rather than an array), set them all.

    Otherwise, $RangeMarks is an array of elements (Photosn), each of
    which starts or ends a range. If a range has been started but no
    marker ends it, it will extend to the last image on the page.
    *****************************************************************/
    $IsByRange = is_array($RangeMarks);
    if ($IsByRange)
    {
        $NumRanges = ceil(count($RangeMarks)/2);
        $Selected = False;
        $NextMark = array_shift($RangeMarks);
        $SwitchAfter = False;
        $NumSelected = 0;
    } else $Selected = $RangeMarks;
    $S = StrTableBegin("0");
    for ($Row=0 ; $Row < $NumRows ; $Row++ )
    {
        if ($IsByRange)
        {
            $PhotoSn = pg_fetch_result($Res, $Row, "photosn");
            if ($PhotoSn == $NextMark)
            {
                if ($Selected) $SwitchAfter = True;
                else $Selected = True;
                $NextMark = array_shift($RangeMarks);
            }
        }
        $S .= ComponentCell($Res, $Row, $NumRows, $Selected, $IsPano);
        if ($Selected and ! $IsPano) $NumSelected++;
        if ($SwitchAfter) $Selected = False;
        $SwitchAfter = False;
    }
    for ($I=0 ; $I < (3 - (($NumRows -1) % 4)) ; $I++)
        $S .= StrCell("&nbsp;");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    if ($IsByRange)
    {
        // Use the $RangeMarks argument to return statistics
        $Plural = ($NumRanges > 1) ? "s" : "";
        $T = "Selected $NumSelected photos in $NumRanges specified";
        $T .= " range$Plural.";
        $RangeMarks = $T;
    }
    return $S;
}

function ComponentThumbnails($Components)
{
    // Display thumbnails of the selected series of panorama
    // components
    $Thumbs = "";
    foreach ($Components as $PhotoSn)
    {
        $Res = ResComponent($PhotoSn);
        list($FolderName, $FileName, $Description, $ThumbW, $ThumbH,
             $Time, $LocName, $IsComponent) = pg_fetch_array($Res, 0);
        list ($FullFolder, $WebFolder, $ThumbFolder)
            = ImageFolder($FolderName);
        $IsTagged = ($IsComponent == 't');
        $T = ($LocName > "") ? " ($LocName)" : "";
        $T .= ($Description > "") ? " $Description" : "";
        if ($T > "") $T = ":$T";
        $T = $FileName.$T;
        $T = HtmlCleanTitle($T);
        $I = "<img src=\"$ThumbFolder/$FileName\" height=\"$ThumbH\"";
        $I .= " width=\"$ThumbW\"";
        $Style = ($IsTagged) ? "background: ".HighlightBg : "";
        if ($IsTagged)
            $I .= " style=\"border: thick solid ".HighlightBg."\"";
        $I .= " title=\"$T\" />";
        $Thumbs .= $I;
    }
    return "<p style=\"text-align: left\">$Thumbs</p>\n";
}

function CountTagged($PhotoSetSn, $TagName)
{
    /* Return number of photos in $PhotoSetSn tagged with $TagName */
    $Q = "SELECT COUNT(*) FROM tag t, tag_photo tp, photo p";
    $Q .= " WHERE t.name = $2 AND tp.tagsn = t.tagsn";
    $Q .= " AND p.photosn = tp.photosn AND p.photosetsn = $1";
    return pg_fetch_result(Query($Q, $PhotoSetSn, $TagName), 0);
}

function DownloadComponents($State, $SerNums)
{
    /* Assemble components into a zip file and download it. */
    list ($PhotoSetSn, $Sort) = UnPackedState($State);
    $FolderName = ColFromSerialNumber("photoset", "foldername",
                                      $PhotoSetSn);
    list($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
    if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
    $Components = explode(",", $SerNums);
    $ZipFile = LoggedInUser()."-".PanoName($Components, False).".zip";
    $Args = "";
    foreach ($Components as $PhotoSn)
    {
        $Args .= " ".ColFromSerialNumber("photo", "filename",
                                         $PhotoSn);
    }
    chdir($FullFolder);
    shell_exec("zip ../.uploads/$ZipFile $Args");
    DownloadFile("../.uploads/$ZipFile");
    unlink("../.uploads/$ZipFile");
    return;
}

function FormSelectComponents($PhotoSetSn, $Sort, $SelectAll,
			      &$RangeMarks)
{
    $State = PackedState(array($PhotoSetSn, $Sort));
    $Res = ResPossibleComponents($PhotoSetSn, $Sort);
    $NumRows = pg_num_rows($Res);
    $Count = CountTagged($PhotoSetSn, TagNamePanoToDo);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("State", htmlentities($State));
    /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
    $S .= HtmlHidden("NumRows", $NumRows);
    $S .= StrSelectionOptions();
    $Width = 4 * ThumbnailLength;
    $S .= "<p style=\"width: $Width; text-align: left\">";
    if ($Count > 0)
    {
        $S .= "Select panorama components, presumably from the";
        $S .= " <span class=\"hi\">$Count highlighted images</span>";
    } else
    {
        $S .= "You can select panorama components but note that";
        $S .= " this folder has no photos";
    }
    $S .= " tagged as &quot;".TagNamePanoToDo."&quot;.";
    $S .= " When all components selected, click: ";
    $S .= HtmlSubmitButton("ComponentsSelected", "Continue");
    $S .= "</p>\n";
    if ($SelectAll) $RangeMarks = True;
    $S .= ComponentCells($Res, $NumRows, $RangeMarks);
    $S .= StrFormEnd();
    return $S;
}

function FormUploadPanorama($PhotoSetSn, $Sort, $Components)
{
    $T = ParsedHumanNum(UploadMaxFileSize, True);
    $UploadSizeLimit = ceil($T[1]);
    $S = StrFormBeginPostUpload(ThisScript(), UploadMaxFileBytes);
    $S .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("SerNums", implode(",", $Components));
    $S .= "<div style=\"".BoxStyle;
    $S .= "; display: table; text-align: center\">\n";
    $S .= "<p style=\"margin-top: 0\">";
    $S .= "Select image (max ".UploadMaxFileSize." bytes)";
    $S .= " to add to photo set:<br />\n";
    $S .= HtmlInputPostUpload("UserFile", 40);
    $S .= "</p>\n<p style=\"margin-bottom: 0\">";
    $S .= HtmlSubmitButton("Upload", "Upload Panorama");
    $S .= "</p></div>\n";
    $S .= StrFormEnd();
    return $S;
}

function MenuPhotoSets($Res, $Selection="", $AutoSubmit=False,
		       $Prompt="&mdash; Select photo set &mdash;")
{
    /* Drop-down menu of photosets in $Res */
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin("PhotoSetSn", $Attribs);
    if ($Selection == "" or $Selection == "none")
        $S .= HtmlSelectOption("none", $Prompt, True);
    for ($Row = 0 ; $Row < pg_num_rows($Res) ; $Row ++)
    {
        list($PhotoSetSn, $FolderName, $Nickname)
            = pg_fetch_array($Res, $Row);
        $S .= HtmlSelectOption($PhotoSetSn, "$FolderName  $Nickname",
                               ($PhotoSetSn == $Selection));
    }
    $S .= HtmlSelectEnd();
//     if ($AutoSubmit) $S = StrWithGoButton($S, "Go");
    return $S;
}

function OptionComponentsDone()
{
    // Return to main panorama page with selections cleared
    $S = StrRowBegin();
    $S .= StrCell("Return to the main panorama page.");
    $S .= StrCell(HtmlSubmitButton("Done", " Done "), "", "",
                  "text-align: right");
    $S .= StrRowEnd();
    return $S;
}

function OptionDownloadComponents()
{
    // Download selected components as a single zip archive
    $S = StrRowBegin();
    $T = "Download them immediately to assemble the panorama, as";
    $T .= " a single zip archive.";
    $S .= StrCell($T);
    $S .= StrCell(HtmlSubmitButton("Download", " Download "), "", "",
                  "text-align: right");
    $S .= StrRowEnd();
    return $S;
}

function PageActions($State, $NumRows, $FormVars)
{
     /* Series of panorama components selected: tag and offer
	component download or finished panorama upload */
    $Components = array();
    for ($Row=0 ; $Row < $NumRows ; $Row++ )
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
            $Components[] = $FormVars["PS$Row"];
    $Count = count($Components);
    if ($Count < 2)
    {
        $S = StrWarning("Specify at least 2 components.",
                        WarningColor);
        $S .= SelectAllOrNone($State, False);
    } else
    {
        TagComponents($Components, TagNamePanoToDo);
        $T = "Either: Download and assemble panorama components:";
        $S .= StrSubTitle($T);
        $S .= PanoActionOptions($State, $Components, $Count);
        $T = "Or: Upload the finished panorama:";
        $S .= StrSubTitle($T);
        list ($PhotoSetSn, $Sort)
            = UnPackedState($State);
	    /* = UnPackedState(stripslashes($State)); */
        $S .= FormUploadPanorama($PhotoSetSn, $Sort, $Components);
        $S .= ComponentThumbnails($Components);
    }
    return $S;
}

function PageSelectComponents($PhotoSetSn, $Sort, $Warning="",
                              $SelectAll=False, &$RangeMarks=False)
{
    /* Main panorama page offering selection of panorama components */
    $S = "";
    if ( $Warning !== "" ) $S .= StrWarning($Warning, WarningColor);
    $S .= BoxSelectSetsPanoToDo($PhotoSetSn, $Sort);
    $T = "Select components to assemble a panorama:";
    $S .= StrSubTitle($T);
    $S .= FormSelectComponents($PhotoSetSn, $Sort, $SelectAll,
                               $RangeMarks);
    return $S;
}

function PanoActionOptions($State, $Components, $Count)
{
    $PanoName = PanoName($Components);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("State", htmlentities($State));
    /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
    $S .= HtmlHidden("SerNums", implode(",", $Components));
    $S .= StrTableBegin("0", "", "", BoxStyle."; width: 50%");
    $S .= StrRowBegin();
    $T = "You have selected these $Count images as components";
    $T .= " of a panorama. All now bear the tag";
    $T .= " &quot;".TagNamePanoToDo."&quot;.";
    $T .= " A good name for the final panorama might be:";
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrHeaderCell($PanoName, "2");
    $S .= StrRowEnd();
    $S .= OptionDownloadComponents();
    $S .= OptionComponentsDone();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function PanoMetaData($Components)
{
    // location, description, timestamp (+ 1sec) from last component
    $NumComponents = count($Components);
    $NumNames = ["zero", "one", "two", "three", "four", "five",
                 "six", "seven", "eight", "nine", "ten", "eleven",
                 "twelve", "thirteen", "fourteen", "fifteen"];
    $NumName = $NumNames[$NumComponents];
    $Addenddum = ", panorama from $NumName preceding photos";
    $PhotoSn = $Components[$NumComponents -1];
    $Q = "SELECT locationsn, description || '$Addenddum',";
    $Q .= " time + INTERVAL '1 second'";
    $Q .= "FROM photo WHERE photosn = $1";
    return pg_fetch_array(Query($Q, $PhotoSn), 0);
}

function PanoName($Components, $AppendExtension=True)
{
    $Count = count($Components);
    $First = pathinfo(ColFromSerialNumber("photo", "filename",
                                          $Components[0]));
    $FirstName = $First["filename"];
    $Last = pathinfo(ColFromSerialNumber("photo", "filename",
                                         $Components[$Count - 1]));
    $LastName = $Last["filename"];
    $Suffix = $AppendExtension ? ".".$Last["extension"] : "";
    return $FirstName."-".$LastName.$Suffix;
}

function ResComponent($PhotoSn)
{
    // Details of a specific photo selected as a panorama component
    $Q = "SELECT foldername, filename, description, thumbw,";
    $Q .= " thumbh, time, l.name AS locname, EXISTS";
    $Q .= " (SELECT 1 FROM tag t, tag_photo tp";
    $Q .= " WHERE name = '".TagNamePanoToDo;
    $Q .= "' AND tp.tagsn = t.tagsn";
    $Q .= " AND photosn = p.photosn) AS iscomponent";
    $Q .= " FROM photoset s, photo p LEFT OUTER JOIN location l";
    $Q .= " USING (locationsn)";
    $Q .= " WHERE p.photosn = $1 AND s.photosetsn = p.photosetsn";
    return Query($Q, $PhotoSn);
}

function ResPhotoSetsPanoToDo()
{
    /*****************************************************************
    Return ($PhotoSetSn, $FolderName, $Nickname) of all photo sets
    containing photos tagged "panorama-to-do", ordered by foldername
    *****************************************************************/
    $Q = "SELECT DISTINCT ON (foldername) s.photosetsn, foldername,";
    $Q .= " nickname";
    $Q .= " FROM tag t, tag_photo tp, photo p, photoset s";
    $Q .= " WHERE t.name = '".TagNamePanoToDo."'";
    $Q .= " AND tp.tagsn = t.tagsn AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY foldername DESC";
    return Query($Q);
}

function ResPossibleComponents($PhotoSetSn, $Sort)
{
    // Images in $PhotoSetSn not currently assigned as stills for
    // videos; flag if tagged as "panorama" or as "panorama-to-do"
    $Q = "SELECT photosn, foldername, filename, description, thumbw,";
    $Q .= " thumbh, time, l.name AS locname, EXISTS";
    $Q .= " (SELECT 1 FROM tag t, tag_photo tp";
    $Q .= " WHERE name = '".TagNamePano."' AND tp.tagsn = t.tagsn";
    $Q .= " AND photosn = p.photosn) AS ispano, EXISTS";
    $Q .= " (SELECT 1 FROM tag t, tag_photo tp";
    $Q .= " WHERE name = '".TagNamePanoToDo;
    $Q .= "' AND tp.tagsn = t.tagsn";
    $Q .= " AND photosn = p.photosn) AS iscomponent";
    $Q .= " FROM photoset s, photo p LEFT OUTER JOIN location l";
    $Q .= " USING (locationsn)";
    $Q .= " WHERE s.photosetsn = $1 AND p.photosetsn = s.photosetsn";
    $Q .= " AND NOT EXISTS";
    $Q .= " (SELECT 1 FROM video WHERE stillsn = p.photosn)";
    $Q .= " ORDER BY ";
    $Q .= ($Sort == "Time") ? "time" : "locname, time";
    $Q .= ", lower(filename)";
    return Query($Q, $PhotoSetSn);
}

function RowSelectPhotoSets($Res, $PhotoSetSn)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("Select photo set:"));
    $S .= "<td align=\"right\">";
    $S .= MenuPhotoSets($Res, $PhotoSetSn, True);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function SelectAllOrNone($State, $SelectAll=True)
{
    /*****************************************************************
    Sets or clears selection checkboxes on all images according to
    $SelectAll, then redisplays photo set page.
    *****************************************************************/
    /* $State = stripslashes($State); */
    list ($PhotoSetSn, $Sort) = UnPackedState($State);
    return ($PhotoSetSn > "" and $PhotoSetSn != "none")
        ? PageSelectComponents($PhotoSetSn, $Sort, "", $SelectAll)
        : "";
}

function SelectRange($State, $NumThumbs, $FormVars)
{
    /*****************************************************************
    If $NumThumbs > 0, builds an ordered array of selected thumbnails
    which specify starting and stopping points of ranges, then calls
    PageSelectComponents to redisplay photo set page with checkboxes
    set for all images in range(s).
    *****************************************************************/
    $RangeMarks = array();
    for ($Row=0 ; $Row < $NumThumbs ; $Row++ )
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
            $RangeMarks[] = $FormVars["PS$Row"];
    if (count($RangeMarks) > 0)
    {
        /* $State = stripslashes($State); */
        list ($PhotoSetSn, $Sort) = UnPackedState($State);
        $T = PageSelectComponents($PhotoSetSn, $Sort, "", False,
                                  $RangeMarks);
        $S = HtmlPara($RangeMarks).$T;
    } else
    {
        $S = HtmlPara("No selection range specified.");
        $S .= SelectAllOrNone($State, False);
    }
    return $S;
}

function TagComponents($Components, $TagName)
{
    $TagSn = TagSn($TagName);
    foreach ($Components as $PhotoSn)
    {
        TagPhoto($TagSn, $PhotoSn);
    }
    return;
}

function UnTagComponents($Components, $TagName)
{
    $TagSn = TagSn($TagName);
    foreach ($Components as $PhotoSn)
    {
        UnTagPhoto($TagSn, $PhotoSn);
    }
    return;
}

function UpdatePano($PhotoSn, $LocationSn, $TimeStamp, $Description,
                    $WebSize, $ThumbSize)
{
    $Description = trim($Description);
    $Params = array($PhotoSn, $WebSize["width"], $WebSize["height"],
                    $ThumbSize["width"], $ThumbSize["height"],
                    $LocationSn, $Description);
    $Q = "UPDATE photo SET width = $2, height = $3, thumbw = $4,";
    $Q .= " thumbh = $5, locationsn = $6, description = $7, time = ";
    if ($TimeStamp == "") $Q .= "NULL";
    else
    {
        $Params[] = $TimeStamp;
        $Q .= "$8";
    }
    $Q .= " WHERE photosn = $1";
    QueryArray($Q, $Params);
    return;
}

function UploadPano($PhotoSetSn, $Sort, $SerNums)
{
    // Add pano if upload ok, overwriting any existing one
    $Components = explode(",", $SerNums);
    if ($_FILES["UserFile"]["error"] != UPLOAD_ERR_OK)
    {
        $T = "Upload failed.<br />".FileUploadPrompt;
        $S = StrWarning($T);
        $S .= FormUploadPanorama($PhotoSetSn, $Sort, $Components);
    } else
    {
        list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
        if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
        $NewFile = "$FullUploads/".LoggedInUser();
        if (move_uploaded_file($_FILES["UserFile"]["tmp_name"],
                               $NewFile) and filesize($NewFile) > 0)
        {
            list($Width, $Height) = array_values(Geometry($NewFile));
            $WebPicLength = ScaledWebPicLength($Width, $Height);
            $FileName = $_FILES["UserFile"]["name"];
            $FolderName
                = ColFromSerialNumber("photoset", "foldername",
                                      $PhotoSetSn);
            list($LocationSn, $Description, $TimeStamp)
                = PanoMetaData($Components);
            if (!$LocationSn) $LocationSn = "NULL";
            // Add physical images, return final sizes
            list($WebSize, $ThumbSize)
                = AddImages($FolderName, $FileName, $WebPicLength,
                            ThumbnailLength);
            $PhotoSn = PhotoSerialNumber($FileName, $PhotoSetSn);
            if ($PhotoSn)
                UpdatePano($PhotoSn, $LocationSn, $TimeStamp,
                           $Description, $WebSize, $ThumbSize);
            else
                $PhotoSn =
                    AddPhoto($PhotoSetSn, $FileName, $LocationSn,
                             $TimeStamp, $Description, $WebSize,
                             $ThumbSize, "UTC");
            TagPhoto(TagSn(TagNamePano), $PhotoSn);
            TagComponents($Components, TagNameComponent);
            UnTagComponents($Components, TagNamePanoToDo);
            $S = StrFormBegin("viewset.php", "", "_blank");
            $S .= HtmlHidden("PhotoSn", $PhotoSn);
            $S .= HtmlHidden("ViewLarge", 1);
            $S .= "<div style=\"".BoxStyle;
            $S .= "; width: 55%; text-align: left\">\n";
            $S .= "Added panorama ";
            $S .= HtmlSubmitButton("ViewPano", "$FileName");
            $S .= " to photo set $FolderName, tagged it &quot;";
            $S .= TagNamePano."&quot; and removed tag &quot;";
            $S .= TagNamePanoToDo."&quot; from its components.";
            $S .= "</div>\n";
            $S .= StrFormEnd();
            $S .= PageSelectComponents($PhotoSetSn, $Sort);
        }
        else
        {
            UnlinkIfExists($NewFile);
            $T = "Problem handling file. Upload failed.<br />";
            $S = StrWarning($T.FileUploadPrompt);
            $S .= FormUploadPanorama($PhotoSetSn, $Sort, $Components);
        }
    }
    return $S;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "Description",
    "FileName",
    "FolderName",
    "LocationSn",
    "NumRows",
    "OtherLoc",
    "ParentPhotoSn",
    "PhotoSetSn",
    "PicExists",
    "SelectAll",
    "SerNums",
    "Sort",
    "State",
    "ThumbnailLength",
    "TimeText",
    "VidExists",
    "WebPicLength"
    );
$PostVarList = array(
//    "FolderName",
    "MAX_FILE_SIZE",
    "PhotoSetSn",
    "SerNums",
    "Sort",
    "State",
    "TimeZone",
    "UserFile"
   );

foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);
if (isset($FormVars["Download"]))
    DownloadComponents($State, $SerNums);
else
{
    $S = StartOfPage("Assemble a Panorama from Component Images",
                     Styles);
    
    if (! isset($PhotoSetSn) and isset($State))
        list ($PhotoSetSn, $Sort) = UnPackedState($State);
    if (!isset($Sort)) $Sort = SortDefault;
    if (!isset($PhotoSetSn) or $PhotoSetSn == "none")
    {
        $S .= "<div style=\"".IntroStyle."\">".Introduction;
        $S .= "</div>\n";
        $S .= BoxSelectSetsPanoToDo("", $Sort);
    }
    elseif (isset($SelectAll) or isset($FormVars["ClearAll"]))
        $S .= SelectAllOrNone($State, isset($SelectAll));
    elseif (isset($FormVars["SelectRange"]))
        $S .= SelectRange($State, $NumRows, $FormVars);
    elseif (isset($FormVars["ComponentsSelected"]))
        $S .= PageActions($State, $NumRows, $FormVars);
    elseif (isset($_POST["Upload"]))
        $S .= UploadPano($PhotoSetSn, $Sort, $SerNums);
    else
        $S .= PageSelectComponents($PhotoSetSn, $Sort, "");
    $S .= "</div></body></html>\n";
    echo $S;
}

?>
